/***************************************************************************
                           kplayerpart.cpp
                           ---------------
    begin                : Wed Aug 20 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kapplication.h>
#include <ktoolbar.h>
#include <kxmlguifactory.h>
#include <kparts/genericfactory.h>
#include <qpopupmenu.h>
#include <dcopclient.h>

#include "kplayerpart.h"
#include "kplayerpart.moc"
#include "kplayersettings.h"
#include "kplayerslideraction.h"

static const char* description = I18N_NOOP("KPlayerPart, an embeddable KDE media player based on MPlayer");

typedef KParts::GenericFactory<KPlayerPart> KPlayerPartFactory;
K_EXPORT_COMPONENT_FACTORY (libkplayerpart, KPlayerPartFactory);

KPlayerPart::KPlayerPart (QWidget* wparent, const char* wname, QObject* parent, const char* name, const QStringList&)
  : KParts::ReadOnlyPart (parent, name ? name : "kplayerpart")
{
  m_updating = false;
  m_toolbar_visible [0] = true;
  m_toolbar_visible [1] = false;
  m_toolbar_names << "progressToolBar" << "volumeToolBar";
  m_toolbar_actions << "settings_progress_toolbar" << "settings_volume_toolbar";
  KPlayerSettings::setKPlayerConfig (new KConfig ("kplayerrc"));
  KPlayerSettings::createKPlayerSettings();
  setInstance (KPlayerPartFactory::instance());
  m_workspace = new KPlayerWorkspace (wparent, wname ? wname : "kplayerworkspace");
  m_workspace -> setFreeResizeMode (false);
  connect (kPlayerProcess(), SIGNAL (stateChanged(KPlayerProcess::State)), this, SLOT (playerStateChanged(KPlayerProcess::State)));
  connect (kPlayerProcess(), SIGNAL (progressChanged(float, KPlayerProcess::ProgressType)), this, SLOT (playerProgressChanged(float, KPlayerProcess::ProgressType)));
  connect (kPlayerProcess(), SIGNAL (infoAvailable()), this, SLOT (playerInfoAvailable()));
  connect (kPlayerWorkspace(), SIGNAL (contextMenu(QContextMenuEvent*)), this, SLOT (widgetContextMenu(QContextMenuEvent*)));
  connect (kPlayerSettings(), SIGNAL (saved()), this, SLOT (refreshSettings()));
  setWidget (m_workspace);
  initActions();
  setXMLFile ("kplayerpartui.rc");
  readOptions();
  KToolBar* toolbar = toolBar (PROGRESS_TOOLBAR);
  if ( toolbar )
  {
    toolbar -> setCaption (i18n("Progress"));
    sliderAction (m_toolbar_actions [0]) -> slider() -> setOrientation (toolbar -> orientation());
  }
  toolbar = toolBar (VOLUME_TOOLBAR);
  if ( toolbar )
  {
    toolbar -> setCaption (i18n("Volume"));
    sliderAction (m_toolbar_actions [1]) -> slider() -> setOrientation (toolbar -> orientation());
  }
  enablePlayerActions();
  kdDebug() << "Constructor OK\n";
}

KPlayerPart::~KPlayerPart()
{
  closeURL();
  kPlayerWidget() -> close (true);
  KPlayerSettings::deleteKPlayerSettings();
  KPlayerSettings::deleteKPlayerConfig();
}

KAboutData* KPlayerPart::createAboutData (void)
{
  KAboutData* aboutData = new KAboutData ("kplayer", "KPlayerPart",
    VERSION, description, KAboutData::License_GPL,
    "(C) 2002-2003, kiriuja", 0, 0, "kplayer-dev@en-directo.net");
  aboutData -> addAuthor ("kiriuja", 0, "kplayer-dev@en-directo.net");
  return aboutData;
}

bool KPlayerPart::openURL (const KURL& url)
{
  kdDebug() << "Open URL\n";
  emit setWindowCaption (url.prettyURL());
  loadUrl (url);
  return true;
}

bool KPlayerPart::closeURL (void)
{
  kdDebug() << "Close URL\n";
  saveOptions();
  enableScreenSaver();
  kPlayerProcess() -> kill();
  return true;
}

KToolBar* KPlayerPart::toolBar (int index)
{
  index -= PROGRESS_TOOLBAR;
/*QObject* toolbar = sliderAction (m_toolbar_actions [index]) -> slider() -> parent();
  kdDebug() << "Name: '" << toolbar -> name() << "'\n";
  if ( toolbar && toolbar -> inherits ("KToolBar") )
    kdDebug() << "Got parent\n";
  if ( toolbar )
    kdDebug() << "KPlayerSlider parent class name: " << toolbar -> className() << "\n";
  if ( toolbar && toolbar -> inherits ("KToolBar") )
    return (KToolBar*) toolbar;*/
  /*if ( qApp )
    kdDebug() << "App OK\n";
  if ( qApp && qApp -> mainWidget() )
    kdDebug() << "Main widget OK\n";*/
  if ( ! qApp || ! qApp -> mainWidget() )
    return 0;
  /*if ( qApp -> mainWidget() -> child (m_toolbar_names [index]) )
    kdDebug() << "Child OK\n";
  if ( qApp -> mainWidget() -> child (m_toolbar_names [index], "KToolBar") )
    kdDebug() << "KToolBar OK\n";*/
  return (KToolBar*) qApp -> mainWidget() -> child (m_toolbar_names [index], "KToolBar");
}

void KPlayerPart::initActions (void)
{
  KAction* action = new KAction (i18n("&Play"), "player_play", Key_Return, this, SLOT (playerPlay()), actionCollection(), "player_play");
  action -> setStatusText (i18n("Plays the currently loaded file"));
  action = new KToggleAction (i18n("Paus&e"), "player_pause", Key_Space, this, SLOT (playerPause()), actionCollection(), "player_pause");
  action -> setStatusText (i18n("Pauses the player"));
  action = new KAction (i18n("&Stop"), "player_stop", Key_Escape, this, SLOT (playerStop()), actionCollection(), "player_stop");
  action -> setStatusText (i18n("Stops the player"));
  action = new KAction (i18n("&Forward"), 0, Key_Right, this, SLOT (playerForward()), actionCollection(), "player_forward");
  action -> setStatusText (i18n("Moves forward by one percent of video length"));
  action = new KAction (i18n("F&ast Forward"), "player_fwd", CTRL + Key_Right, this, SLOT (playerFastForward()), actionCollection(), "player_fast_forward");
  action -> setStatusText (i18n("Moves forward by ten percent of video length"));
  action = new KAction (i18n("&Backward"), 0, Key_Left, this, SLOT (playerBackward()), actionCollection(), "player_backward");
  action -> setStatusText (i18n("Moves backward by one percent of video length"));
  action = new KAction (i18n("Fast Back&ward"), "player_rew", CTRL + Key_Left, this, SLOT (playerFastBackward()), actionCollection(), "player_fast_backward");
  action -> setStatusText (i18n("Moves backward by ten percent of video length"));
  action = new KAction (i18n("Back &to Start"), 0, CTRL + Key_Home, this, SLOT (playerStart()), actionCollection(), "player_start");
  action -> setStatusText (i18n("Moves to the beginning"));
  action = new KAction (i18n("Volume &Up"), 0, CTRL + Key_Up, this, SLOT (playerVolumeUp()), actionCollection(), "player_volume_up");
  action -> setStatusText (i18n("Increases the volume"));
  action = new KAction (i18n("Volume &Down"), 0, CTRL + Key_Down, this, SLOT (playerVolumeDown()), actionCollection(), "player_volume_down");
  action -> setStatusText (i18n("Decreases the volume"));
  action = new KToggleAction (i18n("&Mute"), "mute", CTRL + Key_Backslash, this, SLOT (playerMute()), actionCollection(), "player_mute");
  action -> setStatusText (i18n("Turns the sound on/off"));
  action = new KToggleAction (i18n("Maintain Original &Aspect"), "viewmagfit", CTRL + Key_A, this, SLOT (viewMaintainOriginalAspect()), actionCollection(), "view_original_aspect");
  action -> setStatusText (i18n("Maintains the original video aspect ratio"));
  action = new KToggleAction (i18n("Show &Progress Toolbar"), 0, 0, this, SLOT (viewProgressToolbar()), actionCollection(), "settings_progress_toolbar");
  action -> setStatusText (i18n("Shows/hides the progress toolbar"));
  action = new KToggleAction (i18n("Show &Volume Toolbar"), "volume", 0, this, SLOT (viewVolumeToolbar()), actionCollection(), "settings_volume_toolbar");
  action -> setStatusText (i18n("Shows/hides the volume toolbar"));
  m_updating = true;
  KPlayerSliderAction* sa = new KPlayerSliderAction (i18n("Progress"), 0, this, SLOT (progressChanged (int)), actionCollection(), "player_progress");
  sa -> slider() -> setup (0, 0, 0, 0, 0);
  sa -> setStatusText (i18n("Shows player progress and allows seeking"));
  sa = new KPlayerSliderAction (i18n("Volume"), 0, this, SLOT (volumeChanged (int)), actionCollection(), "player_volume");
  sa -> setStatusText (i18n("Changes volume level"));
  KPlayerPopupSliderAction* psa = new KPlayerPopupSliderAction (i18n("Volume"), "volume", Key_F9, this, SLOT (volumeChanged (int)), actionCollection(), "popup_volume");
  psa -> setStatusText (i18n("Shows the volume popup slider"));
  m_updating = false;
}

void KPlayerPart::refreshSettings (void)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  int value = kPlayerProcess() -> volume();
  sliderAction ("player_volume") -> slider() -> setup (settings -> volumeMinimum(),
    settings -> volumeMaximum(), value, settings -> volumeMarks());
  popupAction ("popup_volume") -> slider() -> setup (settings -> volumeMinimum(),
    settings -> volumeMaximum(), value, settings -> volumeMarks());
  setVolume (value);
  if ( kPlayerProcess() -> length() > 0 )
    sliderAction ("player_progress") -> slider() ->
      setPageStep (int (kPlayerProcess() -> length() * settings -> progressMarks() + 0.5));
  m_updating = false;
}

void KPlayerPart::saveOptions (void)
{
  kdDebug() << "Saving options\n";
  KConfig* config = kPlayerConfig();
  config -> setGroup ("General Options");
  config -> writeEntry (QString (m_toolbar_names [0]) + " Part Visible", m_toolbar_visible [0]);
  config -> writeEntry (QString (m_toolbar_names [1]) + " Part Visible", m_toolbar_visible [1]);
  config -> setGroup ("Player Options");
  config -> writeEntry ("Maintain Aspect", kPlayerWidget() -> originalAspect());
  config -> writeEntry ("Volume", kPlayerProcess() -> volume());
  config -> writeEntry ("Mute", toggleAction ("player_mute") -> isChecked());
}

void KPlayerPart::readOptions (void)
{
  kdDebug() << "Reading options\n";
  KPlayerSettings* settings = kPlayerSettings();
  KConfig* config = kPlayerConfig();
  config -> setGroup ("General Options");
  config -> writeEntry (QString (m_toolbar_names [0]) + " Part Visible", m_toolbar_visible [0]);
  m_toolbar_visible [0] = config -> readBoolEntry (QString (m_toolbar_names [0]) + " Part Visible", m_toolbar_visible [0]);
  toggleAction (m_toolbar_actions [0]) -> setChecked (m_toolbar_visible [0]);
  if ( toolBar (PROGRESS_TOOLBAR) )
    kdDebug() << "Toolbar OK\n";
  KToolBar* toolbar = toolBar (PROGRESS_TOOLBAR);
  if ( toolbar )
  {
    if ( m_toolbar_visible [0] )
      toolbar -> show();
    else
      toolbar -> hide();
  }
  m_toolbar_visible [1] = config -> readBoolEntry (QString (m_toolbar_names [1]) + " Part Visible", m_toolbar_visible [1]);
  toggleAction (m_toolbar_actions [1]) -> setChecked (m_toolbar_visible [1]);
  toolbar = toolBar (VOLUME_TOOLBAR);
  if ( toolbar )
  {
    if ( m_toolbar_visible [1] )
      toolbar -> show();
    else
      toolbar -> hide();
  }
  config -> setGroup ("Player Options");
  kPlayerWidget() -> maintainOriginalAspect (config -> readBoolEntry ("Maintain Aspect", false));
  kPlayerWorkspace() -> setVideoSize();
  if ( kPlayerWidget() -> originalAspect() )
    toggleAction ("view_original_aspect") -> setChecked (true);
  kPlayerProcess() -> setFrameDropping ((KPlayerProcess::FrameDropping) config -> readNumEntry ("Frame Dropping", KPlayerProcess::None));
  if ( settings -> volumeReset() )
    setVolume (settings -> volumeDefault());
  else
  {
    setVolume (config -> readNumEntry ("Volume", settings -> volumeDefault()));
    if ( config -> readBoolEntry ("Mute", false) )
      toggleAction ("player_mute") -> activate();
  }
  if ( settings -> contrastReset() )
    setContrast (settings -> contrastDefault());
  else
    setContrast (config -> readNumEntry ("Contrast", settings -> contrastDefault()));
  if ( settings -> brightnessReset() )
    setBrightness (settings -> brightnessDefault());
  else
    setBrightness (config -> readNumEntry ("Brightness", settings -> brightnessDefault()));
  if ( settings -> hueReset() )
    setHue (settings -> hueDefault());
  else
    setHue (config -> readNumEntry ("Hue", settings -> hueDefault()));
  if ( settings -> saturationReset() )
    setSaturation (settings -> saturationDefault());
  else
    setSaturation (config -> readNumEntry ("Saturation", settings -> saturationDefault()));
  refreshSettings();
}

void KPlayerPart::enableToolbar (int index)
{
  index -= PROGRESS_TOOLBAR;
  KToggleAction* action = toggleAction (m_toolbar_actions [index]);
  action -> setEnabled (true);
  if ( m_toolbar_visible [index] )
  {
    action -> setChecked (true);
    KToolBar* toolbar = toolBar (index + PROGRESS_TOOLBAR);
    if ( toolbar )
      toolbar -> show();
  }
}

void KPlayerPart::disableToolbar (int index)
{
  index -= PROGRESS_TOOLBAR;
  KToggleAction* action = toggleAction (m_toolbar_actions [index]);
  if ( action -> isChecked() )
  {
    action -> setChecked (false);
    KToolBar* toolbar = toolBar (index + PROGRESS_TOOLBAR);
    if ( toolbar )
      toolbar -> hide();
  }
  action -> setEnabled (false);
}

void KPlayerPart::enablePlayerActions (void)
{
  kdDebug() << "Enable Player Actions\n";
  KPlayerProcess::State state = kPlayerProcess() -> state();
  action ("player_play") -> setEnabled (! kPlayerProcess() -> url().isEmpty()
    && state != KPlayerProcess::Running && state != KPlayerProcess::Playing);
  action ("player_pause") -> setEnabled (state != KPlayerProcess::Idle);
  bool busy = state != KPlayerProcess::Idle;
  if ( ! busy )
  {
    QMouseEvent me (QEvent::MouseButtonRelease, QPoint(0,0), QPoint(0,0), QMouseEvent::LeftButton, QMouseEvent::NoButton);
    QApplication::sendEvent (sliderAction ("player_progress") -> slider(), &me);
  }
  sliderAction ("player_progress") -> slider() -> setEnabled (busy);
  action ("player_stop") -> setEnabled (busy);
  action ("player_forward") -> setEnabled (busy);
  action ("player_fast_forward") -> setEnabled (busy);
  action ("player_backward") -> setEnabled (busy);
  action ("player_fast_backward") -> setEnabled (busy);
  action ("player_start") -> setEnabled (busy);
  kdDebug() << "Enable Toolbar\n";
  if ( kPlayerProcess() -> length() != 0 )
    enableToolbar (PROGRESS_TOOLBAR);
  else
    disableToolbar (PROGRESS_TOOLBAR);
  bool unpaused = state != KPlayerProcess::Paused;
  action ("player_volume_up") -> setEnabled (unpaused);
  action ("player_volume_down") -> setEnabled (unpaused);
  action ("player_mute") -> setEnabled (unpaused);
  action ("popup_volume") -> setEnabled (unpaused);
  sliderAction ("player_volume") -> slider() -> setEnabled (unpaused);
  bool video = ! kPlayerProcess() -> size().isEmpty();
  action ("view_original_aspect") -> setEnabled (video);
}

void KPlayerPart::widgetContextMenu (QContextMenuEvent* event)
{
  kdDebug() << "KPlayerPart context menu\n";
  QPopupMenu* popup = (QPopupMenu*) factory() -> container ("player_popup", this);
  popup -> popup (event -> globalPos());
  event -> accept();
}

void KPlayerPart::viewMaintainOriginalAspect (void)
{
  kPlayerWidget() -> maintainOriginalAspect (toggleAction ("view_original_aspect") -> isChecked());
  kPlayerWorkspace() -> setVideoSize();
}

void KPlayerPart::loadUrl (KURL url, bool play)
{
  kdDebug() << "URL  '" << url.url() << "'\n";
  kdDebug() << "     '" << url.prettyURL (0, KURL::StripFileProtocol) << "'\n";
  if ( url.path().isEmpty() && url.host().isEmpty() )
    return;
  KPlayerSettings* settings = kPlayerSettings();
  play = play && settings -> startPlayingImmediately();
  if ( url == kPlayerProcess() -> url() )
  {
    if ( play )
      playerPlay();
    return;
  }
  kPlayerProcess() -> kill();
  kPlayerProcess() -> setUrl (url);
  if ( settings -> volumeReset() && settings -> volumeEvery() == 0 )
  {
    setVolume (settings -> volumeDefault());
    KToggleAction* action = toggleAction ("player_mute");
    if ( action -> isChecked() )
      action -> activate();
  }
  if ( settings -> contrastReset() && settings -> contrastEvery() == 0 )
    setContrast (settings -> contrastDefault());
  if ( settings -> brightnessReset() && settings -> brightnessEvery() == 0 )
    setBrightness (settings -> brightnessDefault());
  if ( settings -> hueReset() && settings -> hueEvery() == 0 )
    setHue (settings -> hueDefault());
  if ( settings -> saturationReset() && settings -> saturationEvery() == 0 )
    setSaturation (settings -> saturationDefault());
  if ( kPlayerProcess() -> get_info() && play )
    playerProcessPlay();
}

void KPlayerPart::playerPlay (void)
{
  if ( kPlayerProcess() -> state() == KPlayerProcess::Paused )
    kPlayerProcess() -> pause();
  else if ( kPlayerProcess() -> state() == KPlayerProcess::Idle )
    playerProcessPlay();
}

void KPlayerPart::playerProcessPlay (void)
{
  kPlayerProcess() -> play();
}

void KPlayerPart::playerPause (void)
{
  if ( kPlayerProcess() -> state() != KPlayerProcess::Idle )
    kPlayerProcess() -> pause();
  else
  {
    KToggleAction* action = toggleAction ("player_pause");
    if ( action -> isChecked() )
      action -> setChecked (false);
  }
}

void KPlayerPart::playerStop (void)
{
  kPlayerProcess() -> stop();
}

void KPlayerPart::playerForward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressNormalSeekUnits() ? settings -> progressNormalSeek() :
    kPlayerProcess() -> length() == 0 ? 10 :
    int (kPlayerProcess() -> length() * settings -> progressNormalSeek() / 100 + 0.5);
  if ( seek == 0 )
    seek = 1;
  kPlayerProcess() -> relativeSeek (seek);
}

void KPlayerPart::playerFastForward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressFastSeekUnits() ? settings -> progressFastSeek() :
    kPlayerProcess() -> length() == 0 ? 60 :
    int (kPlayerProcess() -> length() * settings -> progressFastSeek() / 100 + 0.5);
  if ( seek == 0 )
    seek = 2;
  kPlayerProcess() -> relativeSeek (seek);
}

void KPlayerPart::playerBackward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressNormalSeekUnits() ? settings -> progressNormalSeek() :
    kPlayerProcess() -> length() == 0 ? 10 :
    int (kPlayerProcess() -> length() * settings -> progressNormalSeek() / 100 + 0.5);
  if ( seek == 0 )
    seek = 1;
  kPlayerProcess() -> relativeSeek (- seek);
}

void KPlayerPart::playerFastBackward (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  int seek = settings -> progressFastSeekUnits() ? settings -> progressFastSeek() :
    kPlayerProcess() -> length() == 0 ? 60 :
    int (kPlayerProcess() -> length() * settings -> progressFastSeek() / 100 + 0.5);
  if ( seek == 0 )
    seek = 2;
  kPlayerProcess() -> relativeSeek (- seek);
}

void KPlayerPart::playerStart (void)
{
  kPlayerProcess() -> absoluteSeek (0);
}

void KPlayerPart::setVolume (int volume)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  volume = limit (volume, settings -> volumeMinimum(), settings -> volumeMaximum());
  if ( volume != kPlayerProcess() -> volume() )
    kPlayerProcess() -> setVolume (volume);
  KPlayerSlider* slider = sliderAction ("player_volume") -> slider();
  if ( volume != slider -> value() )
    slider -> setValue (volume);
  slider = popupAction ("popup_volume") -> slider();
  if ( volume != slider -> value() )
    slider -> setValue (volume);
  m_updating = false;
}

void KPlayerPart::setContrast (int contrast)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  contrast = limit (contrast, settings -> contrastMinimum(), settings -> contrastMaximum());
  if ( contrast != kPlayerProcess() -> contrast() )
    kPlayerProcess() -> setContrast (contrast);
  m_updating = false;
}

void KPlayerPart::setBrightness (int brightness)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  brightness = limit (brightness, settings -> brightnessMinimum(), settings -> brightnessMaximum());
  if ( brightness != kPlayerProcess() -> brightness() )
    kPlayerProcess() -> setBrightness (brightness);
  m_updating = false;
}

void KPlayerPart::setHue (int hue)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  hue = limit (hue, settings -> hueMinimum(), settings -> hueMaximum());
  if ( hue != kPlayerProcess() -> hue() )
    kPlayerProcess() -> setHue (hue);
  m_updating = false;
}

void KPlayerPart::setSaturation (int saturation)
{
  m_updating = true;
  KPlayerSettings* settings = kPlayerSettings();
  saturation = limit (saturation, settings -> saturationMinimum(), settings -> saturationMaximum());
  if ( saturation != kPlayerProcess() -> saturation() )
    kPlayerProcess() -> setSaturation (saturation);
  m_updating = false;
}

void KPlayerPart::playerVolumeUp (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> volume() < settings -> volumeMaximum() )
    setVolume (kPlayerProcess() -> volume() + settings -> volumeStep());
}

void KPlayerPart::playerVolumeDown (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( kPlayerProcess() -> volume() > settings -> volumeMinimum() )
    setVolume (kPlayerProcess() -> volume() - settings -> volumeStep());
}

void KPlayerPart::playerMute (void)
{
  kPlayerProcess() -> mute();
}

void KPlayerPart::viewProgressToolbar (void)
{
  KToolBar* toolbar = toolBar (PROGRESS_TOOLBAR);
  if ( ! toolbar )
    return;
  bool visible = toggleAction ("settings_progress_toolbar") -> isChecked();
  m_toolbar_visible [0] = visible;
  if ( visible )
    toolbar -> show();
  else
    toolbar -> hide();
}

void KPlayerPart::viewVolumeToolbar (void)
{
  KToolBar* toolbar = toolBar (VOLUME_TOOLBAR);
  if ( ! toolbar )
    return;
  bool visible = toggleAction ("settings_volume_toolbar") -> isChecked();
  m_toolbar_visible [1] = visible;
  if ( visible )
    toolbar -> show();
  else
    toolbar -> hide();
}

void KPlayerPart::progressChanged (int progress)
{
  if ( m_updating || ! m_progress_factor )
    return;
  kdDebug() << "Seek to " << progress << ": " << kPlayerProcess() -> position() << " => " << ((progress + m_progress_factor / 2) / m_progress_factor) << " / " << kPlayerProcess() -> length() << "\n";
  kPlayerProcess() -> absoluteSeek ((progress + m_progress_factor / 2) / m_progress_factor);
}

void KPlayerPart::volumeChanged (int volume)
{
  if ( ! m_updating )
    setVolume (volume);
}

void KPlayerPart::playerStateChanged (KPlayerProcess::State state)
{
  static const QString stateMessages [4] = { i18n("Idle"), i18n("Running"), i18n("Playing"), i18n("Paused") };
  kdDebug() << "State change received: " << state << "\n";
  KToggleAction* toggle = toggleAction ("player_pause");
  if ( state != KPlayerProcess::Paused && toggle -> isChecked() )
    toggle -> setChecked (false);
  if ( state == KPlayerProcess::Idle )
    playerProgressChanged (0, KPlayerProcess::Position);
  enablePlayerActions();
  if ( state == KPlayerProcess::Playing )
    disableScreenSaver();
  else
    enableScreenSaver();
}

void KPlayerPart::disableScreenSaver (void)
{
  if ( ! kPlayerSettings() -> disableScreenSaver() || kPlayerProcess() -> size().isEmpty() )
    return;
  QByteArray data, reply;
  QCString type;
  if ( ! kapp -> dcopClient() -> call ("kdesktop", "KScreensaverIface", "isEnabled()", data, type, reply)
      || type != "bool" )
  {
    kdWarning() << "SS: Could not get screen saver status\n";
    return;
  }
  QDataStream replyStream (reply, IO_ReadOnly);
  bool enabled = false;
  replyStream >> enabled;
  if ( ! enabled )
  {
    kdDebug() << "SS: Screen saver is not enabled\n";
    return;
  }
  QDataStream argStream (data, IO_WriteOnly);
  argStream << false;
  if ( kapp -> dcopClient() -> send ("kdesktop", "KScreensaverIface", "enable(bool)", data) )
    kdDebug() << "SS: Screensaver successfully disabled\n";
  else
    kdWarning() << "SS: Could not disable screensaver\n";
  m_enable_screen_saver = true;
}

void KPlayerPart::enableScreenSaver (void)
{
  if ( ! m_enable_screen_saver )
    return;
  QByteArray data;
  QDataStream argStream (data, IO_WriteOnly);
  argStream << true;
  if ( kapp -> dcopClient() -> send ("kdesktop", "KScreensaverIface", "enable(bool)", data) )
    kdDebug() << "SS: Screensaver successfully enabled\n";
  else
    kdWarning() << "SS: Could not enable screensaver\n";
  m_enable_screen_saver = false;
}

void KPlayerPart::playerProgressChanged (float progress, KPlayerProcess::ProgressType type)
{
  static float prev_progress = -1;
  if ( type != KPlayerProcess::Position )
    return;
  float diff = progress - prev_progress;
  prev_progress = progress;
  if ( diff <= 0 && diff > -0.15 && progress > 0 )
    return;
  m_updating = true;
  KPlayerSlider* slider = sliderAction ("player_progress") -> slider();
  int maxValue = slider -> maxValue();
  if ( maxValue )
  {
    int value = int (progress * m_progress_factor + 0.5);
    if ( value > maxValue )
      slider -> setMaxValue (value);
    slider -> setValue (value);
  }
  m_updating = false;
}

void KPlayerPart::playerInfoAvailable (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  kdDebug() << "Info available. Detected length: " << kPlayerProcess() -> length() << "\n";
  KPlayerSlider* slider = sliderAction ("player_progress") -> slider();
  // QRangeControl breaks if its range is more than 524287
  if ( kPlayerProcess() -> length() > 50000 )
    m_progress_factor = 1;
  else if ( kPlayerProcess() -> length() > 5000 )
    m_progress_factor = 10;
  else
    m_progress_factor = 100;
  slider -> setMaxValue (int (kPlayerProcess() -> length() * m_progress_factor + 0.5));
  slider -> setPageStep (slider -> maxValue() * settings -> progressMarks() / 100);
  enablePlayerActions();
  kPlayerWorkspace() -> setVideoSize();
}
