/***************************************************************************
                           kplayerpart.h
                           -------------
    begin                : Wed Aug 20 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYERPART_H
#define KPLAYERPART_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kaction.h>
#include <kparts/part.h>

#include "kplayerprocess.h"
#include "kplayerwidget.h"

class KPlayerPopupSliderAction;
class KPlayerSliderAction;

#define PROGRESS_TOOLBAR    1
#define VOLUME_TOOLBAR      2

/**
   The KPlayer KPart class.
  *@author kiriuja
  */
class KPlayerPart : public KParts::ReadOnlyPart
{
  Q_OBJECT

public:
  KPlayerPart (QWidget*, const char*, QObject*, const char*, const QStringList&);
  virtual ~KPlayerPart();

  /** Returns the data for the About dialog.
   **/
  static KAboutData* createAboutData (void);

  virtual bool openURL (const KURL&);

  /** Loads the given URL and optionally starts playback.
    */
  void loadUrl (KURL, bool play = true);

  /** Starts playback.
    */
  void playerProcessPlay (void);

  /** Disables screen saver if it is enabled.
   */
  void disableScreenSaver (void);
  /** Enables screen saver if it was disabled.
   */
  void enableScreenSaver (void);

  /** Enables or disables player actions.
    */
  void enablePlayerActions (void);

  /** Enables a toolbar.
   * @param index the toolbar number
   */
  void enableToolbar (int index);
  /** Disables a toolbar.
   * @param index the toolbar number
   */
  void disableToolbar (int index);

  /** Sets the sound volume.
    */
  void setVolume (int);
  /** Sets the video contrast.
    */
  void setContrast (int);
  /** Sets the video brightness.
    */
  void setBrightness (int);
  /** Sets the video hue.
    */
  void setHue (int);
  /** Sets the video saturation.
    */
  void setSaturation (int);

  /** Retrieves an action from the actionCollection by name.
    */
  KAction* action (const char* name) const
    { return actionCollection() -> action (name); }
  /** Retrieves a slider action from the actionCollection by name.
    */
  KPlayerSliderAction* sliderAction (const char* name) const
    { return (KPlayerSliderAction*) action (name); }
  /** Retrieves a slider action from the actionCollection by name.
    */
  KPlayerPopupSliderAction* popupAction (const char* name) const
    { return (KPlayerPopupSliderAction*) action (name); }
  /** Retrieves a toggle action from the actionCollection by name.
    */
  KToggleAction* toggleAction (const char* name) const
    { return (KToggleAction*) action (name); }

  /** Returns the KPlayerProcess object.
    */
  KPlayerProcess* kPlayerProcess (void) const
    { return kPlayerWidget() -> kPlayerProcess(); }
  /** Returns the KPlayerWidget object.
    */
  KPlayerWidget* kPlayerWidget (void) const
    { return kPlayerWorkspace() -> kPlayerWidget(); }
  /** Returns the KPlayerWorkspace object.
    */
  KPlayerWorkspace* kPlayerWorkspace (void) const
    { return m_workspace; }

  /** Returns a pointer to the toolbar with the given index.
   */
  KToolBar* toolBar (int);

public slots:
  /** Stops the KPlayerProcess and closes the KPlayerWidget.
    */
  virtual bool closeURL (void);

  /** Plays the currently loaded file.
    */
  void playerPlay (void);
  /** Pauses the player.
    */
  void playerPause (void);
  /** Stops the player.
    */
  void playerStop (void);
  /** Seeks forward.
    */
  void playerForward (void);
  /** Seeks forward fast.
    */
  void playerFastForward (void);
  /** Seeks backward.
    */
  void playerBackward (void);
  /** Seeks backward fast.
    */
  void playerFastBackward (void);
  /** Seeks to the beginning.
    */
  void playerStart (void);
  /** Increases the volume.
    */
  void playerVolumeUp (void);
  /** Decreases the volume.
    */
  void playerVolumeDown (void);
  /** Toggles audio playback.
    */
  void playerMute (void);
  /** Toggles the option of maintaining the original video aspect ratio.
    */
  void viewMaintainOriginalAspect (void);
  /** Toggles the progress toolbar.
    */
  void viewProgressToolbar (void);
  /** Toggles the volume toolbar.
    */
  void viewVolumeToolbar (void);

  /** Moves to the position chosen.
    */
  void progressChanged (int);
  /** Changes the volume level.
    */
  void volumeChanged (int);

  /** Receives the stateChanged signal from KPlayerProcess.
    */
  void playerStateChanged (KPlayerProcess::State);
  /** Receives the progressChanged signal from KPlayerProcess.
    */
  void playerProgressChanged (float, KPlayerProcess::ProgressType);
  /** Receives the infoAvailable signal from KPlayerProcess.
    */
  void playerInfoAvailable (void);
  /** Receives the saved signal from KPlayerSettings. Updates the slider settings.
   */
  void refreshSettings (void);
  /** Displays the right click popup menu.
    */
  void widgetContextMenu (QContextMenuEvent*);

protected:
  /** Initializes the KActions of the KPart.
    */
  void initActions (void);
  /** Dummy openFile required because of KParts::ReadOnlyPart design flaw.
    */
  virtual bool openFile (void)
    { return false; }

  /** Saves options to the configuration file.
   */
  void saveOptions (void);
  /** Reads options from the configuration file.
    */
  void readOptions (void);

  // Following should really be private
  bool m_updating;
  bool m_enable_screen_saver;
  int m_progress_factor; // work around QRangeControl bug
  KPlayerWorkspace* m_workspace; // widget() non-const in KParts::Part
  QStringList m_toolbar_names;
  QStringList m_toolbar_actions;
  bool m_toolbar_visible [2];
};

#endif
