/***************************************************************************
                          kplayerplaylist.cpp
                          --------------------
    begin                : Wed Sep 3 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <kdebug.h>
#include <klineedit.h>
#include <klocale.h>
#include <krandomsequence.h>
#include <kurl.h>
#include <kurldrag.h>
#include <qdockarea.h>
#include <qfileinfo.h>
#include <qtooltip.h>

#include "kplayerplaylist.h"
#include "kplayerplaylist.moc"
#include "kplayersettings.h"

KPlayerPlaylistItem::KPlayerPlaylistItem (const KURL& url)
{
  m_url = url;
  m_name = url.fileName().isEmpty() ? url.prettyURL()
    : QFileInfo (url.fileName()).baseName (true);
  m_length = m_video_bitrate = m_audio_bitrate = 0;
}

void KPlayerPlaylistItem::load (void)
{
  kPlayerPlaylistConfig() -> setGroup (url().url());
  setName (kPlayerPlaylistConfig() -> readEntry ("Name"));
  setLength (kPlayerPlaylistConfig() -> readDoubleNumEntry ("Length"));
  setVideoSize (kPlayerPlaylistConfig() -> readSizeEntry ("Video Size"));
  setVideoCodec (kPlayerPlaylistConfig() -> readEntry ("Video Codec"));
  setVideoBitrate (kPlayerPlaylistConfig() -> readNumEntry ("Video Bitrate"));
  setAudioCodec (kPlayerPlaylistConfig() -> readEntry ("Audio Codec"));
  setAudioBitrate (kPlayerPlaylistConfig() -> readNumEntry ("Audio Bitrate"));
}

void KPlayerPlaylistItem::setName (const QString& name)
{
  if ( ! name.isEmpty() )
    m_name = name;
}

KPlayerPlaylistItem::~KPlayerPlaylistItem()
{
  kPlayerPlaylistConfig() -> setGroup (url().url());
  kPlayerPlaylistConfig() -> writeEntry ("Name", name());
  kPlayerPlaylistConfig() -> writeEntry ("Length", length());
  kPlayerPlaylistConfig() -> writeEntry ("Video Size", videoSize());
  kPlayerPlaylistConfig() -> writeEntry ("Video Codec", videoCodec());
  kPlayerPlaylistConfig() -> writeEntry ("Video Bitrate", videoBitrate());
  kPlayerPlaylistConfig() -> writeEntry ("Audio Codec", audioCodec());
  kPlayerPlaylistConfig() -> writeEntry ("Audio Bitrate", audioBitrate());
  kPlayerPlaylistConfig() -> writeEntry ("Date", QDateTime::currentDateTime());
  kdDebug() << "KPlayerPlaylistItem deleted\n";
}

KPlayerPlaylist::KPlayerPlaylist (void)
{
  m_loop = m_shuffle = false;
}

KPlayerPlaylist::~KPlayerPlaylist()
{
  kdDebug() << "KPlayerPlaylist deleted\n";
}

void KPlayerPlaylist::setShuffle (bool shuffle)
{
  m_shuffle = shuffle;
  randomize();
}

void KPlayerPlaylist::randomize (KPlayerPlaylistItem* item)
{
  if ( ! shuffle() || ! first() )
    return;
  KPlayerPlaylistItem* cur = current();
  if ( item && findRef (item) < 0 )
    item = 0;
  //KPlayerPlaylistItem* last_item = item ? prev() : 0;
  //if ( item )
  //  findRef (item);
  //else
  if ( item )
    next();
  else
    first();
  QPtrList<KPlayerPlaylistItem> list;
  while ( current() && current() != item )
    list.append (take());
  int offset = count();
  if ( list.first() )
    append (list.take());
  KRandomSequence rs;
  while ( list.first() )
    insertAt (rs.getLong (count() - offset + 1) + offset, list.take());
  if ( cur )
    setCurrent (cur);
}

void KPlayerPlaylist::setCurrent (KPlayerPlaylistItem* item)
{
  if ( ! item )
    return;
  KPlayerPlaylistItem* cur = current();
  int index = findRef (item);
  if ( index < 0 && cur )
    findRef (cur);
}

void KPlayerPlaylist::setCurrent (int index)
{
  setCurrent (at (index));
}

bool KPlayerPlaylist::has (KPlayerPlaylistItem* item)
{
  if ( ! item )
    return false;
  KPlayerPlaylistItem* cur = current();
  int index = findRef (item);
  findRef (cur);
  return index >= 0;
}

bool KPlayerPlaylist::next (void)
{
  bool ok = false;
  if ( ! current() && first() || QPtrList<KPlayerPlaylistItem>::next() )
    ok = true;
  else if ( m_loop && first() )
  {
    randomize();
    first();
    ok = true;
  }
  if ( ! ok )
    last();
  return ok;
}

bool KPlayerPlaylist::previous (void)
{
  bool ok = false;
  if ( ! current() && last() || prev() )
    ok = true;
  else if ( m_loop && last() )
  {
    randomize();
    last();
    ok = true;
  }
  if ( ! ok )
    first();
  return ok;
}

void KPlayerPlaylist::add (KPlayerPlaylistItem* item)
{
  if ( item )
    append (item);
}

void KPlayerPlaylist::remove (KPlayerPlaylistItem* item)
{
  KPlayerPlaylistItem* cur = current();
  int index = findRef (item);
  if ( index >= 0 )
    QPtrList<KPlayerPlaylistItem>::remove();
  if ( cur != item )
    setCurrent (cur);
  else if ( current() )
    setCurrent (current());
}

void KPlayerPlaylist::move (KPlayerPlaylistItem* item, KPlayerPlaylistItem* after)
{
  KPlayerPlaylistItem* cur = current();
  int to = after ? findRef (after) + 1 : 0;
  int from = findRef (item);
  if ( from >= 0 )
    insert (to, take());
  if ( cur )
    setCurrent (cur);
}

void KPlayerPlaylist::clear (void)
{
  QPtrList<KPlayerPlaylistItem>::clear();
}

void KPlayerPlaylist::rename (KPlayerPlaylistItem* item, const QString& s)
{
  QString name (s.isEmpty() ? item -> name() : s);
  item -> setName (name);
}

KPlayerPlaylistCombobox::KPlayerPlaylistCombobox (QWidget* parent, const char* name)
  : KComboBox (parent, name)
{
}

QSize KPlayerPlaylistCombobox::sizeHint() const
{
  QSize hint = KComboBox::sizeHint();
  //kdDebug() << "KPlayerPlaylistCombobox size hint " << hint.width() << "x" << hint.height() << "\n";
  hint.setWidth (kPlayerSettings() -> preferredSliderLength());
  return hint;
}

QSize KPlayerPlaylistCombobox::minimumSizeHint() const
{
  QSize hint = KComboBox::minimumSizeHint();
  //kdDebug() << "KPlayerPlaylistCombobox minimum size hint " << hint.width() << "x" << hint.height() << "\n";
  hint.setWidth (kPlayerSettings() -> minimumSliderLength());
  return hint;
}

void KPlayerPlaylistCombobox::randomize (KPlayerPlaylistItem* item)
{
  if ( ! shuffle() || ! first() )
    return;
  KPlayerPlaylistItem* cur = current();
  KPlayerPlaylist::randomize (item);
  KComboBox::clear();
  kdDebug() << KPlayerPlaylist::count() << " items\n";
  for ( item = first(); item; item = QPtrList<KPlayerPlaylistItem>::next() )
  {
    kdDebug() << index() << ": " << item -> name() << "\n";
    insertItem (item -> name());
  }
  if ( cur )
    setCurrent (cur);
}

void KPlayerPlaylistCombobox::setCurrent (KPlayerPlaylistItem* item)
{
  if ( ! item )
    return;
  KPlayerPlaylistItem* cur = current();
  int index = findRef (item);
  if ( index < 0 )
    findRef (cur);
  else
    setCurrentItem (index);
}

bool KPlayerPlaylistCombobox::next (void)
{
  bool ok = KPlayerPlaylist::next();
  if ( ok )
    setCurrentItem (index());
  return ok;
}

bool KPlayerPlaylistCombobox::previous (void)
{
  bool ok = KPlayerPlaylist::previous();
  if ( ok )
    setCurrentItem (index());
  return ok;
}

void KPlayerPlaylistCombobox::add (KPlayerPlaylistItem* item)
{
  if ( ! item )
    return;
  KPlayerPlaylist::add (item);
  insertItem (item -> name());
  setCurrentItem (index());
}

void KPlayerPlaylistCombobox::remove (KPlayerPlaylistItem* item)
{
  KPlayerPlaylistItem* cur = current();
  int index = findRef (item);
  if ( index >= 0 )
  {
    QPtrList<KPlayerPlaylistItem>::remove();
    removeItem (index);
  }
  if ( cur != item )
    setCurrent (cur);
  else if ( current() )
    setCurrent (current());
}

void KPlayerPlaylistCombobox::move (KPlayerPlaylistItem* item, KPlayerPlaylistItem* after)
{
  KPlayerPlaylistItem* cur = current();
  int to = after ? findRef (after) + 1 : 0;
  int from = findRef (item);
  if ( from >= 0 )
  {
    insert (to, take());
    QString text (text (from));
    removeItem (from);
    insertItem (text, to);
  }
  if ( cur )
    setCurrent (cur);
}

void KPlayerPlaylistCombobox::clear (void)
{
  KPlayerPlaylist::clear();
  KComboBox::clear();
}

void KPlayerPlaylistCombobox::rename (KPlayerPlaylistItem* item, const QString& s)
{
  KPlayerPlaylist::rename (item, s);
  if ( s.isEmpty() )
    return;
  KPlayerPlaylistItem* cur = current();
  int index = findRef (item);
  if ( index >= 0 )
  {
    setCurrentItem (index);
    setCurrentText (item -> name());
  }
  if ( cur )
    setCurrent (cur);
}

KPlayerPlaylistViewItem::KPlayerPlaylistViewItem (KPlayerPlaylistWidget* parent, const KURL& url)
  : KListViewItem (parent), KPlayerPlaylistItem (url)
{
  setText (0, name());
  setText (1, QFileInfo (url.fileName()).extension(false).lower());
  setText (8, url.isLocalFile() ? url.path() : url.prettyURL());
}

KPlayerPlaylistViewItem::~KPlayerPlaylistViewItem()
{
  kPlayerPlaylistConfig() -> setGroup (url().url());
  kPlayerPlaylistConfig() -> writeEntry ("Selected", isSelected());
  kdDebug() << "KPlayerPlaylistViewItem deleted\n";
}

void KPlayerPlaylistViewItem::load (void)
{
  KPlayerPlaylistItem::load();
  setSelected (kPlayerPlaylistConfig() -> readBoolEntry ("Selected"));
}

void KPlayerPlaylistViewItem::setName (const QString& name)
{
  if ( name.isEmpty() )
    return;
  KPlayerPlaylistItem::setName (name);
  setText (0, name);
}

void KPlayerPlaylistViewItem::setLength (float length)
{
  KPlayerPlaylistItem::setLength (length);
  setText (2, timeString (length));
}

void KPlayerPlaylistViewItem::setVideoSize (const QSize& video_size)
{
  KPlayerPlaylistItem::setVideoSize (video_size);
  setText (3, video_size.isEmpty() ? QString ("") :
    QString::number (video_size.width()) + "x" + QString::number (video_size.height()));
}

void KPlayerPlaylistViewItem::setVideoCodec (const QString& video_codec)
{
  KPlayerPlaylistItem::setVideoCodec (video_codec);
  setText (4, video_codec);
}

void KPlayerPlaylistViewItem::setVideoBitrate (int video_bitrate)
{
  KPlayerPlaylistItem::setVideoBitrate (video_bitrate);
  setText (5, video_bitrate > 0 ? QString::number (video_bitrate) : QString (""));
}

void KPlayerPlaylistViewItem::setAudioCodec (const QString& audio_codec)
{
  KPlayerPlaylistItem::setAudioCodec (audio_codec);
  setText (6, audio_codec);
}

void KPlayerPlaylistViewItem::setAudioBitrate (int audio_bitrate)
{
  KPlayerPlaylistItem::setAudioBitrate (audio_bitrate);
  setText (7, audio_bitrate > 0 ? QString::number (audio_bitrate) : QString (""));
}

KPlayerPlaylistAction::KPlayerPlaylistAction (const QString& text, const KShortcut& cut,
    const QObject* receiver, const char* slot, KActionCollection* parent, const char* name)
  : KWidgetAction (new KPlayerPlaylistCombobox (0, name), text, cut, receiver, slot, parent, name)
{
  setAutoSized (true);
  connect (playlist(), SIGNAL (activated(int)), this, SLOT (play(int)));
  if ( ! text.isEmpty() )
    QToolTip::add (widget(), text);
}

KPlayerPlaylistAction::~KPlayerPlaylistAction()
{
}

void KPlayerPlaylistAction::play (int index)
{
  playlist() -> KPlayerPlaylist::setCurrent (index);
  emit activated();
}

/*int KPlayerPlaylistAction::plug (QWidget* widget, int index)
{
  int result = KWidgetAction::plug (widget, index);
  if ( result < 0 )
    return result;
  //if ( ! text().isEmpty() )
  //  QToolTip::add (m_slider, text());
  return result;
}

void KPlayerPlaylistAction::unplug (QWidget* widget)
{
  KWidgetAction::unplug (widget);
  if ( ! slider() || ! isPlugged() || widget != slider() -> parent() )
    return;
}*/

KPlayerPlaylistWindow::KPlayerPlaylistWindow (QWidget* parent, const char* name)
  : QDockWindow (parent, name)
{
  m_playlist_widget = new KPlayerPlaylistWidget (this);
  boxLayout() -> addWidget (new QDockArea (Qt::Vertical, QDockArea::Normal, this, "playlistleftdock"));
  boxLayout() -> addWidget (m_playlist_widget);
  boxLayout() -> addWidget (new QDockArea (Qt::Vertical, QDockArea::Normal, this, "playlistrightdock"));
  setResizeEnabled (true);
  setCloseMode (QDockWindow::Always);
  setNewLine (true);
  setCaption (i18n("Playlist Editor"));
}

/*void KPlayerPlaylistWindow::showEvent (QShowEvent* event)
{
  QDockWindow::showEvent (event);
  //kPlayerPlaylistWidget() -> scrollToBottom();
  kPlayerPlaylistWidget() -> ensureItemVisible (kPlayerPlaylistWidget() -> currentItem());
}*/

void KPlayerPlaylistWindow::hideEvent (QHideEvent* event)
{
  QDockWindow::hideEvent (event);
  if ( isHidden() ) // || ! event -> spontaneous() )
    emit windowHidden();
}

void KPlayerPlaylistWindow::focusOutEvent (QFocusEvent* event)
{
  QDockWindow::focusOutEvent (event);
  kdDebug() << "Window focus out " << QFocusEvent::reason() << " " << event -> spontaneous() << "\n";
}

void KPlayerPlaylistWindow::focusInEvent (QFocusEvent* event)
{
  QDockWindow::focusInEvent (event);
  kdDebug() << "Window focus in " << QFocusEvent::reason() << " " << event -> spontaneous() << "\n";
}

void KPlayerPlaylistWindow::windowActivationChange (bool old)
{
  QDockWindow::windowActivationChange (old);
  kdDebug() << "Window activation " << old << " -> " << isActiveWindow() << "\n";
}

KPlayerPlaylistWidget::KPlayerPlaylistWidget (QWidget* parent, const char* name)
  : KListView (parent, name)
{
  m_locked = m_selection_only = false;
  m_shortcuts_enabled = true;
  m_enable_play = m_enable_pause = m_enable_stop = false;
  m_list = 0;
  m_ac = 0;
  connect (this, SIGNAL (executed(QListViewItem*)), this, SLOT (play (QListViewItem*)));
  connect (this, SIGNAL (itemRenamed (QListViewItem*, const QString&, int)), this, SLOT (rename (QListViewItem*, const QString&, int)));
  connect (this, SIGNAL (moved (QListViewItem*, QListViewItem*, QListViewItem*)), this, SLOT (move (QListViewItem*, QListViewItem*, QListViewItem*)));
  connect (this, SIGNAL (selectionChanged()), this, SLOT (updateSelection()));
  connect (kPlayerProcess(), SIGNAL (infoAvailable()), this, SLOT (playerInfoAvailable()));
  connect (kPlayerProcess(), SIGNAL (stateChanged(KPlayerProcess::State)), this, SLOT (playerStateChanged(KPlayerProcess::State)));
}

KPlayerPlaylistWidget::~KPlayerPlaylistWidget() 
{
}

void KPlayerPlaylistWidget::saveOptions (void)
{
  kdDebug() << "Saving playlist options\n";
  KConfig* config = kPlayerConfig();
  config -> setGroup ("Player Options");
  config -> writeEntry ("Playlist Locked", m_locked);
  config -> writeEntry ("Playlist Loop", m_list -> loop());
  config -> writeEntry ("Playlist Shuffle", m_list -> shuffle());
  config -> writeEntry ("Playlist Selection Only", m_selection_only);
  int i = 0;
  for ( QListViewItem* item = firstChild(); item; item = item -> nextSibling() )
  {
    KPlayerPlaylistViewItem* pli = (KPlayerPlaylistViewItem*) item;
    kPlayerPlaylistConfig() -> setGroup ("Playlist Entries");
    kPlayerPlaylistConfig() -> writeEntry ("Entry " + QString::number (i), pli -> url().url());
    i ++;
  }
  kPlayerPlaylistConfig() -> setGroup ("Playlist Entries");
  kPlayerPlaylistConfig() -> writeEntry ("Entries", i);
  for ( i = 0; i < columns(); i ++ )
    config -> writeEntry ("Playlist Column " + QString::number (i) + " Width", columnWidth (i));
}

void KPlayerPlaylistWidget::readOptions (void)
{
  kdDebug() << "Reading playlist options\n";
  //KPlayerSettings* settings = kPlayerSettings();
  KConfig* config = kPlayerConfig();
  config -> setGroup ("Player Options");
  m_locked = config -> readBoolEntry ("Playlist Locked");
  m_list -> setLoop (config -> readBoolEntry ("Playlist Loop"));
  m_list -> setShuffle (config -> readBoolEntry ("Playlist Shuffle"));
  m_selection_only = config -> readBoolEntry ("Playlist Selection Only");
  QString captions [9] = {
    i18n("Name"),
    i18n("Type"),
    i18n("Length"),
    i18n("Video Size"),
    i18n("Video Codec"),
    i18n("Video Bitrate"),
    i18n("Audio Codec"),
    i18n("Audio Bitrate"),
    i18n("Path")
  };
  int i, entries;
  for ( i = 0; i < 9; i ++ )
    addColumn (captions [i], config -> readNumEntry ("Playlist Column " + QString::number (i) + " Width", -1));
  setSorting (-1);
  kPlayerPlaylistConfig() -> setGroup ("Playlist Entries");
  entries = kPlayerPlaylistConfig() -> readNumEntry ("Entries");
  for ( i = 0; i < entries; i ++ )
  {
    kPlayerPlaylistConfig() -> setGroup ("Playlist Entries");
    KURL url (kPlayerPlaylistConfig() -> readEntry ("Entry " + QString::number (i)));
    if ( ! url.isEmpty() )
    {
      KPlayerPlaylistViewItem* item = new KPlayerPlaylistViewItem (this, url);
      item -> moveItem (lastItem());
      item -> load();
      if ( ! selectionOnly() || item -> isSelected() )
        m_list -> add (item);
    }
  }
  if ( ! m_list -> isEmpty() )
  {
    m_list -> randomize();
    m_list -> setCurrent (0);
  }
  setSelectionMode (QListView::Extended); // Multi or Extended
  //setSelectionModeExt (KListView::FileManager);
  setAllColumnsShowFocus (true);
  setSorting (-1);
  setResizeMode (QListView::NoColumn);
  setDragEnabled (true);
  setAcceptDrops (true);
  //setDropVisualizer (true);
  setItemsRenameable (true);
  setDefaultRenameAction (Accept);
  setAlternateBackground (KGlobalSettings::alternateBackgroundColor());
  //setTooltipColumn (0);
  //setShowToolTips (true);
  enableActions();
}

void KPlayerPlaylistWidget::refreshSettings (void)
{
  makeRoom (0);
  int i;
  if ( ! kPlayerSettings() -> allowDuplicateEntries() )
  {
    KPlayerPlaylistItem *item, *cur = m_list -> current();
    if ( m_list -> isTemporary() )
      for ( i = 0; i < m_list -> count() - 1; i ++ )
      {
        item = m_list -> item (i);
        if ( item )
        {
          KURL url (item -> url());
          for ( int j = i + 1; j < m_list -> count(); j ++ )
          {
            item = m_list -> item (j);
            if ( item && item -> url() == url )
            {
              if ( item == cur )
                m_list -> setCurrent (cur);
              m_list -> remove (item);
              if ( item == cur )
                cur = m_list -> current();
              j --;
            }
          }
        }
      }
    for ( QListViewItem* li = firstChild(); li; li = li -> nextSibling() )
    {
      KPlayerPlaylistViewItem* pli = (KPlayerPlaylistViewItem*) li;
      KURL url (pli -> url());
      for ( QListViewItem *next, *lj = li -> nextSibling(); lj; lj = next )
      {
        KPlayerPlaylistViewItem* plj = (KPlayerPlaylistViewItem*) lj;
        next = lj -> nextSibling();
        if ( plj -> url() == url )
        {
          if ( cur == (KPlayerPlaylistItem*) plj )
            m_list -> setCurrent (cur);
          m_list -> remove (plj);
          if ( cur == (KPlayerPlaylistItem*) plj )
            cur = m_list -> current();
          delete plj;
        }
      }
    }
    if ( cur )
      m_list -> setCurrent (cur);
  }
  QStringList groups (kPlayerPlaylistConfig() -> groupList());
  kdDebug() << "Cache has " << groups.count() << " entries\n";
  if ( int (groups.count()) > kPlayerSettings() -> cacheSizeLimit() + 1 )
  {
    QMap<QString,QString> map;
    i = 0;
    for ( QStringList::Iterator it = groups.begin(); it != groups.end(); ++ it )
      if ( *it != "Entries" )
      {
        kPlayerPlaylistConfig() -> setGroup (*it);
        QDateTime dt (kPlayerPlaylistConfig() -> readDateTimeEntry ("Date"));
        if ( ! dt.isNull() )
          map.insert (dt.toString (Qt::ISODate) + QString().sprintf ("-%04u", i ++), *it);
      }
    i = groups.count() - kPlayerSettings() -> cacheSizeLimit();
    for ( QMap<QString,QString>::Iterator mapit = map.begin(); i > 1 && mapit != map.end(); ++ mapit )
    {
      kdDebug() << "Deleting entry for " << mapit.data() << " from " << mapit.key() << "\n";
      kPlayerPlaylistConfig() -> deleteGroup (mapit.data());
      i --;
    }
  }
  enableActions();
}

void KPlayerPlaylistWidget::enableNextPrevious (bool same_url)
{
  if ( ! m_ac )
    return;
  kdDebug() << "Enabling next and previous actions\n";
  if ( ! same_url && m_list -> current() )
    same_url = kPlayerProcess() -> url() == m_list -> current() -> url();
  int count = m_list -> count();
  m_ac -> action ("playlist_next") -> setEnabled (count > 0 && ! same_url
    || count > 1 && (loop() || m_list -> index() < count - 1));
  m_ac -> action ("playlist_previous") -> setEnabled (count > 1
    && (loop() || m_list -> index() > 0));
}

void KPlayerPlaylistWidget::enableActions (void)
{
  if ( ! m_ac )
    return;
  bool selection = false;
  for ( QListViewItem* li = firstChild(); li; li = li -> nextSibling() )
    if ( li -> isSelected() )
    {
      selection = true;
      break;
    }
  bool visible = isVisible();
  m_ac -> action ("playlist_playlist") -> setEnabled (firstChild() != 0 && m_list -> isTemporary());
  m_ac -> action ("playlist_selection") -> setEnabled (! m_list -> isTemporary());
  m_ac -> action ("playlist_play") -> setEnabled (visible && firstChild() != 0);
  m_ac -> action ("playlist_rename") -> setEnabled (visible && currentItem() != 0);
  m_ac -> action ("playlist_move_up") -> setEnabled (visible && selection);
  m_ac -> action ("playlist_move_down") -> setEnabled (visible && selection);
  m_ac -> action ("playlist_remove") -> setEnabled (visible && selection);
  m_ac -> action ("playlist_clear") -> setEnabled (visible && firstChild() != 0);
  enableNextPrevious();
}

KURL KPlayerPlaylistWidget::current (void)
{
  KURL url (m_list -> current() ? m_list -> current() -> url() : KURL());
  enableNextPrevious (! url.isEmpty());
  return url;
}

KURL KPlayerPlaylistWidget::next (void)
{
  KURL url (kPlayerProcess() -> url() != current() || m_list -> next() ? current() : KURL());
  enableNextPrevious (! url.isEmpty());
  return url;
}

KURL KPlayerPlaylistWidget::previous (void)
{
  KURL url (kPlayerProcess() -> url() != current() || m_list -> previous() ? current() : KURL());
  enableNextPrevious (! url.isEmpty());
  return url;
}

void KPlayerPlaylistWidget::setSelectionOnly (bool selection_only)
{
  if ( m_selection_only == selection_only )
    return;
  m_selection_only = selection_only;
  if ( m_list -> isTemporary() )
    return;
  reload();
  enableNextPrevious();
}

void KPlayerPlaylistWidget::setLoop (bool loop)
{
  m_list -> setLoop (loop);
  enableNextPrevious();
}

void KPlayerPlaylistWidget::setShuffle (bool shuffle)
{
  if ( shuffle == m_list -> shuffle() )
    return;
  m_list -> setShuffle (shuffle);
  if ( ! shuffle && ! m_list -> isTemporary() )
    reload();
  enableNextPrevious();
}

void KPlayerPlaylistWidget::reload (void)
{
  KPlayerPlaylistItem* cur = m_list -> current();
  m_list -> clear();
  KPlayerPlaylistViewItem* item ((KPlayerPlaylistViewItem*) firstChild());
  bool find_selected = false;
  while ( item )
  {
    if ( ! selectionOnly() || item -> isSelected() )
      m_list -> add (item);
    if ( find_selected && item -> isSelected() )
    {
      cur = item;
      find_selected = false;
    }
    else if ( selectionOnly() && ! m_list -> shuffle() && ! item -> isSelected()
        && cur == (KPlayerPlaylistItem*) item )
    {
      find_selected = true;
      cur = 0;
    }
    item = (KPlayerPlaylistViewItem*) item -> nextSibling();
  }
  if ( ! m_list -> isEmpty() )
  {
    m_list -> randomize();
    m_list -> setCurrent (0);
    if ( cur )
      m_list -> setCurrent (cur);
  }
  m_list -> setTemporary (false);
}

void KPlayerPlaylistWidget::play (void)
{
  QListViewItem* item = currentItem();
  if ( ! item )
    item = firstChild();
  if ( item )
    play (item);
}

void KPlayerPlaylistWidget::play (QListViewItem* i)
{
  enablePlayerShortcuts (true);
  enablePlayerShortcuts (false);
  if ( m_list -> isTemporary() )
    reload();
  KPlayerPlaylistViewItem* item = (KPlayerPlaylistViewItem*) i;
  if ( selectionOnly() && ! i -> isSelected() )
  {
    i -> setSelected (true);
    updateSelection();
  }
  m_list -> setCurrent (item);
  enableActions();
  emit activated();
}

void KPlayerPlaylistWidget::rename (void)
{
  QListViewItem* item = currentItem();
  //if ( ! item )
  //  item = firstChild();
  if ( item )
    rename (item, 0);
}

void KPlayerPlaylistWidget::rename (QListViewItem* i, int column)
{
  KListView::rename (i, column);
  kdDebug() << "Renaming item " << i -> text (0) << "\n";
  enablePlayerShortcuts (false);
}

void KPlayerPlaylistWidget::rename (QListViewItem* i, const QString& name, int column)
{
  kdDebug() << "Rename signal: column " << column << " " << name << "\n";
  enablePlayerShortcuts (true);
  if ( column != 0 )
    return;
  KPlayerPlaylistViewItem* item = (KPlayerPlaylistViewItem*) i;
  m_list -> rename (item, name);
}

void KPlayerPlaylistWidget::updateSelection (void)
{
  if ( selectionOnly() && ! m_list -> isTemporary() )
  {
    if ( m_list -> shuffle() )
    {
      QPtrList<KPlayerPlaylistItem> selected, unselected;
      KPlayerPlaylistItem *pli, *cur = m_list -> current();
      for ( QListViewItem* item = firstChild(); item; item = item -> nextSibling() )
      {
        pli = (KPlayerPlaylistViewItem*) item;
        if ( m_list -> has (pli) != item -> isSelected() )
          (item -> isSelected() ? selected : unselected).append (pli);
      }
      for ( pli = selected.first(); pli; pli = selected.next() )
        m_list -> add (pli);
      if ( cur )
        m_list -> setCurrent (cur);
      else
        m_list -> setCurrent (0);
      m_list -> randomize (m_list -> current());
      for ( pli = unselected.first(); pli; pli = unselected.next() )
        m_list -> remove (pli);
    }
    else
      reload();
  }
  enableActions();
}

/*void KPlayerPlaylistWidget::setSelected (QListViewItem* item, bool selected)
{
  KListView::setSelected (item, selected);
  if ( ! selectionOnly() || m_list -> isTemporary() )
    return;
  if ( selected )
    m_list -> add ((KPlayerPlaylistViewItem*) item);
  else
    m_list -> remove ((KPlayerPlaylistViewItem*) item);
}*/

void KPlayerPlaylistWidget::move (QListViewItem* item, QListViewItem*, QListViewItem* after)
{
  if ( ! selectionOnly() && ! m_list -> shuffle() && ! m_list -> isTemporary() )
    m_list -> move ((KPlayerPlaylistViewItem*) item, (KPlayerPlaylistViewItem*) after);
  enableNextPrevious();
}

bool KPlayerPlaylistWidget::makeRoom (int room, KPlayerPlaylistViewItem* pli)
{
  KPlayerPlaylistItem* cur = m_list -> current();
  int limit = kPlayerSettings() -> playlistSizeLimit();
  bool result = false;
  while ( childCount() + room > limit && childCount() > 0 )
  {
    KPlayerPlaylistViewItem* item = (KPlayerPlaylistViewItem*) firstChild();
    if ( item == pli )
      kdDebug() << "Removing the given item\n";
    if ( item == pli )
      result = true;
    m_list -> remove (item);
    delete item;
  }
  while ( m_list -> count() + room > limit && m_list -> count() > 0 )
  {
    m_list -> setCurrent (0);
    m_list -> remove (m_list -> current());
  }
  if ( cur )
    m_list -> setCurrent (cur);
  return result;
}

void KPlayerPlaylistWidget::add (const KURL& url)
{
  KURL::List urls;
  urls << url;
  add (urls);
/*if ( url.path().isEmpty() && url.host().isEmpty() )
    return;
  if ( locked() && ! m_list -> isTemporary() )
  {
    m_list -> clear();
    m_list -> setTemporary (true);
  }
  KPlayerPlaylistItem* item;
  if ( ! kPlayerSettings() -> allowDuplicateEntries() )
  {
    if ( m_list -> isTemporary() )
      for ( int i = 0; i < m_list -> count(); i ++ )
      {
	m_list -> setCurrent (i);
	item = m_list -> current();
	if ( item && item -> url() == url )
	{
          m_list -> remove (item);
	  i --;
	}
      }
    else
      for ( QListViewItem *n, *i = firstChild(); i; i = n )
      {
	n = i -> nextSibling();
        KPlayerPlaylistViewItem* pli = (KPlayerPlaylistViewItem*) i;
        if ( pli -> url() == url )
        {
          m_list -> remove (pli);
          delete pli;
        }
      }
  }
  makeRoom (1);
  if ( locked() )
    item = new KPlayerPlaylistItem (url);
  else
  {
    KPlayerPlaylistViewItem* pli = new KPlayerPlaylistViewItem (this, url);
    pli -> moveItem (lastItem());
    if ( selectionOnly() )
      pli -> setSelected (true);
    item = pli;
  }
  item -> KPlayerPlaylistItem::load();
  m_list -> add (item);*/
}

void KPlayerPlaylistWidget::add (KURL::List& urls)
{
  add (urls, (KPlayerPlaylistViewItem*) lastChild());
}

void KPlayerPlaylistWidget::add (KURL::List& urls, KPlayerPlaylistViewItem* after)
{
  KPlayerPlaylistItem* item;
  if ( locked() && ! m_list -> isTemporary() )
  {
    m_list -> clear();
    m_list -> setTemporary (true);
  }
  if ( ! kPlayerSettings() -> allowDuplicateEntries() )
  {
    kdDebug() << "Checking for duplicate URLs\n";
    for ( QValueListIterator<KURL> it = urls.begin(); it != urls.end(); ++ it )
    {
      QValueListIterator<KURL> iter (it);
      for ( ++ iter; iter != urls.end(); *iter == *it ? (iter = urls.remove (iter)) : ++ iter );
    }
    kdDebug() << "Removing duplicate entries\n";
    if ( m_list -> isTemporary() )
      for ( int i = 0; i < m_list -> count(); i ++ )
      {
	item = m_list -> item (i);
	if ( item )
          for ( QValueListConstIterator<KURL> it = urls.begin(); it != urls.end(); ++ it )
            if ( item -> url() == *it )
            {
              m_list -> remove (item);
              i --;
	      break;
	    }
      }
    else
    {
      KPlayerPlaylistViewItem* last = 0;
      for ( QListViewItem *n, *i = firstChild(); i; i = n )
      {
	n = i -> nextSibling();
        KPlayerPlaylistViewItem* pli = (KPlayerPlaylistViewItem*) i;
	bool deleted = false;
        for ( QValueListConstIterator<KURL> it = urls.begin(); ! deleted && it != urls.end(); ++ it )
          if ( pli -> url() == *it )
          {
            kdDebug() << "Removing entry: " << pli -> url().prettyURL() << "\n";
	    if ( after == pli )
	      after = last;
            m_list -> remove (pli);
            delete pli;
	    deleted = true;
          }
	if ( ! deleted )
	  last = pli;
      }
    }
  }
  kdDebug() << "Making room for new entries\n";
  if ( makeRoom (urls.count(), after) )
    after = 0;
  KPlayerPlaylistItem* after_item = m_list -> shuffle() ?
    (m_list -> isEmpty() ? 0 : m_list -> item (m_list -> count() - 1)) : after;
  if ( after && selectionOnly() && ! m_list -> shuffle() && ! after -> isSelected() )
  {
    kdDebug() << "Looking for a selected entry to insert after\n";
    after_item = 0;
    for ( QListViewItem* i = firstChild(); i; i = i -> nextSibling() )
      if ( i -> isSelected() )
	after_item = (KPlayerPlaylistViewItem*) i;
      else if ( i == after )
	break;
  }
  bool at_end = after_item == (m_list -> isEmpty() ? 0 : m_list -> item (m_list -> count() - 1));
  KPlayerPlaylistItem* prev = after_item;
  if ( prev != (KPlayerPlaylistItem*) after )
    kdDebug() << "Different after item\n";
  if ( prev )
  {
    m_list -> setCurrent (prev);
    kdDebug() << "Index " << m_list -> index() << "\n";
  }
  else
    kdDebug() << "After item is zero\n";
  kdDebug() << "Adding new entries\n";
  for ( QValueListConstIterator<KURL> it = urls.begin(); it != urls.end(); ++ it )
  {
    if ( (*it).path().isEmpty() && (*it).host().isEmpty() )
      continue;
    if ( locked() )
      item = new KPlayerPlaylistItem (*it);
    else
    {
      kdDebug() << "Creating list view item\n";
      KPlayerPlaylistViewItem* pli = new KPlayerPlaylistViewItem (this, *it);
      if ( after )
        kdDebug() << "Moving the new item\n";
      if ( after )
        pli -> moveItem (after);
      if ( selectionOnly() )
        pli -> setSelected (true);
      item = pli;
      after = pli;
    }
    kdDebug() << "Loading the new item\n";
    item -> KPlayerPlaylistItem::load();
    kdDebug() << "Adding the new item\n";
    m_list -> add (item);
    if ( ! at_end )
      kdDebug() << "Moving the new item\n";
    if ( ! at_end )
      m_list -> move (item, after_item);
    after_item = item;
    kdDebug() << "Added entry: " << item -> url().prettyURL() << "\n";
  }
  kdDebug() << "Randomizing\n";
  m_list -> randomize (prev);
  kdDebug() << "Index " << m_list -> index() << "\n";
  if ( prev )
  {
    m_list -> setCurrent (prev);
    kdDebug() << "Index " << m_list -> index() << "\n";
    m_list -> next();
  }
  else
    m_list -> setCurrent (0);
  kdDebug() << "Index " << m_list -> index() << "\n";
  enableActions();
}

/*void KPlayerPlaylistWidget::moveUp (void)
{
  QListViewItem *item (firstChild()), *previous, *after = 0;
  int i = 0;
  while ( item && item -> isSelected() )
  {
    after = item;
    item = item -> nextSibling();
    i ++;
  }
  if ( ! item )
    return;
  item = item -> nextSibling();
  i ++;
  while ( item )
  {
    previous = item;
    item = item -> nextSibling();
    if ( previous -> isSelected() )
    {
      if ( after )
	previous -> moveItem (after);
      else
      {
	takeItem (previous);
	insertItem (previous);
      }
      after = previous;
      if ( ! selectionOnly() && ! m_list -> shuffle() && ! m_list -> isTemporary() )
        m_list -> move (i - 1, i);
    }
    else
    {
      if ( after )
	after = after -> nextSibling();
      else
	after = firstChild();
    }
    i ++;
  }
}

void KPlayerPlaylistWidget::moveDown (void)
{
  QListViewItem *item (firstChild()), *previous, *after (0), *next (item);
  int i = 0, j = -1, k = 0;
  while ( item && ! item -> isSelected() )
  {
    after = item;
    j = i;
    item = item -> nextSibling();
    i ++;
  }
  while ( item )
  {
    previous = item;
    item = item -> nextSibling();
    if ( previous -> isSelected() )
    {
      next = previous;
      k = i;
    }
    else if ( (after ? after -> nextSibling() : firstChild()) == previous )
    {
      after = previous;
      j = i;
    }
    else
    {
      if ( after )
	previous -> moveItem (after);
      else
      {
	takeItem (previous);
	insertItem (previous);
      }
      if ( ! selectionOnly() && ! m_list -> shuffle() && ! m_list -> isTemporary() )
        m_list -> move (j + 1, i);
      after = next;
      j = ++ k;
    }
    i ++;
  }
}*/

void KPlayerPlaylistWidget::moveUp (void)
{
  QListViewItem *item (firstChild()), *previous, *after = 0;
  while ( item && item -> isSelected() )
  {
    after = item;
    item = item -> nextSibling();
  }
  if ( ! item )
    return;
  item = item -> nextSibling();
  while ( item )
  {
    previous = item;
    item = item -> nextSibling();
    if ( previous -> isSelected() )
    {
      if ( after )
        previous -> moveItem (after);
      else
      {
        takeItem (previous);
        insertItem (previous);
      }
      if ( ! selectionOnly() && ! m_list -> shuffle() && ! m_list -> isTemporary() )
        m_list -> move ((KPlayerPlaylistViewItem*) previous, (KPlayerPlaylistViewItem*) after);
      after = previous;
    }
    else if ( after )
      after = after -> nextSibling();
    else
      after = firstChild();
  }
  enableNextPrevious();
}

void KPlayerPlaylistWidget::moveDown (void)
{
  QListViewItem *item (firstChild()), *previous, *after (0), *next (item);
  while ( item && ! item -> isSelected() )
  {
    after = item;
    item = item -> nextSibling();
  }
  while ( item )
  {
    previous = item;
    item = item -> nextSibling();
    if ( previous -> isSelected() )
      next = previous;
    else if ( (after ? after -> nextSibling() : firstChild()) == previous )
      after = previous;
    else
    {
      if ( after )
	previous -> moveItem (after);
      else
      {
	takeItem (previous);
	insertItem (previous);
      }
      if ( ! selectionOnly() && ! m_list -> shuffle() && ! m_list -> isTemporary() )
        m_list -> move ((KPlayerPlaylistViewItem*) previous, (KPlayerPlaylistViewItem*) after);
      after = next;
    }
  }
  enableNextPrevious();
}

void KPlayerPlaylistWidget::remove (void)
{
  QListViewItem *item (firstChild()), *previous;
  while ( item )
  {
    previous = item;
    item = item -> nextSibling();
    if ( previous -> isSelected() )
    {
      m_list -> remove ((KPlayerPlaylistViewItem*) previous);
      delete previous;
    }
  }
  enableActions();
}

void KPlayerPlaylistWidget::clear (void)
{
  KListView::clear();
  if ( ! m_list -> isTemporary() )
    m_list -> clear();
  enableActions();
}

void KPlayerPlaylistWidget::playerStateChanged (KPlayerProcess::State state)
{
  static KPlayerProcess::State previous = KPlayerProcess::Idle;
  if ( state == KPlayerProcess::Playing && previous < state )
    playerInfoAvailable();
  previous = state;
}

void KPlayerPlaylistWidget::playerInfoAvailable (void)
{
  KPlayerPlaylistItem* item = m_list -> current();
  if ( item -> url() != kPlayerProcess() -> url() )
    return;
  item -> setLength (kPlayerProcess() -> length());
  item -> setVideoSize (kPlayerProcess() -> size());
  item -> setVideoCodec (kPlayerProcess() -> videoCodec());
  item -> setVideoBitrate (kPlayerProcess() -> videoBitrate());
  item -> setAudioCodec (kPlayerProcess() -> audioCodec());
  item -> setAudioBitrate (kPlayerProcess() -> audioBitrate());
}

bool KPlayerPlaylistWidget::acceptDrag (QDropEvent* event) const
{
  return KURLDrag::canDecode (event) || KListView::acceptDrag (event);
}

void KPlayerPlaylistWidget::contentsDropEvent (QDropEvent* event)
{
  if ( KURLDrag::canDecode (event) )
  {
    cleanDropVisualizer();
    cleanItemHighlighter();
    event -> acceptAction();
    QListViewItem *parent, *after;
    findDrop (event -> pos(), parent, after);
    KURL::List urls;
    if ( ! KURLDrag::decode (event, urls) )
      return;
    bool locked = m_locked;
    m_locked = false;
    if ( m_list -> isTemporary() )
      reload();
    add (urls, (KPlayerPlaylistViewItem*) after);
    m_locked = locked;
    emit activated();
  }
  else
    KListView::contentsDropEvent (event);
}

void KPlayerPlaylistWidget::showEvent (QShowEvent* event)
{
  KListView::showEvent (event);
  ensureItemVisible (currentItem());
  enableActions();
}

void KPlayerPlaylistWidget::hideEvent (QHideEvent* event)
{
  KListView::hideEvent (event);
  enableActions();
}

void KPlayerPlaylistWidget::resizeEvent (QResizeEvent* event)
{
  bool at_bottom = contentsHeight() == contentsY() + visibleHeight();
  KListView::resizeEvent (event);
  if ( at_bottom && contentsHeight() != contentsY() + visibleHeight() )
    setContentsPos (contentsX(), contentsHeight() - visibleHeight());
}

void KPlayerPlaylistWidget::focusOutEvent (QFocusEvent* event)
{
  KListView::focusOutEvent (event);
  kdDebug() << "Widget focus out " << QFocusEvent::reason() << " " << event -> spontaneous() << "\n";
}

void KPlayerPlaylistWidget::focusInEvent (QFocusEvent* event)
{
  KListView::focusInEvent (event);
  kdDebug() << "Widget focus in " << QFocusEvent::reason() << " " << event -> spontaneous() << "\n";
}

void KPlayerPlaylistWidget::enablePlayerShortcuts (bool enable)
{
  if ( enable == m_shortcuts_enabled || ! m_ac )
    return;
  if ( ! enable && (! renameLineEdit() || ! renameLineEdit() -> isVisible()) )
    return;
  kdDebug() << "enablePlayerShortcut (" << enable << ")\n";
  KShortcut shortcut (Key_Return);
  KAction* action = m_ac -> action ("player_play");
  if ( enable && m_enable_play )
  {
    action -> setShortcut (shortcut);
    m_enable_play = false;
  }
  else if ( ! enable && action -> shortcut() == shortcut )
  {
    action -> setShortcut (KShortcut::null());
    m_enable_play = true;
  }
  shortcut.init (Key_Space);
  action = m_ac -> action ("player_pause");
  if ( enable && m_enable_pause )
  {
    action -> setShortcut (shortcut);
    m_enable_pause = false;
  }
  else if ( ! enable && action -> shortcut() == shortcut )
  {
    action -> setShortcut (KShortcut::null());
    m_enable_pause = true;
  }
  shortcut.init (Key_Escape);
  action = m_ac -> action ("player_stop");
  if ( enable && m_enable_stop )
  {
    action -> setShortcut (shortcut);
    m_enable_stop = false;
  }
  else if ( ! enable && action -> shortcut() == shortcut )
  {
    action -> setShortcut (KShortcut::null());
    m_enable_stop = true;
  }
  m_shortcuts_enabled = enable;
}

void KPlayerPlaylistWidget::windowActivationChange (bool old)
{
  KListView::windowActivationChange (old);
  kdDebug() << "Widget activation " << old << " -> " << isActiveWindow() << "\n";
  enablePlayerShortcuts (! isActiveWindow());
}
