/***************************************************************************
                          kplayerprocess.cpp
                          ------------------
    begin                : Sat Jan 11 2003
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcmdlineargs.h>
#include <kprocctrl.h>
#include <kstandarddirs.h>
#include <qapplication.h>
#include <qfileinfo.h>
#include <qregexp.h>
#include <unistd.h>

#include <kdebug.h>

#include "kplayerprocess.h"
#include "kplayerprocess.moc"
#include "kplayersettings.h"

#define MIN_VIDEO_LENGTH 5

//#define DEBUG_KPLAYER_PROCESS

static QRegExp re_a_or_v ("^[AV]: *([0-9,:.-]+)");
static QRegExp re_a_and_v ("^A: *([0-9,:.-]+) +V: *([0-9,:.-]+)");
static QRegExp re_video ("^V(?:IDE)?O: *\\S+ +(\\d+)x(\\d+)");
static QRegExp re_vo ("^V(?:IDE)?O:.* => +(\\d+)x(\\d+)");
static QRegExp re_vbr ("^VIDEO:.* (\\d+),?\\d* *kbps");
static QRegExp re_abr ("^AUDIO:.*\\((\\d+),?\\d* *kbit\\)");
static QRegExp re_vc ("^Selected video codec: \\[([A-Za-z0-9,:.-]+)\\]");
static QRegExp re_ac ("^Selected audio codec: \\[([A-Za-z0-9,:.-]+)\\]");
static QRegExp re_start ("^Start playing\\.\\.\\.");
static QRegExp re_playing ("^Playing ");
static QRegExp re_exiting ("^Exiting", false);
static QRegExp re_success ("^Exiting\\.\\.\\. \\((?:Quit|End of file)\\)", false);
static QRegExp re_extension ("\\.[^/.]+$");
static QRegExp re_ans_length ("^ANS_LENGTH=(\\d+)$");
static QRegExp re_split ("\\s*[,;:. ]\\s*");
static QRegExp re_cache_fill ("^Cache fill: *([0-9]+)[.,]?[0-9]* *%", false);
static QRegExp re_generating_index ("^Generating Index: *([0-9]+)[.,]?[0-9]* *%", false);
static QRegExp re_mpeg12 ("mpeg[12]", false);
static QRegExp re_version ("^MPlayer *0\\.9.* \\(C\\) ");
static QRegExp re_crash ("MPlayer crashed");
static QRegExp re_dvd_vcd ("^(?:vc|dv)d://", false);

KPlayerLineOutputProcess::KPlayerLineOutputProcess (void)
{
  m_stdout_line_length = m_stderr_line_length = 0;
  m_stdout_buffer_length = m_stderr_buffer_length = 129;
  m_stdout_buffer = new char [m_stdout_buffer_length];
  m_stderr_buffer = new char [m_stderr_buffer_length];
  m_merge = false;
  QApplication::connect (this, SIGNAL (receivedStdout (KProcess*, char*, int)),
    this, SLOT (slotReceivedStdout (KProcess*, char*, int)));
  QApplication::connect (this, SIGNAL (receivedStderr (KProcess*, char*, int)),
    this, SLOT (slotReceivedStderr (KProcess*, char*, int)));
}

KPlayerLineOutputProcess::~KPlayerLineOutputProcess()
{
  delete [] m_stdout_buffer;
  delete [] m_stderr_buffer;
}

void KPlayerLineOutputProcess::processHasExited (int state)
{
  status = state;
  runs = false;
  commClose();
  if ( m_stdout_line_length )
    emit receivedStdoutLine (this, m_stdout_buffer, m_stdout_line_length); // , None
  if ( m_stderr_line_length )
    emit receivedStderrLine (this, m_stderr_buffer, m_stderr_line_length); // , None
  if ( run_mode != DontCare )
    emit processExited (this);
}

void KPlayerLineOutputProcess::slotReceivedStdout (KProcess* proc, char* str, int len)
{
//kdDebug() << "StdOut: " << len << " '" << str << "'\n";
  receivedOutput (proc, str, len, m_stdout_buffer, m_stdout_buffer_length, m_stdout_line_length, true);
}

void KPlayerLineOutputProcess::slotReceivedStderr (KProcess* proc, char* str, int len)
{
  if ( m_merge )
    slotReceivedStdout (proc, str, len);
  else
    receivedOutput (proc, str, len, m_stderr_buffer, m_stderr_buffer_length, m_stderr_line_length, false);
}

void KPlayerLineOutputProcess::receivedOutput (KProcess* proc, char* str, int len, char* buf, int blen, int llen, bool stdout)
{
  static int avlen = 0;
  static char* av = 0;
  if ( proc != this )
    return;
  //kdDebug() << "stdout received length: " << len << "\n";
  //kdDebug() << llen << "/" << blen << ": " << buf << "\n";
  while ( len > 0 && ! str [len - 1] )
    len --;
  while ( len > 0 )
  {
    char* lf = (char*) memchr (str, '\n', len);
    if ( ! lf )
      lf = str + len;
    char* eol = (char*) memchr (str, '\r', lf - str);
    if ( ! eol )
      eol = lf;
    if ( eol - str + llen >= blen )
    {
      char* old_buffer = buf;
      blen = eol - str + llen + 10;
      kdDebug() << "new buffer length: " << blen << "\n";
      buf = new char [blen];
      if ( stdout )
      {
        m_stdout_buffer = buf;
        m_stdout_buffer_length = blen;
      }
      else
      {
        m_stderr_buffer = buf;
        m_stderr_buffer_length = blen;
      }
      if ( llen )
        memcpy (buf, old_buffer, llen);
      delete[] old_buffer;
    }
    if ( eol > str )
    {
      memcpy (buf + llen, str, eol - str);
      llen += eol - str;
      if ( stdout )
        m_stdout_line_length = llen;
      else
        m_stderr_line_length = llen;
    }
    buf [llen] = 0;
    if ( eol - str == len )
      return;
    if ( re_a_or_v.search (buf) >= 0 || re_cache_fill.search (buf) >= 0 || re_generating_index.search (buf) >= 0 )
    {
      if ( avlen <= llen )
      {
        if ( av )
          delete[] av;
        avlen = llen + 10;
        av = new char [avlen];
        kdDebug() << "new av buffer length: " << avlen << "\n";
      }
      memcpy (av, buf, llen + 1);
    }
    else if ( stdout )
      emit receivedStdoutLine (this, buf, llen); // , *cr == '\r' ? CR : LF
    else
      emit receivedStderrLine (this, buf, llen); // , *cr == '\r' ? CR : LF
//  if ( *buf ) // && eol == lf
//  {
//    write (STDOUT_FILENO, buf, llen);
//    write (STDOUT_FILENO, "\n", 1);
//  }
//  kdDebug() << "Buffer: " << buf << "\n";
    llen = 0;
    if ( stdout )
      m_stdout_line_length = llen;
    else
      m_stderr_line_length = llen;
    len -= eol - str + 1;
    str = eol + 1;
  }
  if ( av && *av )
  {
    if ( stdout )
      emit receivedStdoutLine (this, av, strlen (av) - 1);
    else
      emit receivedStderrLine (this, av, strlen (av) - 1);
    *av = 0;
  }
//kdDebug() << "normal return\n";
}

KPlayerProcess::KPlayerProcess (void)
{
  m_player = m_helper = 0;
  m_state = Idle;
  m_frame_drop = None;
  m_width = m_height = 0;
  m_volume = 50;
  m_show_subtitles = true;
  m_mute = m_paused = m_quit = m_size_sent = m_info_available = m_09_version = false;
  m_delayed_play = false;
  m_brightness = m_contrast = m_hue = m_saturation = 0;
  m_length = m_position = m_max_position = m_helper_position = m_paused_position = 0;
  m_seek = m_seek_count = m_ans_length = 0;
  m_video_bitrate = m_audio_bitrate = 0;
}

KPlayerProcess::~KPlayerProcess()
{
  if ( m_player )
    delete m_player;
  if ( m_helper )
    delete m_helper;
}

void KPlayerProcess::setUrl (KURL url)
{
  if ( m_url == url )
    return;
  kdDebug() << "KPP New URL: '" << url.url() << "'\n";
  m_url = url;
  setSubtitleUrl ("");
  KPlayerSettings* settings = kPlayerSettings();
  if ( ! settings -> autoloadSubtitles() || ! url.isLocalFile() )
    return;
  QStringList exts;
  if ( settings -> autoloadOtherSubtitles() )
    exts = QStringList::split (re_split, settings -> autoloadExtensionList());
  if ( settings -> autoloadAqtSubtitles() )
    exts.append ("aqt");
  if ( settings -> autoloadJssSubtitles() )
    exts.append ("jss");
  if ( settings -> autoloadRtSubtitles() )
    exts.append ("rt");
  if ( settings -> autoloadSmiSubtitles() )
    exts.append ("smi");
  if ( settings -> autoloadSrtSubtitles() )
    exts.append ("srt");
  if ( settings -> autoloadSsaSubtitles() )
    exts.append ("ssa");
  if ( settings -> autoloadSubSubtitles() )
    exts.append ("sub");
  if ( settings -> autoloadTxtSubtitles() )
    exts.append ("txt");
  if ( settings -> autoloadUtfSubtitles() )
    exts.append ("utf");
  QString urls (url.path()), urlc (urls);
  bool has_ext = re_extension.search (urls) >= 0;
  QFileInfo fi;
  for ( QStringList::Iterator it = exts.begin(); it != exts.end(); ++ it )
  {
    QString ext ("." + (*it).lower());
    QString extu (ext.upper());
    if ( has_ext )
    {
      urlc.replace (re_extension, ext);
      //kdDebug() << "Autoloading " << urlc << "\n";
      fi.setFile (urlc);
      if ( fi.exists() && fi.isReadable() )
      {
        setSubtitleUrl (urlc);
        return;
      }
      urlc.replace (re_extension, extu);
      //kdDebug() << "Autoloading " << urlc << "\n";
      fi.setFile (urlc);
      if ( fi.exists() && fi.isReadable() )
      {
        setSubtitleUrl (urlc);
        return;
      }
    }
    //kdDebug() << "Autoloading " << urls << ext << "\n";
    fi.setFile (urls + ext);
    if ( fi.exists() && fi.isReadable() )
    {
      setSubtitleUrl (urls + ext);
      return;
    }
    //kdDebug() << "Autoloading " << urls << extu << "\n";
    fi.setFile (urls + extu);
    if ( fi.exists() && fi.isReadable() )
    {
      setSubtitleUrl (urls + extu);
      return;
    }
  }
}

void KPlayerProcess::setState (State state)
{
  static State previous = Idle;
  if ( m_state == state )
    return;
  m_state = state;
  if ( previous == Running && state == Playing && ! m_show_subtitles && ! m_subtitle.isEmpty() )
  {
    m_show_subtitles = true;
    setShowSubtitles (false);
  }
  kdDebug() << "New state: " << state << ", previous state: " << previous << ", position: " << m_position << "\n";
  if ( (previous == Running && state != Playing && state != Paused || state == Idle && m_max_position < 0.45) && ! m_quit )
    emit errorDetected();
  previous = state;
  if ( ! m_quit || state == Idle )
    emit stateChanged (state);
}

QString KPlayerProcess::positionString (void) const
{
  QString l (lengthString()), p (timeString (position(), true));
  if ( l.isEmpty() )
    return p;
  return p + " / " + l;
}

bool KPlayerProcess::get_info (void)
{
  m_width = 0;
  m_height = 0;
  m_length = 0;
  m_info_available = false;
  m_video_bitrate = m_audio_bitrate = 0;
  m_video_codec = m_audio_codec = "";
  m_seek = 0;
  m_seek_count = 0;
  m_helper_position = 0;
  m_ans_length = 0;
  m_size_sent = false;
  m_delayed_play = false;
  KPlayerSettings* settings = kPlayerSettings();
  m_helper = new KPlayerLineOutputProcess;
  *m_helper << settings -> executablePath() << "-slave" << "-ao" << "null" << "-vo" << "null";
  QApplication::connect (m_helper, SIGNAL (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)),
    this, SLOT (receivedHelperLine (KPlayerLineOutputProcess*, char*, int)));
  if ( ! run (m_helper) )
  {
    delete m_helper;
    m_helper = 0;
    return false;
  }
  m_helper -> writeStdin ("get_time_length\n", 16);
  return true;
}

bool KPlayerProcess::play (void)
{
  m_position = m_max_position = 0;
  m_paused = m_quit = m_09_version = m_delayed_play = false;
  if ( m_helper && re_dvd_vcd.search (url().url()) >= 0 )
  {
    m_delayed_play = true;
    setState (Running);
    kdDebug() << "Delaying play...\n";
    return true;
  }
  KPlayerSettings* settings = kPlayerSettings();
  m_player = new KPlayerLineOutputProcess;
  *m_player << settings -> executablePath() << "-slave" << "-zoom" << "-noautosub" << "-wid" << QCString().setNum (m_winId);
  QString driver = settings -> videoDriver();
  if ( ! driver.isEmpty() )
  {
    if ( ! settings -> videoDevice().isEmpty() )
      driver += ":" + settings -> videoDevice();
    if ( settings -> videoDriverFallback() )
      driver += ",";
    *m_player << "-vo" << driver;
  }
  driver = settings -> audioDriver();
  if ( ! driver.isEmpty() )
  {
    if ( ! settings -> audioDevice().isEmpty() )
      driver += ":" + settings -> audioDevice();
    if ( settings -> audioDriverFallback() )
      driver += ",";
    *m_player << "-ao" << driver;
  }
  *m_player << "-osdlevel" << QString().setNum (settings -> osdLevel());
  *m_player << "-contrast" << QCString().setNum (m_contrast);
  *m_player << "-brightness" << QCString().setNum (m_brightness);
  *m_player << "-hue" << QCString().setNum (m_hue);
  *m_player << "-saturation" << QCString().setNum (m_saturation);
  if ( m_frame_drop == KPlayerProcess::Soft )
    *m_player << "-framedrop";
  else if ( m_frame_drop == KPlayerProcess::Hard )
    *m_player << "-hardframedrop";
  if ( settings -> videoScaler() > 0 )
    *m_player << "-sws" << QString().setNum (settings -> videoScaler());
  if ( settings -> videoDoublebuffering() )
    *m_player << "-double";
  if ( settings -> videoDirectRendering() )
    *m_player << "-dr";
  if ( settings -> videoDriver() != "sdl" && settings -> videoDriver() != "svga" )
  {
    QString path (KGlobal::dirs() -> findResource ("appdata", "input.conf"));
    if ( path.isEmpty() )
      kdDebug() << "input.conf path is empty\n";
    if ( path.isEmpty() )
      path = KGlobal::dirs() -> findResource ("data", "kplayer/input.conf");
    if ( path.isEmpty() )
      kdDebug() << "input.conf path is still empty\n";
    if ( ! path.isEmpty() )
      *m_player << "-input" << "conf=" + path;
    kdDebug() << "Input.conf: '" << path << "'\n";
  }
  if ( ! m_subtitle.isEmpty() )
    *m_player << "-sub" << (m_subtitle.isLocalFile() ? m_subtitle.path() : m_subtitle.url());
  QApplication::connect (m_player, SIGNAL (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)),
    this, SLOT (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)));
  if ( ! run (m_player) )
  {
    delete m_player;
    m_player = 0;
    setState (Idle);
    return false;
  }
  setState (Running);
  return true;
}

bool KPlayerProcess::run (KPlayerLineOutputProcess* player)
{
  KPlayerSettings* settings = kPlayerSettings();
  QString driver = settings -> videoCodec();
  if ( ! driver.isEmpty() )
  {
    if ( settings -> videoCodecFallback() )
      driver += ",";
    *player << "-vc" << driver;
  }
  driver = settings -> audioCodec();
  if ( ! driver.isEmpty() )
  {
    if ( settings -> audioCodecFallback() )
      driver += ",";
    *player << "-ac" << driver;
  }
  if ( settings -> useCache() )
    *player << "-cache" << QString().setNum (settings -> cacheSize());
  else
    *player << "-nocache";
  if ( settings -> buildNewIndex() )
    *player << "-idx";
  if ( ! settings -> commandLine().isEmpty() )
    *player << QStringList::split (QChar (' '), settings -> commandLine());
  *player << (m_url.isLocalFile() ? m_url.path() : m_url.url());
  QApplication::connect (player, SIGNAL (processExited (KProcess*)),
    this, SLOT (onProcessExited (KProcess*)));
//QApplication::connect (player, SIGNAL (receivedStderrLine (KPlayerLineOutputProcess*, char*, int)),
//  this, SLOT (receivedStderrLine (KPlayerLineOutputProcess*, char*, int)));
  player -> setMerge (true);
  return player -> start (KProcess::NotifyOnExit, KProcess::All);
}

void KPlayerProcess::pause (void)
{
  if ( ! m_player || m_quit )
    return;
  m_player -> writeStdin ("pause\n", 6);
  setState (m_state == Paused ? Playing : Paused);
  kdDebug() << "pause\n";
  m_paused = ! m_paused;
  if ( m_paused )
    m_paused_position = m_position;
}

void KPlayerProcess::stop (void)
{
  m_delayed_play = false;
  if ( m_player )
  {
    m_player -> writeStdin ("quit\n", 5);
    kdDebug() << "quit\n";
  }
  m_quit = true;
  if ( m_player )
  {
    if ( ! m_paused )
    {
      if ( m_player -> isRunning() )
        kdDebug() << "MPlayer is running. Waiting...\n";
      KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_player && m_player -> isRunning() )
        KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_player && m_player -> isRunning() )
        KProcessController::theKProcessController -> waitForProcessExit (1);
    }
    if ( m_player && m_player -> isRunning() )
    {
      kdDebug() << "Closing MPlayer...\n";
      m_player -> kill();
      KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_player && m_player -> isRunning() )
      {
        kdDebug() << "Killing MPlayer...\n";
        m_player -> kill (SIGKILL);
        KProcessController::theKProcessController -> waitForProcessExit (1);
        if ( m_player && m_player -> isRunning() )
        {
          kdDebug() << "Could not shut down MPlayer\n";
          m_player -> detach();
        }
      }
    }
    if ( m_player )
    {
      delete m_player;
      m_player = 0;
    }
  }
  setState (Idle);
}

void KPlayerProcess::kill (void)
{
  m_delayed_play = false;
  if ( m_player )
  {
    m_player -> writeStdin ("quit\n", 5);
    kdDebug() << "quit\n";
  }
  m_quit = true;
  if ( m_helper )
  {
    m_helper -> writeStdin ("quit\n", 5);
    kdDebug() << "quit\n";
    m_seek = 500;
  }
  if ( m_player )
  {
    if ( ! m_paused )
    {
      if ( m_player -> isRunning() )
        kdDebug() << "MPlayer is running. Waiting...\n";
      KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_player && m_player -> isRunning() )
        KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_player && m_player -> isRunning() )
        KProcessController::theKProcessController -> waitForProcessExit (1);
    }
    if ( m_player && m_player -> isRunning() )
    {
      kdDebug() << "Closing MPlayer...\n";
      m_player -> kill();
      KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_player && m_player -> isRunning() )
      {
        kdDebug() << "Killing MPlayer...\n";
        m_player -> kill (SIGKILL);
        KProcessController::theKProcessController -> waitForProcessExit (1);
        if ( m_player && m_player -> isRunning() )
        {
          kdDebug() << "Could not shut down MPlayer\n";
          m_player -> detach();
        }
      }
    }
    if ( m_player )
    {
      delete m_player;
      m_player = 0;
    }
  }
  if ( m_helper )
  {
    if ( m_helper -> isRunning() )
      kdDebug() << "MPlayer helper process is running. Waiting...\n";
    KProcessController::theKProcessController -> waitForProcessExit (1);
    if ( m_helper && m_helper -> isRunning() )
      KProcessController::theKProcessController -> waitForProcessExit (1);
    if ( m_helper && m_helper -> isRunning() )
      KProcessController::theKProcessController -> waitForProcessExit (1);
    if ( m_helper && m_helper -> isRunning() )
    {
      kdDebug() << "Closing MPlayer helper process...\n";
      m_helper -> kill();
      KProcessController::theKProcessController -> waitForProcessExit (1);
      if ( m_helper && m_helper -> isRunning() )
      {
        kdDebug() << "Killing MPlayer helper process...\n";
        m_helper -> kill (SIGKILL);
        KProcessController::theKProcessController -> waitForProcessExit (1);
        if ( m_helper && m_helper -> isRunning() )
        {
          kdDebug() << "Could not shut down MPlayer helper process\n";
          m_helper -> detach();
        }
      }
    }
    if ( m_helper )
    {
      delete m_helper;
      m_helper = 0;
    }
  }
  setState (Idle);
}

void KPlayerProcess::absoluteSeek (int seconds)
{
  if ( ! m_player || m_quit )
    return;
  if ( seconds < 0 )
    seconds = 0;
  QString s;
  // broken codec workaround
  if ( re_mpeg12.search (videoCodec()) >= 0 && length() > 2 )
  {
    seconds = int (float (seconds) / length() * 100 + 0.5);
    if ( seconds > 100 )
      seconds = 100;
    s.sprintf ("seek %u 1\n", seconds);
  }
  else
    s.sprintf ("seek %u 2\n", seconds);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
  m_paused = false;
}

void KPlayerProcess::relativeSeek (int seconds)
{
  if ( ! m_player || m_quit || seconds == 0 )
    return;
  QString s;
  s.sprintf ("seek %d\n", seconds);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
  m_paused = false;
}

void KPlayerProcess::setVolume (int volume)
{
  if ( volume > 100 )
    volume = 100;
  else if ( volume < 0 )
    volume = 0;
  m_volume = volume;
  if ( ! m_player || m_quit || m_mute )
    return;
  QString s;
  s.sprintf ("volume %d 1\n", volume);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
  if ( position() > 0 )
    m_paused = false;
}

void KPlayerProcess::mute (void)
{
  if ( m_mute )
  {
    m_mute = false;
    setVolume (m_volume);
  }
  else
  {
    if ( m_player && ! m_quit )
      m_player -> writeStdin ("volume 0 1\n", 11);
    m_mute = true;
    if ( position() > 0 )
      m_paused = false;
  }
}

void KPlayerProcess::setFrameDropping (KPlayerProcess::FrameDropping frame_drop)
{
  m_frame_drop = frame_drop;
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("frame_drop %u\n", frame_drop);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setBrightness (int brightness)
{
  if ( brightness > 100 )
    brightness = 100;
  else if ( brightness < -100 )
    brightness = -100;
  m_brightness = brightness;
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("brightness %d 1\n", brightness);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setContrast (int contrast)
{
  if ( contrast > 100 )
    contrast = 100;
  else if ( contrast < -100 )
    contrast = -100;
  m_contrast = contrast;
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("contrast %d 1\n", contrast);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setHue (int hue)
{
  if ( hue > 100 )
    hue = 100;
  else if ( hue < -100 )
    hue = -100;
  m_hue = hue;
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("hue %d 1\n", hue);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setSaturation (int saturation)
{
  if ( saturation > 100 )
    saturation = 100;
  else if ( saturation < -100 )
    saturation = -100;
  m_saturation = saturation;
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("saturation %d 1\n", saturation);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::setShowSubtitles (bool show)
{
  if ( m_show_subtitles == show )
    return;
  m_show_subtitles = show;
  if ( ! m_player || m_quit || m_subtitle.isEmpty() )
    return;
  m_player -> writeStdin ("sub_visibility\n", 15);
  kdDebug() << "sub_visibility\n";
  m_paused = false;
}

void KPlayerProcess::subtitleMove (int position)
{
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("sub_pos %d\n", position);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::subtitleDelay (int delay)
{
  if ( ! m_player || m_quit )
    return;
  QString s;
  s.sprintf ("sub_delay %d\n", delay);
  m_player -> writeStdin (s.latin1(), s.length());
  kdDebug() << s.latin1();
}

void KPlayerProcess::onProcessExited (KProcess *proc)
{
  if ( proc == m_player )
  {
    delete m_player;
    m_player = 0;
    setState (Idle);
    kdDebug() << "MPlayer process exited\n";
  }
  else if ( proc == m_helper )
  {
    delete m_helper;
    m_helper = 0;
    if ( m_seek < 500 && m_helper_position >= MIN_VIDEO_LENGTH )
      m_length = m_helper_position;
    //if ( m_length == 0 && m_ans_length > 0 )
    if ( m_length < MIN_VIDEO_LENGTH * 4 && m_ans_length > MIN_VIDEO_LENGTH * 10
        || m_length == 0 && m_ans_length > 0 )
      m_length = m_ans_length;
    m_info_available = true;
    if ( ! m_quit )
      emit infoAvailable();
    if ( ! m_size_sent && ! m_quit )
    {
      emit sizeAvailable();
      m_size_sent = true;
    }
    kdDebug() << "MPlayer helper process exited\n";
    if ( m_delayed_play && ! m_player )
    {
      kdDebug() << "Delayed play...\n";
      play();
    }
  }
  else
  {
    delete proc;
    kdDebug() << "Stray MPlayer process exited\n";
  }
}

float KPlayerProcess::stringToFloat (QString stime)
{
  int comma = stime.find (',');
  if ( comma >= 0 )
    stime [comma] = '.';
  QStringList sl = QStringList::split (':', stime);
  int i = 0, n = 0;
  if ( sl.count() > 4 || sl.count() < 1 )
    return 0;
//  kdDebug() << "WARNING: Unexpected time string '" << stime << "'\n";
  if ( sl.count() > 3 )
    n = sl[i++].toInt() * 86400;
  if ( sl.count() > 2 )
    n += sl[i++].toInt() * 3600;
  if ( sl.count() > 1 )
    n += sl[i++].toInt() * 60;
  return sl[i].toFloat() + n;
}

void KPlayerProcess::receivedStdoutLine (KPlayerLineOutputProcess* proc, char* str, int len)
{
  if ( proc != m_player )
  {
    char buf [1025];
    if ( len > 1024 )
      len = 1024;
    memcpy (buf, str, len);
    buf [len] = 0;
    if ( re_exiting.search (buf) < 0 )
      proc -> writeStdin ("quit\n", 5);
    return;
  }
  bool sent = false;
  float ftime;
  //kdDebug() << "<<< " << str << "\n";
  if ( re_version.search (str) >= 0 )
  {
    m_09_version = true;
    kdDebug() << "MPlayer 0.9x detected\n";
  }
  if ( re_vo.search (str) >= 0 )
  {
    m_width = re_vo.cap(1).toInt();
    m_height = re_vo.cap(2).toInt();
    kdDebug() << "Adjusted Width " << m_width << " Height " << m_height << "\n";
    if ( ! m_size_sent && ! m_quit )
    {
      emit sizeAvailable();
      m_size_sent = true;
    }
  }
  else if ( ! m_width && ! m_height && re_video.search (str) >= 0 )
  {
    m_width = re_video.cap(1).toInt();
    m_height = re_video.cap(2).toInt();
    kdDebug() << "Width " << m_width << " Height " << m_height << "\n";
  }
  if ( m_state != Playing && re_playing.search (str) >= 0 )
  {
    if ( m_mute )
    {
      m_mute = false;
      mute();
    }
    else
      setVolume (m_volume);
    setContrast (m_contrast);
    setBrightness (m_brightness);
    setHue (m_hue);
    setSaturation (m_saturation);
  }
  if ( m_state != Playing && m_state != Paused && re_start.search (str) >= 0 )
    setState (Playing);
  if ( re_exiting.search (str) >= 0 && re_success.search (str) < 0 && ! m_quit )
    emit errorDetected();
  if ( re_crash.search (str) >= 0 )
    emit errorDetected();
  if ( re_vbr.search (str) >= 0 )
    m_video_bitrate = re_vbr.cap(1).toInt();
  if ( re_abr.search (str) >= 0 )
    m_audio_bitrate = re_abr.cap(1).toInt();
  if ( re_vc.search (str) >= 0 )
    m_video_codec = re_vc.cap(1);
  if ( re_ac.search (str) >= 0 )
    m_audio_codec = re_ac.cap(1);
//kdDebug() << "matching a_or_v regex\n";
  if ( re_a_or_v.search (str) >= 0 )
  {
    if ( m_state != Playing && ! m_paused )
      setState (Playing);
    if ( m_quit && ! sent )
    {
      m_player -> writeStdin ("quit\n", 5);
      sent = true;
    }
//  kdDebug() << "regex matched\n";
    if ( re_a_and_v.search (str) >= 0 )
    {
      ftime = stringToFloat (re_a_and_v.cap (1));
      float ftime2 = stringToFloat (re_a_and_v.cap (2));
      if ( ftime2 > ftime )
        ftime = ftime2;
    }
    else
    {
      ftime = stringToFloat (re_a_or_v.cap (1));
//    kdDebug() << "match: " << re_a_or_v.cap (1) << "ftime: " << ftime << "\n";
    }
//  if ( ftime > m_length && ftime >= MIN_VIDEO_LENGTH && m_length > 0 )
    if ( ftime > m_length && m_length >= MIN_VIDEO_LENGTH )
      m_length = ftime;
    if ( ftime != m_position )
    {
      m_position = ftime;
      if ( m_position > m_max_position )
	m_max_position = m_position;
      if ( ! m_quit )
        emit progressChanged (m_position, Position);
//    kdDebug() << "position: " << m_position << " length: " << m_length << "\n";
    }
    if ( m_paused && ! m_quit && ! sent && m_position - m_paused_position >= 1 )
    {
      m_player -> writeStdin ("pause\n", 6);
      kdDebug() << "pause\n";
      m_paused_position = m_position;
      sent = true;
    }
  }
  else if ( re_cache_fill.search (str) >= 0 )
  {
    ftime = stringToFloat (re_cache_fill.cap (1));
    if ( ! m_quit )
      emit progressChanged (ftime, CacheFill);
  }
  else if ( re_generating_index.search (str) >= 0 )
  {
    ftime = stringToFloat (re_generating_index.cap (1));
    if ( ! m_quit )
      emit progressChanged (ftime, IndexGeneration);
  }
  else if ( ! m_quit )
    emit messageReceived (str);
}

void KPlayerProcess::receivedHelperLine (KPlayerLineOutputProcess* proc, char* str, int len)
{
  if ( proc != m_helper )
  {
    char buf [1025];
    if ( len > 1024 )
      len = 1024;
    memcpy (buf, str, len);
    buf [len] = 0;
    if ( re_exiting.search (buf) < 0 )
      proc -> writeStdin ("quit\n", 5);
    return;
  }
  bool sent = false;
  float ftime;
#ifdef DEBUG_KPLAYER_PROCESS
  kdDebug() << str << "\n";
#endif
//kdDebug() << "matching video regex\n";
  if ( re_vo.search (str) >= 0 )
  {
    m_width = re_vo.cap(1).toInt();
    m_height = re_vo.cap(2).toInt();
    kdDebug() << "Adjusted Width " << m_width << " Height " << m_height << "\n";
    if ( ! m_size_sent && ! m_quit )
    {
      emit sizeAvailable();
      m_size_sent = true;
    }
  }
  else if ( ! m_width && ! m_height && re_video.search (str) >= 0 )
  {
    m_width = re_video.cap(1).toInt();
    m_height = re_video.cap(2).toInt();
    kdDebug() << "Width " << m_width << " Height " << m_height << "\n";
  }
  if ( re_ans_length.search (str) >= 0 )
  {
    kdDebug() << "Matched ans_length: " << re_ans_length.cap (1) << " in " << re_ans_length.cap (0) << "\n";
    m_ans_length = re_ans_length.cap (1).toInt();
  }
  if ( ! m_video_bitrate && re_vbr.search (str) >= 0 )
    m_video_bitrate = re_vbr.cap(1).toInt();
  if ( ! m_audio_bitrate && re_abr.search (str) >= 0 )
    m_audio_bitrate = re_abr.cap(1).toInt();
  if ( m_video_codec.isEmpty() && re_vc.search (str) >= 0 )
    m_video_codec = re_vc.cap(1);
  if ( m_audio_codec.isEmpty() && re_ac.search (str) >= 0 )
    m_audio_codec = re_ac.cap(1);
//kdDebug() << "matching a_or_v regex\n";
  if ( re_a_or_v.search (str) >= 0 )
  {
//  kdDebug() << "regex matched\n";
    if ( re_a_and_v.search (str) >= 0 )
    {
      ftime = stringToFloat (re_a_and_v.cap (1));
      float ftime2 = stringToFloat (re_a_and_v.cap (2));
      if ( ftime2 > ftime )
        ftime = ftime2;
    }
    else
    {
      ftime = stringToFloat (re_a_or_v.cap (1));
//    kdDebug() << "match: " << re_a_or_v.cap (1) << "ftime: " << ftime << "\n";
    }
    if ( ftime > m_length && ftime >= MIN_VIDEO_LENGTH && m_length > 0 )
      m_length = ftime;
    if ( ftime != m_helper_position )
    {
      m_helper_position = ftime;
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "helper position: " << m_helper_position << " length: " << m_length << "\n";
#endif
    }
    if ( m_seek > 0 && ! sent )
    {
      if ( ++ m_seek_count < 5 )
        sent = true;
      else
        m_seek_count = 0;
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "m_seek_count: " << m_seek_count << "\n";
#endif
    }
    if ( m_seek > 0 && m_seek < 500 && ftime >= MIN_VIDEO_LENGTH )
    {
      float estlength = ftime * 100 / m_seek;
      if ( m_length < estlength )
        m_length = estlength;
      kdDebug() << "estimated length: " << m_length << "\n";
    }
    if ( m_seek == 0 && ! sent )
    {
      m_helper -> writeStdin ("seek 99 1\n", 10);
//    m_helper -> writeStdin ("get_time_length\n", 16);
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "seek 99 1\n";
#endif
      m_seek = 99;
      m_seek_count = 0;
      sent = true;
    }
    if ( m_seek == 99 && ! sent && m_length < MIN_VIDEO_LENGTH )
    {
      m_helper -> writeStdin ("seek 95 1\n", 10);
//    m_helper -> writeStdin ("get_time_length\n", 16);
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "seek 95 1\n";
#endif
      m_seek = 95;
      m_seek_count = 0;
      sent = true;
    }
    if ( m_seek == 95 && ! sent && m_length < MIN_VIDEO_LENGTH )
    {
      m_helper -> writeStdin ("seek 90 1\n", 10);
//    m_helper -> writeStdin ("get_time_length\n", 16);
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "seek 90 1\n";
#endif
      m_seek = 90;
      m_seek_count = 0;
      sent = true;
    }
    if ( m_seek == 90 && ! sent && m_length < MIN_VIDEO_LENGTH )
    {
      m_helper -> writeStdin ("seek 50 1\n", 10);
//    m_helper -> writeStdin ("get_time_length\n", 16);
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "seek 50 1\n";
#endif
      m_seek = 50;
      m_seek_count = 0;
      sent = true;
    }
    if ( m_seek < 100 && m_length >= MIN_VIDEO_LENGTH )
    {
      m_helper -> writeStdin ("seek 100 1\n", 11);
//    m_helper -> writeStdin ("get_time_length\n", 16);
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "seek 100 1\n";
#endif
      m_seek = 100;
      m_seek_count = 0;
      sent = true;
    }
    if ( (m_seek == 50 || m_seek == 100) && ! sent )
    {
      m_helper -> writeStdin ("quit\n", 5);
#ifdef DEBUG_KPLAYER_PROCESS
      kdDebug() << "quit\n";
#endif
      m_seek = 500;
      sent = true;
    }
  }
}

/*void KPlayerProcess::receivedStderrLine (KPlayerLineOutputProcess*, char* str, int len)
{
  write (STDERR_FILENO, str, len);
  write (STDERR_FILENO, "\n", 1);
}*/
