/***************************************************************************
                          kplayersettings.cpp
                          -------------------
    begin                : Tue Apr 22 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qregexp.h>
//#include <stdlib.h>

#include <kdebug.h>

#include "kplayerprocess.h"
#include "kplayersettings.h"
#include "kplayersettings.moc"

// #define DEBUG_KPLAYER_SETTINGS

KPlayerProcess* KPlayerSettings::m_process = 0;
KPlayerSettings* KPlayerSettings::m_settings = 0;
KConfig* KPlayerSettings::m_config = 0;
KConfig* KPlayerSettings::m_playlist_config = 0;

static QRegExp re_driver ("([A-Za-z0-9]+)\\s+(\\S.*)");
static QRegExp re_codec ("([A-Za-z0-9]+)\\s+\\S+\\s+\\S+\\s+(\\S.*)");
static QRegExp re_multiple ("\\s+"), re_trailing (" $");
static QRegExp re_brackets (" ?\\[.*\\]"), re_parentheses (" ?\\(..+\\)");
static QRegExp re_audio ("[- ](?:audio[- ])?(?:decoder|codec|output)s?", false);
static QRegExp re_video ("[- ](?:video[- ])?(?:decoder|codec|output)s?", false);
static QRegExp re_s ("'s", false), re_layer ("layer-", false);
static QRegExp re_xv ("/XV$", false), re_vidix (" \\(VIDIX\\)", false);
static QRegExp re_matrox ("Matrox .* overlay in", false);
static QRegExp re_macintosh ("Macintosh Audio Compression and Expansion", false);

KPlayerSettings::KPlayerSettings (void)
{
  m_audio_drivers
    << "mpegpes: DVB"
    << "oss: OSS/ioctl"
    << "alsa9: ALSA-0.9.x"
    << "esd: EsounD"
    << "nas: NAS"
    << "sdl: SDLlib"
    << "null: Null"
    << "pcm: RAW PCM/WAVE file writer"
    << "plugin: Plugin";
  m_audio_codecs
    << "wma9dmo: Windows Media Audio 9 DMO"
    << "wmadmo: Windows Media Audio DMO"
    << "qdmc: Quicktime QDMC/QDM2"
    << "qclp: Quicktime QCLP"
    << "qtmace3: Quicktime MACE3"
    << "qtmace6: Quicktime MACE6"
    << "racook: RealAudio COOK"
    << "rasipr: RealAudio Sipro"
    << "raatrc: RealAudio ATRAC3"
    << "imaadpcm: IMA ADPCM"
    << "msadpcm: MS ADPCM"
    << "dk4adpcm: Duck DK4 ADPCM"
    << "dk3adpcm: Duck DK3 ADPCM"
    << "roqaudio: Id RoQ File"
    << "libdv: raw DV"
    << "ffdv: FFmpeg DV"
    << "faad: AAC MPEG2/MPEG4 Audio"
    << "pcm: Uncompressed PCM"
    << "divx: DivX audio"
    << "msadpcmacm: MS ADPCM"
    << "mp3: mp3lib MPEG layer 2, layer 3"
    << "ffwmav1: DivX audio v1"
    << "ffwmav2: DivX audio v2"
    << "ffmac3: Macintosh Audio Comp. and Exp. 3:1"
    << "ffmac6: Macintosh Audio Comp. and Exp. 6:1"
    << "ffmp3: FFmpeg layer 3"
    << "ffmp2: FFmpeg layer 1,2"
    << "mad: libMAD MPEG layer 1-2-3"
    << "mp3acm: MPEG layer 3"
    << "imaadpcmacm: IMA ADPCM"
    << "msgsmacm: MS GSM"
    << "msnaudio: MSN AUDIO"
    << "msgsm: MS GSM"
    << "alaw: aLaw"
    << "ulaw: uLaw"
    << "dvdpcm: Uncompressed DVD/VOB LPCM"
    << "a52: AC3-liba52"
    << "ac3: AC3-libac3"
    << "voxware: VoxWare"
    << "acelp: ACELP.net Sipro Lab"
    << "imc: Intel Music Coder"
    << "hwac3: AC3 through SPDIF"
    << "vorbis: OggVorbis"
    << "vivoaudio: Vivo G.723/Siren"
    << "g72x: G.711/G.721/G.723"
    << "atrac3: Sony ATRAC3"
    << "ALF2: ALF2"
    << "truespeech: DSP Group TrueSpeech"
    << "voxwarert24: VoxWare RT24 speech"
    << "lhacm: Lernout & Hauspie CELP and SBC";
  m_video_drivers
    << "xmga: Matrox overlay in X11 window"
    << "mga: Matrox G200/G4x0/G550 overlay"
    << "3dfx: 3dfx"
    << "tdfxfb: 3Dfx Banshee/Voodoo3/Voodoo5"
    << "xv: X11/XVideo"
    << "x11: X11"
    << "dga: DGA"
    << "sdl: SDL YUV/RGB/BGR renderer"
    << "ggi: General Graphics Interface"
    << "fbdev: Framebuffer Device"
    << "svga: SVGAlib"
    << "aa: AAlib"
    << "dxr3: DXR3/H+ video out"
    << "png: PNG file"
    << "jpeg: JPEG file"
    << "gif89a: animated GIF"
    << "null: Null video"
    << "pgm: PGM file"
    << "md5: MD5 sum"
    << "mpegpes: Mpeg-PES to DVB card"
    << "yuv4mpeg: yuv4mpeg for mjpegtools"
    << "vesa: VESA VBE 2.0 video"
    << "xvidix: X11/VIDIX";
  m_video_codecs
    << "mpegpes: Mpeg PES"
    << "mpeg12: MPEG 1 or 2"
    << "ffmpeg12: FFmpeg MPEG 1/2"
    << "nuv: NuppelVideo"
    << "mpng: PNG images"
    << "mtga: TGA images"
    << "sgi: SGI images"
    << "fli: Autodesk FLI/FLC Animation"
    << "qtrle: Quicktime Animation"
    << "qtrpza: Quicktime Apple Video"
    << "qtsmc: Apple Graphics"
    << "cyuv: Creative YUV"
    << "msrle: Microsoft RLE"
    << "roqvideo: Id RoQ File Video"
    << "lzo: LZO compressed"
    << "cram: Microsoft Video 1"
    << "cvidvfw: Cinepak Video"
    << "huffyuv: HuffYUV"
    << "svq1: Sorenson v1"
    << "video1: Microsoft Video 1"
    << "cvid: Cinepak Video"
    << "mphuffyuv: HuffYUV"
    << "mpmszh: AVImszh"
    << "mpzlib: AVIzlib"
    << "cvidxa: XAnim Radius Cinepak Video"
    << "ffhuffyuv: FFHuffYUV"
    << "ffsvq1: FFmpeg Sorenson Video v1"
    << "ffdivx: FFmpeg DivX ;-)"
    << "ffmp42: FFmpeg M$ MPEG-4 v2"
    << "ffmp41: FFmpeg M$ MPEG-4 v1"
    << "ffwmv1: FFmpeg M$ WMV1/WMV7"
    << "ffwmv2: FFmpeg M$ WMV2/WMV8"
    << "ffodivx: FFmpeg MPEG-4"
    << "xvid: Xvid"
    << "odivx: OpenDivX API"
    << "divx4: DivX4Linux API"
    << "divx4vfw: DivX4Windows-VFW"
    << "divxds: DivX ;-)"
    << "divx: DivX ;-)"
    << "mpeg4ds: Microsoft MPEG-4 v1/v2"
    << "mpeg4: Microsoft MPEG-4 v1/v2"
    << "wmv8: Windows Media Video 8"
    << "wmv7: Windows Media Video 7"
    << "wmv9dmo: Windows Media Video 9 DMO"
    << "wmvdmo: Windows Media Video DMO"
    << "ubmp4: UB Video MPEG 4"
    << "ffmjpeg: FFmpeg MJPEG"
    << "ijpg: Independent JPEG Group"
    << "m3jpeg: Morgan Motion JPEG"
    << "mjpeg: MainConcept Motion JPEG"
    << "avid: AVID Motion JPEG"
    << "LEAD: LEAD (M)Jpeg"
    << "imagepower: ImagePower MJPEG2000"
    << "m3jpeg2k: Morgan MJPEG2000"
    << "m3jpegds: Morgan MJPEG"
    << "pegasusm: Pegasus Motion JPEG"
    << "pegasusl: Pegasus Lossless JPEG"
    << "pegasusmwv: Pegasus Motion Wavelet 2000"
    << "vivo: Vivo H.263"
    << "u263: UB Video H.263/H.263+/H.263++"
    << "i263: I263"
    << "ffi263: FFmpeg I263"
    << "ffh263: FFmpeg H263+"
    << "h263xa: XAnim CCITT H.263"
    << "h261xa: XAnim CCITT H.261"
    << "m261: M261"
    << "indeo5ds: Intel Indeo 5"
    << "indeo5: Intel Indeo 5"
    << "indeo4: Intel Indeo 4.1"
    << "indeo3: Intel Indeo 3.1/3.2"
    << "indeo5xa: XAnim Intel Indeo 5"
    << "indeo4xa: XAnim Intel Indeo 4.1"
    << "indeo3xa: XAnim Intel Indeo 3.1/3.2"
    << "qdv: Sony Digital Video"
    << "ffdv: FFmpeg DV"
    << "libdv: Raw DV"
    << "mcdv: MainConcept DV"
    << "3ivXxa: XAnim 3ivX Delta 3.5 plugin"
    << "3ivX: 3ivX Delta 3.5"
    << "rv40: Linux RealPlayer 9 RV40"
    << "rv40win: Win32 RealPlayer 9 RV40"
    << "rv30: Linux RealPlayer 8 RV30"
    << "rv30win: Win32 RealPlayer 8 RV30"
    << "rv20: Linux RealPlayer 8 RV20"
    << "rv20win: Win32 RealPlayer 8 RV20"
    << "ffrv10: FFmpeg RV10"
    << "vp3: On2 OpenSource VP3"
    << "mwv1: Motion Wavelets"
    << "asv2: ASUS V2"
    << "asv1: ASUS V1"
    << "mss1: Windows Screen Video"
    << "ucod: UCOD-ClearVideo"
    << "vcr2: ATI VCR-2"
    << "CJPG: CJPG"
    << "tm20: TrueMotion 2.0"
    << "sp5x: SP5x - used by Aiptek MegaCam"
    << "qt3ivx: win32/quicktime 3IV1"
    << "qth263: win32/quicktime H.263"
    << "qtrlerpza: win32/quicktime RLE/RPZA"
    << "qtvp3: win32/quicktime VP3"
    << "qtzygo: win32/quicktime ZyGo"
    << "qtbhiv: win32/quicktime BeHereiVideo"
    << "qtcvid: win32/quicktime Cinepak"
    << "qtindeo: win32/quicktime Indeo"
    << "qtmjpeg: win32/quicktime MJPEG"
    << "qtmpeg4: win32/quicktime MPEG-4"
    << "qtsvq3: win32/quicktime SVQ3"
    << "qtsvq1: win32/quicktime SVQ1"
    << "zlib: AVIzlib"
    << "mszh: AVImszh"
    << "alaris: Alaris VideoGramPiX"
    << "vcr1: ATI VCR-1"
    << "pim1: Pinnacle Hardware MPEG1"
    << "qpeg: Q-Team QPEG"
    << "rricm: rricm"
    << "camtasia: TechSmith Camtasia Screen"
    << "rawrgb32: RAW RGB32"
    << "rawrgb24: RAW RGB24"
    << "rawrgb16: RAW RGB16"
    << "rawbgr32flip: RAW BGR32"
    << "rawbgr32: RAW BGR32"
    << "rawbgr24flip: RAW BGR24"
    << "rawbgr24: RAW BGR24"
    << "rawbgr16flip: RAW BGR15"
    << "rawbgr16: RAW BGR15"
    << "rawbgr15flip: RAW BGR15"
    << "rawbgr15: RAW BGR15"
    << "rawbgr8flip: RAW BGR8"
    << "rawbgr8: RAW BGR8"
    << "rawbgr1: RAW BGR1"
    << "rawyuy2: RAW YUY2"
    << "rawuyvy: RAW UYVY"
    << "rawyv12: RAW YV12"
    << "rawi420: RAW I420"
    << "rawyvu9: RAW YVU9"
    << "null: Null";
  defaults();
  load();
  refresh();
}

void KPlayerSettings::defaults (void)
{
  setResizeAutomatically (true);
  setMinimumInitialWidth (450);
  setRecentFileListSize (10);
  setShowFilePath (false);
  setStartPlayingImmediately (true);
  setShowOpenDialog (false);
  setDisableScreenSaver (true);
  setAllowDuplicateEntries (false);
  setPlaylistSizeLimit (100);
  setCacheSizeLimit (1000);
  setPreferredSliderLength (200);
  setMinimumSliderLength (100);
  setProgressNormalSeekUnits (0);
  setProgressNormalSeek (1);
  setProgressFastSeekUnits (0);
  setProgressFastSeek (10);
  setProgressMarks (10);
  setVolumeMinimum (0);
  setVolumeMaximum (100);
  setVolumeMarks (10);
  setVolumeStep (1);
  setVolumeDefault (50);
  setVolumeReset (false);
  setVolumeEvery (0);
  setContrastMinimum (-100);
  setContrastMaximum (100);
  setContrastMarks (20);
  setContrastStep (1);
  setContrastDefault (0);
  setContrastReset (false);
  setContrastEvery (0);
  setBrightnessMinimum (-50);
  setBrightnessMaximum (50);
  setBrightnessMarks (10);
  setBrightnessStep (1);
  setBrightnessDefault (0);
  setBrightnessReset (false);
  setBrightnessEvery (0);
  setHueMinimum (-100);
  setHueMaximum (100);
  setHueMarks (20);
  setHueStep (1);
  setHueDefault (0);
  setHueReset (false);
  setHueEvery (0);
  setSaturationMinimum (-100);
  setSaturationMaximum (100);
  setSaturationMarks (20);
  setSaturationStep (1);
  setSaturationDefault (0);
  setSaturationReset (false);
  setSaturationEvery (0);
  setVideoDriver ("xv");
  setVideoDriverFallback (true);
  setVideoDevice ("");
  setVideoCodec ("");
  setVideoCodecFallback (true);
  setVideoScaler (0);
  setVideoDoublebuffering (false);
  setVideoDirectRendering (false);
  setAudioDriver ("");
  setAudioDriverFallback (true);
  setAudioDevice ("");
  setAudioCodec ("");
  setAudioCodecFallback (true);
  setAutoloadSubtitles (true);
  setAutoloadAqtSubtitles (true);
  setAutoloadJssSubtitles (true);
  setAutoloadRtSubtitles (true);
  setAutoloadSmiSubtitles (true);
  setAutoloadSrtSubtitles (true);
  setAutoloadSsaSubtitles (true);
  setAutoloadSubSubtitles (true);
  setAutoloadTxtSubtitles (true);
  setAutoloadUtfSubtitles (true);
  setAutoloadOtherSubtitles (false);
  setAutoloadExtensionList ("");
  setShowMessagesOnError (true);
  setShowMessagesBeforePlaying (false);
  setShowMessagesAfterPlaying (false);
  setClearMessagesBeforePlaying (true);
  setExecutablePath ("mplayer");
  setCommandLine ("");
  setUseCache (false);
  setCacheSize (0);
  setBuildNewIndex (false);
  setOsdLevel (0);
}

void KPlayerSettings::load (void)
{
  KConfig* config = kPlayerConfig();
  config -> setGroup ("Player Options");
  setResizeAutomatically (config -> readBoolEntry ("Resize Main Window Automatically", resizeAutomatically()));
  setMinimumInitialWidth (config -> readUnsignedNumEntry ("Minimum Initial Width", minimumInitialWidth()));
  setRecentFileListSize (config -> readUnsignedNumEntry ("Recent File List Size", recentFileListSize()));
  setShowFilePath (config -> readBoolEntry ("Show File Path", showFilePath()));
  setStartPlayingImmediately (config -> readBoolEntry ("Start Playing Immediately", startPlayingImmediately()));
  setShowOpenDialog (config -> readBoolEntry ("Show Open Dialog", showOpenDialog()));
  setDisableScreenSaver (config -> readBoolEntry ("Disable Screen Saver", disableScreenSaver()));
  setAllowDuplicateEntries (config -> readBoolEntry ("Allow Duplicate Entries", allowDuplicateEntries()));
  setPlaylistSizeLimit (config -> readUnsignedNumEntry ("Playlist Size Limit", playlistSizeLimit()));
  setCacheSizeLimit (config -> readUnsignedNumEntry ("Cache Size Limit", cacheSizeLimit()));
  setPreferredSliderLength (config -> readUnsignedNumEntry ("Preferred Slider Length", preferredSliderLength()));
  setMinimumSliderLength (config -> readUnsignedNumEntry ("Minimum Slider Length", minimumSliderLength()));
  setProgressNormalSeekUnits (config -> readUnsignedNumEntry ("Normal Seek Units", progressNormalSeekUnits()));
  setProgressNormalSeek (config -> readUnsignedNumEntry ("Normal Seek", progressNormalSeek()));
  setProgressFastSeekUnits (config -> readUnsignedNumEntry ("Fast Seek Units", progressFastSeekUnits()));
  setProgressFastSeek (config -> readUnsignedNumEntry ("Fast Seek", progressFastSeek()));
  setProgressMarks (config -> readUnsignedNumEntry ("Progress Marks", progressMarks()));
  setVolumeMinimum (config -> readNumEntry ("Volume Minimum", volumeMinimum()));
  setVolumeMaximum (config -> readNumEntry ("Volume Maximum", volumeMaximum()));
  setVolumeMarks (config -> readNumEntry ("Volume Marks", volumeMarks()));
  setVolumeStep (config -> readNumEntry ("Volume Step", volumeStep()));
  setVolumeDefault (config -> readNumEntry ("Volume Default", volumeDefault()));
  setVolumeReset (config -> readBoolEntry ("Volume Reset", volumeReset()));
  setVolumeEvery (config -> readNumEntry ("Volume Every", volumeEvery()));
  setContrastMinimum (config -> readNumEntry ("Contrast Minimum", contrastMinimum()));
  setContrastMaximum (config -> readNumEntry ("Contrast Maximum", contrastMaximum()));
  setContrastMarks (config -> readNumEntry ("Contrast Marks", contrastMarks()));
  setContrastStep (config -> readNumEntry ("Contrast Step", contrastStep()));
  setContrastDefault (config -> readNumEntry ("Contrast Default", contrastDefault()));
  setContrastReset (config -> readBoolEntry ("Contrast Reset", contrastReset()));
  setContrastEvery (config -> readNumEntry ("Contrast Every", contrastEvery()));
  setBrightnessMinimum (config -> readNumEntry ("Brightness Minimum", brightnessMinimum()));
  setBrightnessMaximum (config -> readNumEntry ("Brightness Maximum", brightnessMaximum()));
  setBrightnessMarks (config -> readNumEntry ("Brightness Marks", brightnessMarks()));
  setBrightnessStep (config -> readNumEntry ("Brightness Step", brightnessStep()));
  setBrightnessDefault (config -> readNumEntry ("Brightness Default", brightnessDefault()));
  setBrightnessReset (config -> readBoolEntry ("Brightness Reset", brightnessReset()));
  setBrightnessEvery (config -> readNumEntry ("Brightness Every", brightnessEvery()));
  setHueMinimum (config -> readNumEntry ("Hue Minimum", hueMinimum()));
  setHueMaximum (config -> readNumEntry ("Hue Maximum", hueMaximum()));
  setHueMarks (config -> readNumEntry ("Hue Marks", hueMarks()));
  setHueStep (config -> readNumEntry ("Hue Step", hueStep()));
  setHueDefault (config -> readNumEntry ("Hue Default", hueDefault()));
  setHueReset (config -> readBoolEntry ("Hue Reset", hueReset()));
  setHueEvery (config -> readNumEntry ("Hue Every", hueEvery()));
  setSaturationMinimum (config -> readNumEntry ("Saturation Minimum", saturationMinimum()));
  setSaturationMaximum (config -> readNumEntry ("Saturation Maximum", saturationMaximum()));
  setSaturationMarks (config -> readNumEntry ("Saturation Marks", saturationMarks()));
  setSaturationStep (config -> readNumEntry ("Saturation Step", saturationStep()));
  setSaturationDefault (config -> readNumEntry ("Saturation Default", saturationDefault()));
  setSaturationReset (config -> readBoolEntry ("Saturation Reset", saturationReset()));
  setSaturationEvery (config -> readNumEntry ("Saturation Every", saturationEvery()));
  setVideoDriver (config -> readEntry ("Video Driver", videoDriver()));
  setVideoDriverFallback (config -> readBoolEntry ("Video Driver Fallback", videoDriverFallback()));
  setVideoDevice (config -> readEntry ("Video Device", videoDevice()));
  setVideoCodec (config -> readEntry ("Video Codec", videoCodec()));
  setVideoCodecFallback (config -> readBoolEntry ("Video Codec Fallback", videoCodecFallback()));
  setVideoScaler (config -> readUnsignedNumEntry ("Video Scaler", videoScaler()));
  setVideoDoublebuffering (config -> readBoolEntry ("Enable Doublebuffering", videoDoublebuffering()));
  setVideoDirectRendering (config -> readBoolEntry ("Enable Direct Rendering", videoDirectRendering()));
  setAudioDriver (config -> readEntry ("Audio Driver", audioDriver()));
  setAudioDriverFallback (config -> readBoolEntry ("Audio Driver Fallback", audioDriverFallback()));
  setAudioDevice (config -> readEntry ("Audio Device", audioDevice()));
  setAudioCodec (config -> readEntry ("Audio Codec", audioCodec()));
  setAudioCodecFallback (config -> readBoolEntry ("Audio Codec Fallback", audioCodecFallback()));
  setAutoloadSubtitles (config -> readBoolEntry ("Autoload Subtitles", autoloadSubtitles()));
  setAutoloadAqtSubtitles (config -> readBoolEntry ("Autoload Aqt Subtitles", autoloadAqtSubtitles()));
  setAutoloadJssSubtitles (config -> readBoolEntry ("Autoload Jss Subtitles", autoloadJssSubtitles()));
  setAutoloadRtSubtitles (config -> readBoolEntry ("Autoload Rt Subtitles", autoloadRtSubtitles()));
  setAutoloadSmiSubtitles (config -> readBoolEntry ("Autoload Smi Subtitles", autoloadSmiSubtitles()));
  setAutoloadSrtSubtitles (config -> readBoolEntry ("Autoload Srt Subtitles", autoloadSrtSubtitles()));
  setAutoloadSsaSubtitles (config -> readBoolEntry ("Autoload Ssa Subtitles", autoloadSsaSubtitles()));
  setAutoloadSubSubtitles (config -> readBoolEntry ("Autoload Sub Subtitles", autoloadSubSubtitles()));
  setAutoloadTxtSubtitles (config -> readBoolEntry ("Autoload Txt Subtitles", autoloadTxtSubtitles()));
  setAutoloadUtfSubtitles (config -> readBoolEntry ("Autoload Utf Subtitles", autoloadUtfSubtitles()));
  setAutoloadOtherSubtitles (config -> readBoolEntry ("Autoload Other Subtitles", autoloadOtherSubtitles()));
  setAutoloadExtensionList (config -> readEntry ("Autoload Extension List", autoloadExtensionList()));
  setShowMessagesOnError (config -> readBoolEntry ("Show Messages On Error", showMessagesOnError()));
  setShowMessagesBeforePlaying (config -> readBoolEntry ("Show Messages Before Playing", showMessagesBeforePlaying()));
  setShowMessagesAfterPlaying (config -> readBoolEntry ("Show Messages After Playing", showMessagesAfterPlaying()));
  setClearMessagesBeforePlaying (config -> readBoolEntry ("Clear Messages Before Playing", clearMessagesBeforePlaying()));
  setExecutablePath (config -> readEntry ("Executable Path", executablePath()));
  setCommandLine (config -> readEntry ("Command Line", commandLine()));
  setUseCache (config -> readBoolEntry ("Use Cache", useCache()));
  setCacheSize (config -> readUnsignedNumEntry ("Cache Size", cacheSize()));
  setBuildNewIndex (config -> readBoolEntry ("Build Index", buildNewIndex()));
  setOsdLevel (config -> readUnsignedNumEntry ("OSD Level", osdLevel()));
}

void KPlayerSettings::save (void)
{
  KConfig* config = kPlayerConfig();
  config -> setGroup ("Player Options");
  config -> writeEntry ("Resize Main Window Automatically", resizeAutomatically());
  config -> writeEntry ("Minimum Initial Width", minimumInitialWidth());
  config -> writeEntry ("Recent File List Size", recentFileListSize());
  config -> writeEntry ("Show File Path", showFilePath());
  config -> writeEntry ("Start Playing Immediately", startPlayingImmediately());
  config -> writeEntry ("Show Open Dialog", showOpenDialog());
  config -> writeEntry ("Disable Screen Saver", disableScreenSaver());
  config -> writeEntry ("Allow Duplicate Entries", allowDuplicateEntries());
  config -> writeEntry ("Playlist Size Limit", playlistSizeLimit());
  config -> writeEntry ("Cache Size Limit", cacheSizeLimit());
  config -> writeEntry ("Preferred Slider Length", preferredSliderLength());
  config -> writeEntry ("Minimum Slider Length", minimumSliderLength());
  config -> writeEntry ("Normal Seek", progressNormalSeek());
  config -> writeEntry ("Normal Seek Units", progressNormalSeekUnits());
  config -> writeEntry ("Fast Seek", progressFastSeek());
  config -> writeEntry ("Fast Seek Units", progressFastSeekUnits());
  config -> writeEntry ("Progress Marks", progressMarks());
  config -> writeEntry ("Volume Minimum", volumeMinimum());
  config -> writeEntry ("Volume Maximum", volumeMaximum());
  config -> writeEntry ("Volume Marks", volumeMarks());
  config -> writeEntry ("Volume Step", volumeStep());
  config -> writeEntry ("Volume Default", volumeDefault());
  config -> writeEntry ("Volume Reset", volumeReset());
  config -> writeEntry ("Volume Every", volumeEvery());
  config -> writeEntry ("Contrast Minimum", contrastMinimum());
  config -> writeEntry ("Contrast Maximum", contrastMaximum());
  config -> writeEntry ("Contrast Marks", contrastMarks());
  config -> writeEntry ("Contrast Step", contrastStep());
  config -> writeEntry ("Contrast Default", contrastDefault());
  config -> writeEntry ("Contrast Reset", contrastReset());
  config -> writeEntry ("Contrast Every", contrastEvery());
  config -> writeEntry ("Brightness Minimum", brightnessMinimum());
  config -> writeEntry ("Brightness Maximum", brightnessMaximum());
  config -> writeEntry ("Brightness Marks", brightnessMarks());
  config -> writeEntry ("Brightness Step", brightnessStep());
  config -> writeEntry ("Brightness Default", brightnessDefault());
  config -> writeEntry ("Brightness Reset", brightnessReset());
  config -> writeEntry ("Brightness Every", brightnessEvery());
  config -> writeEntry ("Hue Minimum", hueMinimum());
  config -> writeEntry ("Hue Maximum", hueMaximum());
  config -> writeEntry ("Hue Marks", hueMarks());
  config -> writeEntry ("Hue Step", hueStep());
  config -> writeEntry ("Hue Default", hueDefault());
  config -> writeEntry ("Hue Reset", hueReset());
  config -> writeEntry ("Hue Every", hueEvery());
  config -> writeEntry ("Saturation Minimum", saturationMinimum());
  config -> writeEntry ("Saturation Maximum", saturationMaximum());
  config -> writeEntry ("Saturation Marks", saturationMarks());
  config -> writeEntry ("Saturation Step", saturationStep());
  config -> writeEntry ("Saturation Default", saturationDefault());
  config -> writeEntry ("Saturation Reset", saturationReset());
  config -> writeEntry ("Saturation Every", saturationEvery());
  config -> writeEntry ("Video Driver", videoDriver());
  config -> writeEntry ("Video Driver Fallback", videoDriverFallback());
  config -> writeEntry ("Video Device", videoDevice());
  config -> writeEntry ("Video Codec", videoCodec());
  config -> writeEntry ("Video Codec Fallback", videoCodecFallback());
  config -> writeEntry ("Video Scaler", videoScaler());
  config -> writeEntry ("Enable Doublebuffering", videoDoublebuffering());
  config -> writeEntry ("Enable Direct Rendering", videoDirectRendering());
  config -> writeEntry ("Audio Driver", audioDriver());
  config -> writeEntry ("Audio Driver Fallback", audioDriverFallback());
  config -> writeEntry ("Audio Device", audioDevice());
  config -> writeEntry ("Audio Codec", audioCodec());
  config -> writeEntry ("Audio Codec Fallback", audioCodecFallback());
  config -> writeEntry ("Autoload Subtitles", autoloadSubtitles());
  config -> writeEntry ("Autoload Aqt Subtitles", autoloadAqtSubtitles());
  config -> writeEntry ("Autoload Jss Subtitles", autoloadJssSubtitles());
  config -> writeEntry ("Autoload Rt Subtitles", autoloadRtSubtitles());
  config -> writeEntry ("Autoload Smi Subtitles", autoloadSmiSubtitles());
  config -> writeEntry ("Autoload Srt Subtitles", autoloadSrtSubtitles());
  config -> writeEntry ("Autoload Ssa Subtitles", autoloadSsaSubtitles());
  config -> writeEntry ("Autoload Sub Subtitles", autoloadSubSubtitles());
  config -> writeEntry ("Autoload Txt Subtitles", autoloadTxtSubtitles());
  config -> writeEntry ("Autoload Utf Subtitles", autoloadUtfSubtitles());
  config -> writeEntry ("Autoload Other Subtitles", autoloadOtherSubtitles());
  config -> writeEntry ("Autoload Extension List", autoloadExtensionList());
  config -> writeEntry ("Show Messages On Error", showMessagesOnError());
  config -> writeEntry ("Show Messages Before Playing", showMessagesBeforePlaying());
  config -> writeEntry ("Show Messages After Playing", showMessagesAfterPlaying());
  config -> writeEntry ("Clear Messages Before Playing", clearMessagesBeforePlaying());
  config -> writeEntry ("Executable Path", executablePath());
  config -> writeEntry ("Command Line", commandLine());
  config -> writeEntry ("Use Cache", useCache());
  config -> writeEntry ("Cache Size", cacheSize());
  config -> writeEntry ("Build Index", buildNewIndex());
  config -> writeEntry ("OSD Level", osdLevel());
  emit saved();
}

void KPlayerSettings::refresh (QString path)
{
  if ( path.isEmpty() )
    path = m_executable_path;
  if ( path == m_path )
    return;
  m_path = path;
#ifdef DEBUG_KPLAYER_SETTINGS
  kdDebug() << "KPlayerSettings::refresh (" << path << ")\n";
#endif
  m_audio_codecs_ready = m_audio_drivers_ready = m_video_codecs_ready = m_video_drivers_ready = false;
  m_player_ac = new KPlayerLineOutputProcess;
  *m_player_ac << path << "-ac" << "help";
  QApplication::connect (m_player_ac, SIGNAL (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)),
    this, SLOT (receivedAudioCodec (KPlayerLineOutputProcess*, char*, int)));
  QApplication::connect (m_player_ac, SIGNAL (processExited (KProcess*)),
    this, SLOT (audioCodecProcessExited (KProcess*)));
  m_player_ac -> start (KProcess::NotifyOnExit, KProcess::All);
  m_player_ao = new KPlayerLineOutputProcess;
  *m_player_ao << path << "-ao" << "help";
  QApplication::connect (m_player_ao, SIGNAL (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)),
    this, SLOT (receivedAudioDriver (KPlayerLineOutputProcess*, char*, int)));
  QApplication::connect (m_player_ao, SIGNAL (processExited (KProcess*)),
    this, SLOT (audioDriverProcessExited (KProcess*)));
  m_player_ao -> start (KProcess::NotifyOnExit, KProcess::All);
  m_player_vc = new KPlayerLineOutputProcess;
  *m_player_vc << path << "-vc" << "help";
  QApplication::connect (m_player_vc, SIGNAL (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)),
    this, SLOT (receivedVideoCodec (KPlayerLineOutputProcess*, char*, int)));
  QApplication::connect (m_player_vc, SIGNAL (processExited (KProcess*)),
    this, SLOT (videoCodecProcessExited (KProcess*)));
  m_player_vc -> start (KProcess::NotifyOnExit, KProcess::All);
  m_player_vo = new KPlayerLineOutputProcess;
  *m_player_vo << path << "-vo" << "help";
  QApplication::connect (m_player_vo, SIGNAL (receivedStdoutLine (KPlayerLineOutputProcess*, char*, int)),
    this, SLOT (receivedVideoDriver (KPlayerLineOutputProcess*, char*, int)));
  QApplication::connect (m_player_vo, SIGNAL (processExited (KProcess*)),
    this, SLOT (videoDriverProcessExited (KProcess*)));
  m_player_vo -> start (KProcess::NotifyOnExit, KProcess::All);
}

void KPlayerSettings::receivedAudioCodec (KPlayerLineOutputProcess*, char* str, int)
{
  if ( ! m_audio_codecs_ready && strcmp (str, "Available audio codecs:") == 0 )
  {
    m_audio_codecs.clear();
    m_audio_codecs_ready = true;
    return;
  }
  if ( ! m_audio_codecs_ready || re_codec.search (str) < 0 )
    return;
#ifdef DEBUG_KPLAYER_SETTINGS
  kdDebug() << str << "\n";
#endif
  QString desc (re_codec.cap(2));
  desc.replace (re_multiple, " ");
  desc.replace (re_trailing, "");
  desc.replace (re_brackets, "");
  desc.replace (re_parentheses, "");
  desc.replace (re_audio, "");
  desc.replace (re_s, "");
  desc.replace (re_layer, "layer ");
  desc.replace (re_macintosh, "Macintosh Audio Comp. and Exp.");
  m_audio_codecs += re_codec.cap(1) + ": " + desc;
}

void KPlayerSettings::audioCodecProcessExited (KProcess* proc)
{
  if ( proc == m_player_ac )
    m_player_ac = 0;
  delete proc;
  if ( (m_audio_codecs_ready || m_audio_drivers_ready || m_video_codecs_ready || m_video_drivers_ready)
      && ! m_player_ac && ! m_player_ao && ! m_player_vc && ! m_player_vo )
    emit refreshed();
}

void KPlayerSettings::receivedAudioDriver (KPlayerLineOutputProcess*, char* str, int)
{
  if ( ! m_audio_drivers_ready && strcmp (str, "Available audio output drivers:") == 0 )
  {
    m_audio_drivers.clear();
    m_audio_drivers_ready = true;
    return;
  }
  if ( ! m_audio_drivers_ready || re_driver.search (str) < 0 )
    return;
#ifdef DEBUG_KPLAYER_SETTINGS
  kdDebug() << str << "\n";
#endif
  QString desc (re_driver.cap(2));
  desc.replace (re_multiple, " ");
  desc.replace (re_trailing, "");
  desc.replace (re_brackets, "");
  desc.replace (re_parentheses, "");
  desc.replace (re_audio, "");
  desc.replace (re_s, "");
  desc.replace (re_layer, "layer ");
  m_audio_drivers += re_driver.cap(1) + ": " + desc;
}

void KPlayerSettings::audioDriverProcessExited (KProcess* proc)
{
  if ( proc == m_player_ao )
    m_player_ao = 0;
  delete proc;
  if ( (m_audio_codecs_ready || m_audio_drivers_ready || m_video_codecs_ready || m_video_drivers_ready)
      && ! m_player_ac && ! m_player_ao && ! m_player_vc && ! m_player_vo )
    emit refreshed();
}

void KPlayerSettings::receivedVideoCodec (KPlayerLineOutputProcess*, char* str, int)
{
  if ( ! m_video_codecs_ready && strcmp (str, "Available video codecs:") == 0 )
  {
    m_video_codecs.clear();
    m_video_codecs_ready = true;
    return;
  }
  if ( ! m_video_codecs_ready || re_codec.search (str) < 0 )
    return;
#ifdef DEBUG_KPLAYER_SETTINGS
  kdDebug() << str << "\n";
#endif
  QString desc (re_codec.cap(2));
  desc.replace (re_multiple, " ");
  desc.replace (re_trailing, "");
  desc.replace (re_brackets, "");
  desc.replace (re_parentheses, "");
  desc.replace (re_video, "");
  desc.replace (re_s, "");
  desc.replace (re_layer, "layer ");
  m_video_codecs += re_codec.cap(1) + ": " + desc;
}

void KPlayerSettings::videoCodecProcessExited (KProcess* proc)
{
  if ( proc == m_player_vc )
    m_player_vc = 0;
  delete proc;
  if ( (m_audio_codecs_ready || m_audio_drivers_ready || m_video_codecs_ready || m_video_drivers_ready)
      && ! m_player_ac && ! m_player_ao && ! m_player_vc && ! m_player_vo )
    emit refreshed();
}

void KPlayerSettings::receivedVideoDriver (KPlayerLineOutputProcess*, char* str, int)
{
  if ( ! m_video_drivers_ready && strcmp (str, "Available video output drivers:") == 0 )
  {
    m_video_drivers.clear();
    m_video_drivers_ready = true;
    return;
  }
  if ( ! m_video_drivers_ready || re_driver.search (str) < 0 )
    return;
#ifdef DEBUG_KPLAYER_SETTINGS
  kdDebug() << str << "\n";
#endif
  QString desc (re_driver.cap(2));
  desc.replace (re_multiple, " ");
  desc.replace (re_trailing, "");
  desc.replace (re_xv, "/XVideo");
  desc.replace (re_vidix, "/VIDIX");
  desc.replace (re_matrox, "Matrox overlay in");
  desc.replace (re_brackets, "");
  desc.replace (re_parentheses, "");
  desc.replace (re_video, "");
  desc.replace (re_s, "");
  desc.replace (re_layer, "layer ");
  m_video_drivers += re_driver.cap(1) + ": " + desc;
}

void KPlayerSettings::videoDriverProcessExited (KProcess* proc)
{
  if ( proc == m_player_vo )
    m_player_vo = 0;
  delete proc;
  if ( (m_audio_codecs_ready || m_audio_drivers_ready || m_video_codecs_ready || m_video_drivers_ready)
      && ! m_player_ac && ! m_player_ao && ! m_player_vc && ! m_player_vo )
    emit refreshed();
}

int listIndex (const QStringList& sl, const QString& str)
{
  QString entry, stru (str.upper()), strusc (stru + ":");
  int index = 0;
  for ( QStringList::ConstIterator sli = sl.begin(); sli != sl.end(); ++ sli )
  {
    entry = (*sli).upper();
    if ( entry == stru || entry.startsWith (strusc) )
      return index;
    index ++;
  }
  return -1;
}

QString timeString (float l, bool zero_ok)
{
  l += 0.02;
  if ( ! zero_ok && l < 0.1 )
    return QString ("");
  int lHour = int (l) / 3600;
  if ( lHour > 0 )
    l -= lHour * 3600;
  int lMinute = int (l) / 60;
  if ( lMinute > 0 )
    l -= lMinute * 60;
  QString s;
  if ( lHour > 0 )
    s.sprintf ("%u:%02u:%04.1f", lHour, lMinute, l);
  else if ( lMinute > 0 )
    s.sprintf ("%u:%04.1f", lMinute, l);
  else
    s.sprintf ("%03.1f", l);
  return s;
}
