/***************************************************************************
                          kplayersettings.h
                          -----------------
    begin                : Tue Apr 22 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef KPLAYERSETTINGS_H
#define KPLAYERSETTINGS_H

#include <kconfig.h>
#include <limits.h>
#include <qapplication.h>

class KConfig;
class KPlayerLineOutputProcess;
class KPlayerProcess;
class KProcess;

int listIndex (const QStringList&, const QString&);

inline int limit (int value, int minValue, int maxValue = INT_MAX)
{
  return value < minValue ? minValue : value > maxValue ? maxValue : value;
}

inline void limit4 (int& lowerValue, int& higherValue, int minValue, int maxValue)
{
  if ( lowerValue > higherValue )
  {
    int i = lowerValue;
    lowerValue = higherValue;
    higherValue = i;
  }
  lowerValue = limit (lowerValue, minValue, maxValue);
  higherValue = limit (higherValue, minValue, maxValue);
}

/** The KPlayer settings.
  *@author kiriuja
  */

class KPlayerSettings : public QObject
{
  Q_OBJECT

public:
  KPlayerSettings (void);

  void load (void);
  void save (void);
  void defaults (void);
  /** Starts loading of all mplayer drivers and codecs. The refreshed signal is emitted when loading finishes.
   */
  void refresh (QString = QString::null);

  /** Returns the static settings object.
   */
  static KPlayerSettings* kPlayerSettings (void)
    { return m_settings; }
  /** Creates the static settings object.
   */
  static void createKPlayerSettings (void)
  {
    if ( ! m_settings )
      m_settings = new KPlayerSettings;
  }
  /** Deletes the static settings object.
   */
  static void deleteKPlayerSettings (void)
  {
    if ( m_settings )
      delete m_settings;
    m_settings = 0;
  }

  /** Returns the static process object.
   */
  static KPlayerProcess* kPlayerProcess (void)
    { return m_process; }
  /** Sets the static process object.
   */
  static void setKPlayerProcess (KPlayerProcess* process)
    { m_process = process; }

  /** Returns the static config object.
   */
  static KConfig* kPlayerConfig (void)
    { return m_config; }
  /** Sets the static config object.
   */
  static void setKPlayerConfig (KConfig* config)
    { m_config = config; }
  /** Deletes the static config object.
   */
  static void deleteKPlayerConfig (void)
  {
    if ( m_config )
      delete m_config;
    m_config = 0;
  }

  /** Returns the static playlist config object.
   */
  static KConfig* kPlayerPlaylistConfig (void)
    { return m_playlist_config; }
  /** Creates the static playlist config object.
   */
  static void createKPlayerPlaylistConfig (void)
  {
    if ( ! m_playlist_config )
      m_playlist_config = new KConfig ("kplayerplaylistrc");
  }
  /** Deletes the static playlist config object.
   */
  static void deleteKPlayerPlaylistConfig (void)
  {
    if ( m_playlist_config )
      delete m_playlist_config;
    m_playlist_config = 0;
  }

  // General settings

  bool resizeAutomatically (void)
    { return m_resize_automatically; }
  void setResizeAutomatically (bool resize)
    { m_resize_automatically = resize; }

  int minimumInitialWidth (void)
    { return m_minimum_initial_width; }
  void setMinimumInitialWidth (int width)
    { m_minimum_initial_width = limit (width, 200, QApplication::desktop() -> width()); }

  int recentFileListSize (void)
    { return m_recent_file_list_size; }
  void setRecentFileListSize (int size)
    { m_recent_file_list_size = limit (size, 0); }

  bool showFilePath (void)
    { return m_show_file_path; }
  void setShowFilePath (bool show)
    { m_show_file_path = show; }

  bool showOpenDialog (void)
    { return m_show_open_dialog; }
  void setShowOpenDialog (bool show)
    { m_show_open_dialog = show; }

  bool startPlayingImmediately (void)
    { return m_start_playing_immediately; }
  void setStartPlayingImmediately (bool start)
    { m_start_playing_immediately = start; }

  bool disableScreenSaver (void)
    { return m_disable_screen_saver; }
  void setDisableScreenSaver (bool disable)
    {  m_disable_screen_saver = disable; }

  // Playlist settings

  bool allowDuplicateEntries (void)
    { return m_allow_duplicate_entries; }
  void setAllowDuplicateEntries (bool allow)
    { m_allow_duplicate_entries = allow; }

  int playlistSizeLimit (void)
    { return m_playlist_size_limit; }
  void setPlaylistSizeLimit (int size)
    { m_playlist_size_limit = limit (size, 10, 1000); }

  int cacheSizeLimit (void)
    { return m_cache_size_limit; }
  void setCacheSizeLimit (int size)
    { m_cache_size_limit = limit (size, 10, 10000); }

  // Control settings

  int preferredSliderLength (void)
    { return m_preferred_slider_length; }
  void setPreferredSliderLength (int length)
    { m_preferred_slider_length = limit (length, 50, QApplication::desktop() -> width()); }

  int minimumSliderLength (void)
    { return m_minimum_slider_length; }
  void setMinimumSliderLength (int length)
    { m_minimum_slider_length = limit (length, 50, QApplication::desktop() -> width()); }

  // Volume settings

  int volumeMinimum (void)
    { return m_volume_minimum; }
  void setVolumeMinimum (int volume)
    { m_volume_minimum = limit (volume, 0, 100); }

  int volumeMaximum (void)
    { return m_volume_maximum; }
  void setVolumeMaximum (int volume)
    { m_volume_maximum = limit (volume, 0, 100); }

  void setVolumeMinimumMaximum (int minimum, int maximum)
  {
    m_volume_minimum = minimum;
    m_volume_maximum = maximum;
    limit4 (m_volume_minimum, m_volume_maximum, 0, 100);
  }

  int volumeMarks (void)
    { return m_volume_marks; }
  void setVolumeMarks (int volume)
    { m_volume_marks = limit (volume, 1, m_volume_maximum - m_volume_minimum); }

  int volumeStep (void)
    { return m_volume_step; }
  void setVolumeStep (int volume)
    { m_volume_step = limit (volume, 1, m_volume_maximum - m_volume_minimum); }

  bool volumeReset (void)
    { return m_volume_reset; }
  void setVolumeReset (bool volume)
    { m_volume_reset = volume; }

  int volumeEvery (void)
    { return m_volume_every; }
  void setVolumeEvery (int volume)
    { m_volume_every = limit (volume, 0, 1); }

  int volumeDefault (void)
    { return m_volume_default; }
  void setVolumeDefault (int volume)
    { m_volume_default = limit (volume, m_volume_minimum, m_volume_maximum); }

  // Contrast settings

  int contrastMinimum (void)
    { return m_contrast_minimum; }
  void setContrastMinimum (int contrast)
    { m_contrast_minimum = limit (contrast, -100, 100); }

  int contrastMaximum (void)
    { return m_contrast_maximum; }
  void setContrastMaximum (int contrast)
    { m_contrast_maximum = limit (contrast, -100, 100); }

  void setContrastMinimumMaximum (int minimum, int maximum)
  {
    m_contrast_minimum = minimum;
    m_contrast_maximum = maximum;
    limit4 (m_contrast_minimum, m_contrast_maximum, -100, 100);
  }

  int contrastMarks (void)
    { return m_contrast_marks; }
  void setContrastMarks (int contrast)
    { m_contrast_marks = limit (contrast, 1, m_contrast_maximum - m_contrast_minimum); }

  int contrastStep (void)
    { return m_contrast_step; }
  void setContrastStep (int contrast)
    { m_contrast_step = limit (contrast, 1, m_contrast_maximum - m_contrast_minimum); }

  bool contrastReset (void)
    { return m_contrast_reset; }
  void setContrastReset (bool contrast)
    { m_contrast_reset = contrast; }

  int contrastEvery (void)
    { return m_contrast_every; }
  void setContrastEvery (int contrast)
    { m_contrast_every = limit (contrast, 0, 1); }

  int contrastDefault (void)
    { return m_contrast_default; }
  void setContrastDefault (int contrast)
    { m_contrast_default = limit (contrast, m_contrast_minimum, m_contrast_maximum); }

  // Brightness settings

  int brightnessMinimum (void)
    { return m_brightness_minimum; }
  void setBrightnessMinimum (int brightness)
    { m_brightness_minimum = limit (brightness, -100, 100); }

  int brightnessMaximum (void)
    { return m_brightness_maximum; }
  void setBrightnessMaximum (int brightness)
    { m_brightness_maximum = limit (brightness, -100, 100); }

  void setBrightnessMinimumMaximum (int minimum, int maximum)
  {
    m_brightness_minimum = minimum;
    m_brightness_maximum = maximum;
    limit4 (m_brightness_minimum, m_brightness_maximum, -100, 100);
  }

  int brightnessMarks (void)
    { return m_brightness_marks; }
  void setBrightnessMarks (int brightness)
    { m_brightness_marks = limit (brightness, 1, m_brightness_maximum - m_brightness_minimum); }

  int brightnessStep (void)
    { return m_brightness_step; }
  void setBrightnessStep (int brightness)
    { m_brightness_step = limit (brightness, 1, m_brightness_maximum - m_brightness_minimum); }

  bool brightnessReset (void)
    { return m_brightness_reset; }
  void setBrightnessReset (bool brightness)
    { m_brightness_reset = brightness; }

  int brightnessEvery (void)
    { return m_brightness_every; }
  void setBrightnessEvery (int brightness)
    { m_brightness_every = limit (brightness, 0, 1); }

  int brightnessDefault (void)
    { return m_brightness_default; }
  void setBrightnessDefault (int brightness)
    { m_brightness_default = limit (brightness, m_brightness_minimum, m_brightness_maximum); }

  // Hue settings

  int hueMinimum (void)
    { return m_hue_minimum; }
  void setHueMinimum (int hue)
    { m_hue_minimum = limit (hue, -100, 100); }

  int hueMaximum (void)
    { return m_hue_maximum; }
  void setHueMaximum (int hue)
    { m_hue_maximum = limit (hue, -100, 100); }

  void setHueMinimumMaximum (int minimum, int maximum)
  {
    m_hue_minimum = minimum;
    m_hue_maximum = maximum;
    limit4 (m_hue_minimum, m_hue_maximum, -100, 100);
  }

  int hueMarks (void)
    { return m_hue_marks; }
  void setHueMarks (int hue)
    { m_hue_marks = limit (hue, 1, m_hue_maximum - m_hue_minimum); }

  int hueStep (void)
    { return m_hue_step; }
  void setHueStep (int hue)
    { m_hue_step = limit (hue, 1, m_hue_maximum - m_hue_minimum); }

  bool hueReset (void)
    { return m_hue_reset; }
  void setHueReset (bool hue)
    { m_hue_reset = hue; }

  int hueEvery (void)
    { return m_hue_every; }
  void setHueEvery (int hue)
    { m_hue_every = limit (hue, 0, 1); }

  int hueDefault (void)
    { return m_hue_default; }
  void setHueDefault (int hue)
    { m_hue_default = limit (hue, m_hue_minimum, m_hue_maximum); }

  // Saturation settings

  int saturationMinimum (void)
    { return m_saturation_minimum; }
  void setSaturationMinimum (int saturation)
    { m_saturation_minimum = limit (saturation, -100, 100); }

  int saturationMaximum (void)
    { return m_saturation_maximum; }
  void setSaturationMaximum (int saturation)
    { m_saturation_maximum = limit (saturation, -100, 100); }

  void setSaturationMinimumMaximum (int minimum, int maximum)
  {
    m_saturation_minimum = minimum;
    m_saturation_maximum = maximum;
    limit4 (m_saturation_minimum, m_saturation_maximum, -100, 100);
  }

  int saturationMarks (void)
    { return m_saturation_marks; }
  void setSaturationMarks (int saturation)
    { m_saturation_marks = limit (saturation, 1, m_saturation_maximum - m_saturation_minimum); }

  int saturationStep (void)
    { return m_saturation_step; }
  void setSaturationStep (int saturation)
    { m_saturation_step = limit (saturation, 1, m_saturation_maximum - m_saturation_minimum); }

  bool saturationReset (void)
    { return m_saturation_reset; }
  void setSaturationReset (bool saturation)
    { m_saturation_reset = saturation; }

  int saturationEvery (void)
    { return m_saturation_every; }
  void setSaturationEvery (int saturation)
    { m_saturation_every = limit (saturation, 0, 1); }

  int saturationDefault (void)
    { return m_saturation_default; }
  void setSaturationDefault (int saturation)
    { m_saturation_default = limit (saturation, m_saturation_minimum, m_saturation_maximum); }

  // Progress and seeking settings

  int progressNormalSeek (void)
    { return m_progress_normal_seek; }
  void setProgressNormalSeek (int seek)
    { m_progress_normal_seek = limit (seek, 1, m_progress_normal_seek_units ? INT_MAX : 100); }

  int progressNormalSeekUnits (void)
    { return m_progress_normal_seek_units; }
  void setProgressNormalSeekUnits (int units)
    { m_progress_normal_seek_units = limit (units, 0, 1); }

  int progressFastSeek (void)
    { return m_progress_fast_seek; }
  void setProgressFastSeek (int seek)
    { m_progress_fast_seek = limit (seek, m_progress_normal_seek, m_progress_fast_seek_units ? INT_MAX : 100); }

  int progressFastSeekUnits (void)
    { return m_progress_fast_seek_units; }
  void setProgressFastSeekUnits (int units)
    { m_progress_fast_seek_units = limit (units, 0, 1); }

  int progressMarks (void)
    { return m_progress_marks; }
  void setProgressMarks (int marks)
    { m_progress_marks = limit (marks, 1, 100); }

  // Audio settings

  QString audioDriver (void)
    { return m_audio_driver; }
  void setAudioDriver (QString driver)
    { m_audio_driver = driver; }

  bool audioDriverFallback (void)
    { return m_audio_driver_fallback; }
  void setAudioDriverFallback (bool fallback)
    { m_audio_driver_fallback = fallback; }

  QString audioDevice (void)
    { return m_audio_device; }
  void setAudioDevice (QString device)
    { m_audio_device = device; }

  QString audioCodec (void)
    { return m_audio_codec; }
  void setAudioCodec (QString codec)
    { m_audio_codec = codec; }

  bool audioCodecFallback (void)
    { return m_audio_codec_fallback; }
  void setAudioCodecFallback (bool fallback)
    { m_audio_codec_fallback = fallback; }

  int audioCodecCount (void)
    { return m_audio_codecs.count(); }
  QString audioCodecName (int index)
    { return m_audio_codecs[index]; }
  int audioCodecIndex (QString codec = QString::null)
    { return listIndex (m_audio_codecs, codec.isNull() ? m_audio_codec : codec) + 1; }

  int audioDriverCount (void)
    { return m_audio_drivers.count(); }
  QString audioDriverName (int index)
    { return m_audio_drivers[index]; }
  int audioDriverIndex (QString driver = QString::null)
    { return listIndex (m_audio_drivers, driver.isNull() ? m_audio_driver : driver) + 1; }

  // Video settings

  QString videoDriver (void)
    { return m_video_driver; }
  void setVideoDriver (QString driver)
    { m_video_driver = driver; }

  bool videoDriverFallback (void)
    { return m_video_driver_fallback; }
  void setVideoDriverFallback (bool fallback)
    { m_video_driver_fallback = fallback; }

  QString videoDevice (void)
    { return m_video_device; }
  void setVideoDevice (QString device)
    { m_video_device = device; }

  QString videoCodec (void)
    { return m_video_codec; }
  void setVideoCodec (QString codec)
    { m_video_codec = codec; }

  bool videoCodecFallback (void)
    { return m_video_codec_fallback; }
  void setVideoCodecFallback (bool fallback)
    { m_video_codec_fallback = fallback; }

  int videoScaler (void)
    { return m_video_scaler; }
  void setVideoScaler (int scaler)
    { m_video_scaler = limit (scaler, 0, 10); }

  bool videoDoublebuffering (void)
    { return m_video_doublebuffering; }
  void setVideoDoublebuffering (bool doublebuffering)
    { m_video_doublebuffering = doublebuffering; }

  bool videoDirectRendering (void)
    { return m_video_direct_rendering; }
  void setVideoDirectRendering (bool directrendering)
    { m_video_direct_rendering = directrendering; }

  int videoCodecCount (void)
    { return m_video_codecs.count(); }
  QString videoCodecName (int index)
    { return m_video_codecs[index]; }
  int videoCodecIndex (QString codec = QString::null)
    { return listIndex (m_video_codecs, codec.isNull() ? m_video_codec : codec) + 1; }

  int videoDriverCount (void)
    { return m_video_drivers.count(); }
  QString videoDriverName (int index)
    { return m_video_drivers[index]; }
  int videoDriverIndex (QString driver = QString::null)
    { return listIndex (m_video_drivers, driver.isNull() ? m_video_driver : driver) + 1; }

  // Subtitle settings

  bool autoloadSubtitles (void)
    { return m_autoload_subtitles; }
  void setAutoloadSubtitles (bool autoload)
    { m_autoload_subtitles = autoload; }

  bool autoloadAqtSubtitles (void)
    { return m_autoload_aqt_subtitles; }
  void setAutoloadAqtSubtitles (bool autoload)
    { m_autoload_aqt_subtitles = autoload; }

  bool autoloadJssSubtitles (void)
    { return m_autoload_jss_subtitles; }
  void setAutoloadJssSubtitles (bool autoload)
    { m_autoload_jss_subtitles = autoload; }

  bool autoloadRtSubtitles (void)
    { return m_autoload_rt_subtitles; }
  void setAutoloadRtSubtitles (bool autoload)
    { m_autoload_rt_subtitles = autoload; }

  bool autoloadSmiSubtitles (void)
    { return m_autoload_smi_subtitles; }
  void setAutoloadSmiSubtitles (bool autoload)
    { m_autoload_smi_subtitles = autoload; }

  bool autoloadSrtSubtitles (void)
    { return m_autoload_srt_subtitles; }
  void setAutoloadSrtSubtitles (bool autoload)
    { m_autoload_srt_subtitles = autoload; }

  bool autoloadSsaSubtitles (void)
    { return m_autoload_ssa_subtitles; }
  void setAutoloadSsaSubtitles (bool autoload)
    { m_autoload_ssa_subtitles = autoload; }

  bool autoloadSubSubtitles (void)
    { return m_autoload_sub_subtitles; }
  void setAutoloadSubSubtitles (bool autoload)
    { m_autoload_sub_subtitles = autoload; }

  bool autoloadTxtSubtitles (void)
    { return m_autoload_txt_subtitles; }
  void setAutoloadTxtSubtitles (bool autoload)
    { m_autoload_txt_subtitles = autoload; }

  bool autoloadUtfSubtitles (void)
    { return m_autoload_utf_subtitles; }
  void setAutoloadUtfSubtitles (bool autoload)
    { m_autoload_utf_subtitles = autoload; }

  bool autoloadOtherSubtitles (void)
    { return m_autoload_other_subtitles; }
  void setAutoloadOtherSubtitles (bool autoload)
    { m_autoload_other_subtitles = autoload; }

  QString autoloadExtensionList (void)
    { return m_autoload_extension_list; }
  void setAutoloadExtensionList (QString autoload)
    { m_autoload_extension_list = autoload; }

  // Message settings

  bool showMessagesOnError (void)
    { return m_show_messages_on_error; }
  void setShowMessagesOnError (bool show)
    { m_show_messages_on_error = show; }

  bool showMessagesBeforePlaying (void)
    { return m_show_messages_before_playing; }
  void setShowMessagesBeforePlaying (bool show)
    { m_show_messages_before_playing = show; }

  bool showMessagesAfterPlaying (void)
    { return m_show_messages_after_playing; }
  void setShowMessagesAfterPlaying (bool show)
    { m_show_messages_after_playing = show; }

  bool clearMessagesBeforePlaying (void)
    { return m_clear_messages_before_playing; }
  void setClearMessagesBeforePlaying (bool clear)
    { m_clear_messages_before_playing = clear; }

  // Advanced settings

  QString executablePath (void)
    { return m_executable_path; }
  void setExecutablePath (QString path)
  {
    if ( path.isEmpty() )
      m_executable_path = "mplayer";
    else
      m_executable_path = path;
  }

  QString commandLine (void)
    { return m_command_line; }
  void setCommandLine (QString commandline)
    { m_command_line = commandline; }

  bool useCache (void)
    { return m_use_cache; }
  void setUseCache (bool use)
    { m_use_cache = use; }

  int cacheSize (void)
    { return m_cache_size; }
  void setCacheSize (int size)
    { m_cache_size = limit (size, 0); }

  bool buildNewIndex (void)
    { return m_build_new_index; }
  void setBuildNewIndex (bool build)
    { m_build_new_index = build; }

  int osdLevel (void)
    { return m_osd_level; }
  void setOsdLevel (int level)
    { m_osd_level = limit (level, 0, 3); }

protected:
  bool m_resize_automatically;
  int m_minimum_initial_width;
  int m_recent_file_list_size;
  bool m_show_file_path;
  bool m_show_open_dialog;
  bool m_start_playing_immediately;
  bool m_disable_screen_saver;
  bool m_allow_duplicate_entries;
  int m_playlist_size_limit;
  int m_cache_size_limit;
  int m_preferred_slider_length;
  int m_minimum_slider_length;
  int m_volume_minimum;
  int m_volume_maximum;
  int m_volume_marks;
  int m_volume_step;
  bool m_volume_reset;
  int m_volume_every;
  int m_volume_default;
  int m_contrast_minimum;
  int m_contrast_maximum;
  int m_contrast_marks;
  int m_contrast_step;
  bool m_contrast_reset;
  int m_contrast_every;
  int m_contrast_default;
  int m_brightness_minimum;
  int m_brightness_maximum;
  int m_brightness_marks;
  int m_brightness_step;
  bool m_brightness_reset;
  int m_brightness_every;
  int m_brightness_default;
  int m_hue_minimum;
  int m_hue_maximum;
  int m_hue_marks;
  int m_hue_step;
  bool m_hue_reset;
  int m_hue_every;
  int m_hue_default;
  int m_saturation_minimum;
  int m_saturation_maximum;
  int m_saturation_marks;
  int m_saturation_step;
  bool m_saturation_reset;
  int m_saturation_every;
  int m_saturation_default;
  int m_progress_normal_seek;
  int m_progress_normal_seek_units;
  int m_progress_fast_seek;
  int m_progress_fast_seek_units;
  int m_progress_marks;
  QString m_audio_driver;
  bool m_audio_driver_fallback;
  QString m_audio_device;
  QString m_audio_codec;
  bool m_audio_codec_fallback;
  QString m_video_driver;
  bool m_video_driver_fallback;
  QString m_video_device;
  QString m_video_codec;
  bool m_video_codec_fallback;
  int m_video_scaler;
  bool m_video_doublebuffering;
  bool m_video_direct_rendering;
  bool m_autoload_subtitles;
  bool m_autoload_aqt_subtitles;
  bool m_autoload_jss_subtitles;
  bool m_autoload_rt_subtitles;
  bool m_autoload_smi_subtitles;
  bool m_autoload_srt_subtitles;
  bool m_autoload_ssa_subtitles;
  bool m_autoload_sub_subtitles;
  bool m_autoload_txt_subtitles;
  bool m_autoload_utf_subtitles;
  bool m_autoload_other_subtitles;
  QString m_autoload_extension_list;
  bool m_show_messages_on_error;
  bool m_show_messages_before_playing;
  bool m_show_messages_after_playing;
  bool m_clear_messages_before_playing;
  QString m_executable_path;
  QString m_command_line;
  bool m_use_cache;
  int m_cache_size;
  bool m_build_new_index;
  int m_osd_level;

  KPlayerLineOutputProcess* m_player_ac;
  KPlayerLineOutputProcess* m_player_ao;
  KPlayerLineOutputProcess* m_player_vc;
  KPlayerLineOutputProcess* m_player_vo;
  bool m_audio_codecs_ready;
  bool m_audio_drivers_ready;
  bool m_video_codecs_ready;
  bool m_video_drivers_ready;
  QStringList m_audio_codecs;
  QStringList m_audio_drivers;
  QStringList m_video_codecs;
  QStringList m_video_drivers;

  QString m_path;

  /** The static settings object.
   */
  static KPlayerSettings* m_settings;
  /** The static config object.
   */
  static KConfig* m_config;
  /** The static playlist config object.
   */
  static KConfig* m_playlist_config;
  /** The static process object.
   */
  static KPlayerProcess* m_process;

signals:
  /** Emitted when all drivers and codecs have been loaded.
   */
  void refreshed (void);
  /** Emitted when the settings have been saved.
   */
  void saved (void);

protected slots:
  /** Processes an MPlayer audio codec line.
    */
  void receivedAudioCodec (KPlayerLineOutputProcess*, char*, int);
  /** Processes an MPlayer audio driver line.
    */
  void receivedAudioDriver (KPlayerLineOutputProcess*, char*, int);
  /** Processes an MPlayer video codec line.
    */
  void receivedVideoCodec (KPlayerLineOutputProcess*, char*, int);
  /** Processes an MPlayer video driver line.
    */
  void receivedVideoDriver (KPlayerLineOutputProcess*, char*, int);
  /** Finishes refreshing audio codecs.
    */
  void audioCodecProcessExited (KProcess*);
  /** Finishes refreshing audio drivers.
    */
  void audioDriverProcessExited (KProcess*);
  /** Finishes refreshing video codecs.
    */
  void videoCodecProcessExited (KProcess*);
  /** Finishes refreshing video drivers.
    */
  void videoDriverProcessExited (KProcess*);
};

inline KPlayerProcess* kPlayerProcess (void)
{
  return KPlayerSettings::kPlayerProcess();
}

inline KConfig* kPlayerConfig (void)
{
  return KPlayerSettings::kPlayerConfig();
}

inline KConfig* kPlayerPlaylistConfig (void)
{
  return KPlayerSettings::kPlayerPlaylistConfig();
}

inline KPlayerSettings* kPlayerSettings (void)
{
  return KPlayerSettings::kPlayerSettings();
}

/** Returns the given time length as string.
  */
QString timeString (float length, bool zero_ok = false);

#endif
