/***************************************************************************
                          kplayersettingsdialog.cpp
                          -------------------------
    begin                : Tue Apr 22 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

//#include <kapplication.h>
//#include <kconfig.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qregexp.h>
#include <stdlib.h>

#include <kdebug.h>

#include "kplayersettings.h"
#include "kplayersettingsdialog.h"
#include "kplayersettingsdialog.moc"

static QRegExp re_key_value ("([^:]+): *(.*)");

QString listEntry (QString entry)
{
  if ( entry == i18n("auto") )
    entry = "";
  else if ( re_key_value.search (entry) >= 0 )
    entry = re_key_value.cap(1);
  return entry;
}

KPlayerSettingsDialog::KPlayerSettingsDialog (void)
  : KDialogBase (TreeList, i18n("KPlayer Preferences"), Default|Ok|Apply|Cancel, Ok) // Help|
{
  QApplication::connect (this, SIGNAL (aboutToShowPage (QWidget*)), this, SLOT (pageAboutToShow()));
  QFrame* frame = addPage (i18n("General"), i18n("General Settings"));
  QVBoxLayout* layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_general = new KPlayerSettingsGeneral (frame);
  frame = addPage (i18n("Playlist"), i18n("Playlist Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_playlist = new KPlayerSettingsPlaylist (frame);
  frame = addPage (i18n("Controls"), i18n("Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_controls = new KPlayerSettingsControls (frame);
  frame = addPage (i18n("Video"), i18n("Video Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_video = new KPlayerSettingsVideo (frame);
  frame = addPage (i18n("Audio"), i18n("Audio Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_audio = new KPlayerSettingsAudio (frame);
  frame = addPage (i18n("Subtitles"), i18n("Subtitle Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_subtitles = new KPlayerSettingsSubtitles (frame);
  frame = addPage (i18n("Messages"), i18n("Message Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_messages = new KPlayerSettingsMessages (frame);
  frame = addPage (i18n("Advanced"), i18n("Advanced Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_advanced = new KPlayerSettingsAdvanced (frame);
  frame = addPage (QStringList() << i18n("Controls") << i18n("Progress"), i18n("Progress Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_progress = new KPlayerSettingsProgress (frame);
  frame = addPage (QStringList() << i18n("Controls") << i18n("Volume"), i18n("Volume Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_volume = new KPlayerSettingsVolume (frame);
  frame = addPage (QStringList() << i18n("Controls") << i18n("Contrast"), i18n("Contrast Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_contrast = new KPlayerSettingsContrast (frame);
  frame = addPage (QStringList() << i18n("Controls") << i18n("Brightness"), i18n("Brightness Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_brightness = new KPlayerSettingsBrightness (frame);
  frame = addPage (QStringList() << i18n("Controls") << i18n("Hue"), i18n("Hue Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_hue = new KPlayerSettingsHue (frame);
  frame = addPage (QStringList() << i18n("Controls") << i18n("Saturation"), i18n("Saturation Control Settings"));
  layout = new QVBoxLayout (frame, 0, 0);
  layout -> setAutoAdd (true);
  m_saturation = new KPlayerSettingsSaturation (frame);
  KConfig* config = kPlayerConfig();
  config -> setGroup ("General Options");
//QSize hint = minimumSizeHint();
  QSize size (config -> readNumEntry ("Settings Dialog Width"),
    config -> readNumEntry ("Settings Dialog Height"));
//if ( size.width() < hint.width() || size.height() < hint.height() )
//  size = sizeHint();
//kdDebug() << "KPSD Size " << size.width() << "x" << size.height() << " Minimum Size Hint " << hint.width() << "x" << hint.height() << "\n";
//int x = config -> readNumEntry ("Settings Dialog Left");
//int y = config -> readNumEntry ("Settings Dialog Top");
//setGeometry (x, y, size.width(), size.height());
  resize (size);
}

KPlayerSettingsDialog::~KPlayerSettingsDialog (void)
{
  kPlayerSettings() -> refresh();
  KConfig* config = kPlayerConfig();
  config -> setGroup ("General Options");
//config -> writeEntry ("Settings Dialog Left", x());
//config -> writeEntry ("Settings Dialog Top", y());
//kdDebug() << "KPSD " << x() << "x" << y() << " " << width() << "x" << height() << " Hint " << sizeHint().width() << "x" << sizeHint().height() << "\n";
/*if ( size() == sizeHint() )
  {
    config -> deleteEntry ("Settings Dialog Width");
    config -> deleteEntry ("Settings Dialog Height");
  }
  else
  {*/
  config -> writeEntry ("Settings Dialog Width", width());
  config -> writeEntry ("Settings Dialog Height", height());
}

void KPlayerSettingsDialog::slotDefault (void)
{
  if ( KMessageBox::warningYesNo (this, i18n("All configuration settings will be reset.\n\nAre you sure?"))
      != KMessageBox::Yes )
    return;
  kPlayerSettings() -> defaults();
  kPlayerSettings() -> save();
  m_general -> load();
  m_playlist -> load();
  m_controls -> load();
  m_video -> load();
  m_audio -> load();
  m_subtitles -> load();
  m_messages -> load();
  m_advanced -> load();
  m_progress -> load();
  m_volume -> load();
  m_contrast -> load();
  m_brightness -> load();
  m_hue -> load();
  m_saturation -> load();
  setButtonCancelText (i18n("&Close"));
  KDialogBase::slotDefault();
}

void KPlayerSettingsDialog::pageAboutToShow (void)
{
  m_advanced -> refresh();
}

void KPlayerSettingsDialog::slotOk (void)
{
  slotApply();
  KDialogBase::slotOk();
}

void KPlayerSettingsDialog::slotApply (void)
{
  m_advanced -> save();
  m_audio -> save();
  m_controls -> save();
  m_general -> save();
  m_playlist -> save();
  m_messages -> save();
  m_subtitles -> save();
  m_video -> save();
  m_progress -> save();
  m_volume -> save();
  m_contrast -> save();
  m_brightness -> save();
  m_hue -> save();
  m_saturation -> save();
  kPlayerSettings() -> save();
  setButtonCancelText (i18n("&Close"));
  KDialogBase::slotApply();
}

KPlayerSettingsAdvanced::KPlayerSettingsAdvanced (QWidget* parent, const char* name)
  : KPlayerSettingsAdvancedPage (parent, name)
{
  load();
}

void KPlayerSettingsAdvanced::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_executable_path -> setText (settings -> executablePath());
  c_command_line -> setText (settings -> commandLine());
  c_use_cache -> setChecked (settings -> useCache());
  useCacheChanged (c_use_cache -> isChecked());
  c_build_new_index -> setChecked (settings -> buildNewIndex());
  c_osd_level -> setCurrentItem (settings -> osdLevel());
}

void KPlayerSettingsAdvanced::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setExecutablePath (c_executable_path -> text());
  settings -> setCommandLine (c_command_line -> text());
  settings -> setUseCache (c_use_cache -> isChecked());
  if ( c_use_cache -> isChecked() )
    settings -> setCacheSize (labs (atol (c_cache_size -> text())));
  settings -> setBuildNewIndex (c_build_new_index -> isChecked());
  settings -> setOsdLevel (c_osd_level -> currentItem());
}

void KPlayerSettingsAdvanced::refresh (void)
{
  kPlayerSettings() -> refresh (c_executable_path -> text());
}

void KPlayerSettingsAdvanced::useCacheChanged (bool useCacheChecked)
{
  if ( useCacheChecked )
    c_cache_size -> setText (QString().setNum (kPlayerSettings() -> cacheSize()));
  else
    c_cache_size -> setText ("");
  l_cache_size -> setEnabled (useCacheChecked);
  c_cache_size -> setEnabled (useCacheChecked);
  l_cache_size_kb -> setEnabled (useCacheChecked);
  if ( useCacheChecked && sender() )
  {
    c_cache_size -> setFocus();
    c_cache_size -> selectAll();
  }
}

KPlayerSettingsAudio::KPlayerSettingsAudio (QWidget* parent, const char* name)
  : KPlayerSettingsAudioPage (parent, name)
{
  loadLists();
  load();
  QApplication::connect (kPlayerSettings(), SIGNAL (refreshed()), this, SLOT (refresh()));
}

void KPlayerSettingsAudio::refresh (void)
{
  QString driver (listEntry (c_audio_driver -> currentText()));
  QString codec (listEntry (c_audio_codec -> currentText()));
  loadLists();
  KPlayerSettings* settings = kPlayerSettings();
  int index = settings -> audioDriverIndex (driver);
  kdDebug() << "Audio Driver: " << driver << " => " << index << " / " << settings -> audioDriverCount() << "\n";
  c_audio_driver -> setCurrentItem (index);
  if ( ! driver.isEmpty() && index == 0 )
    driverChanged (index);
  index = settings -> audioCodecIndex (codec);
  kdDebug() << "Audio Codec: " << codec << " => " << index << " / " << settings -> audioCodecCount() << "\n";
  c_audio_codec -> setCurrentItem (index);
  if ( ! codec.isEmpty() && index == 0 )
    codecChanged (index);
}

void KPlayerSettingsAudio::loadLists (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( settings -> audioCodecCount() )
  {
    c_audio_codec -> clear();
    c_audio_codec -> insertItem (i18n("auto"));
    for ( int i = 0; i < settings -> audioCodecCount(); i ++ )
      c_audio_codec -> insertItem (settings -> audioCodecName (i));
  }
  if ( settings -> audioDriverCount() )
  {
    c_audio_driver -> clear();
    c_audio_driver -> insertItem (i18n("auto"));
    for ( int i = 0; i < settings -> audioDriverCount(); i ++ )
      c_audio_driver -> insertItem (settings -> audioDriverName (i));
  }
}

void KPlayerSettingsAudio::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_audio_driver -> setCurrentItem (settings -> audioDriverIndex());
  driverChanged (c_audio_driver -> currentItem());
  c_audio_codec -> setCurrentItem (settings -> audioCodecIndex());
  codecChanged (c_audio_codec -> currentItem());
}

void KPlayerSettingsAudio::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setAudioDriver (listEntry (c_audio_driver -> currentText()));
  if ( c_audio_driver -> currentItem() > 0 )
  {
    settings -> setAudioDriverFallback (c_audio_driver_fallback -> isChecked());
    settings -> setAudioDevice (c_audio_device -> text());
  }
  settings -> setAudioCodec (listEntry (c_audio_codec -> currentText()));
  if ( c_audio_codec -> currentItem() > 0 )
    settings -> setAudioCodecFallback (c_audio_codec_fallback -> isChecked());
}

void KPlayerSettingsAudio::driverChanged (int index)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( index > 0 )
  {
    c_audio_driver_fallback -> setChecked (settings -> audioDriverFallback());
    c_audio_device -> setText (settings -> audioDevice());
  }
  else
  {
    c_audio_driver_fallback -> setChecked (false);
    c_audio_device -> setText ("");
  }
  c_audio_driver_fallback -> setEnabled (index > 0);
  c_audio_device -> setEnabled (index > 0);
}

void KPlayerSettingsAudio::codecChanged (int index)
{
  if ( index > 0 )
    c_audio_codec_fallback -> setChecked (kPlayerSettings() -> audioCodecFallback());
  else
    c_audio_codec_fallback -> setChecked (false);
  c_audio_codec_fallback -> setEnabled (index > 0);
}

KPlayerSettingsControls::KPlayerSettingsControls (QWidget* parent, const char* name)
  : KPlayerSettingsControlsPage (parent, name)
{
  load();
}

void KPlayerSettingsControls::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_preferred_slider_length -> setText (QString().setNum (settings -> preferredSliderLength()));
  c_minimum_slider_length -> setText (QString().setNum (settings -> minimumSliderLength()));
}

void KPlayerSettingsControls::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setPreferredSliderLength (labs (atol (c_preferred_slider_length -> text())));
  settings -> setMinimumSliderLength (labs (atol (c_minimum_slider_length -> text())));
}

KPlayerSettingsGeneral::KPlayerSettingsGeneral (QWidget* parent, const char* name)
  : KPlayerSettingsGeneralPage (parent, name)
{
  load();
}

void KPlayerSettingsGeneral::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_resize_automatically -> setChecked (settings -> resizeAutomatically());
  resizeAutomaticallyChanged (settings -> resizeAutomatically());
  c_recent_file_list_size -> setText (QString().setNum (settings -> recentFileListSize()));
  c_show_file_path -> setChecked (settings -> showFilePath());
  c_start_playing_immediately -> setChecked (settings -> startPlayingImmediately());
  c_show_open_dialog -> setChecked (settings -> showOpenDialog());
  c_disable_screen_saver -> setChecked (settings -> disableScreenSaver());
}

void KPlayerSettingsGeneral::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setResizeAutomatically (c_resize_automatically -> isChecked());
  if ( settings -> resizeAutomatically() )
    settings -> setMinimumInitialWidth (labs (atol (c_minimum_initial_width -> text())));
  settings -> setRecentFileListSize (labs (atol (c_recent_file_list_size -> text())));
  settings -> setShowFilePath (c_show_file_path -> isChecked());
  settings -> setStartPlayingImmediately (c_start_playing_immediately -> isChecked());
  settings -> setShowOpenDialog (c_show_open_dialog -> isChecked());
  settings -> setDisableScreenSaver (c_disable_screen_saver -> isChecked());
}

void KPlayerSettingsGeneral::resizeAutomaticallyChanged (bool resizeAutomaticallyChecked)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( resizeAutomaticallyChecked )
    c_minimum_initial_width -> setText (QString().setNum (settings -> minimumInitialWidth()));
  else
    c_minimum_initial_width -> setText ("");
  c_minimum_initial_width -> setEnabled (resizeAutomaticallyChecked);
  l_minimum_initial_width -> setEnabled (resizeAutomaticallyChecked);
}

KPlayerSettingsPlaylist::KPlayerSettingsPlaylist (QWidget* parent, const char* name)
  : KPlayerSettingsPlaylistPage (parent, name)
{
  load();
}

void KPlayerSettingsPlaylist::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_allow_duplicate_entries -> setChecked (settings -> allowDuplicateEntries());
  c_playlist_size_limit -> setText (QString().setNum (settings -> playlistSizeLimit()));
  c_cache_size_limit -> setText (QString().setNum (settings -> cacheSizeLimit()));
}

void KPlayerSettingsPlaylist::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setAllowDuplicateEntries (c_allow_duplicate_entries -> isChecked());
  settings -> setPlaylistSizeLimit (labs (atol (c_playlist_size_limit -> text())));
  settings -> setCacheSizeLimit (labs (atol (c_cache_size_limit -> text())));
}

KPlayerSettingsMessages::KPlayerSettingsMessages (QWidget* parent, const char* name)
  : KPlayerSettingsMessagesPage (parent, name)
{
  load();
}

void KPlayerSettingsMessages::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_show_on_error -> setChecked (settings -> showMessagesOnError());
  c_show_before -> setChecked (settings -> showMessagesBeforePlaying());
  c_show_after -> setChecked (settings -> showMessagesAfterPlaying());
  c_clear_before -> setChecked (settings -> clearMessagesBeforePlaying());
}

void KPlayerSettingsMessages::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setShowMessagesOnError (c_show_on_error -> isChecked());
  settings -> setShowMessagesBeforePlaying (c_show_before -> isChecked());
  settings -> setShowMessagesAfterPlaying (c_show_after -> isChecked());
  settings -> setClearMessagesBeforePlaying (c_clear_before -> isChecked());
}

KPlayerSettingsSubtitles::KPlayerSettingsSubtitles (QWidget* parent, const char* name)
  : KPlayerSettingsSubtitlesPage (parent, name)
{
  load();
}

void KPlayerSettingsSubtitles::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_subtitles_autoload -> setChecked (settings -> autoloadSubtitles());
  autoloadSubtitlesChanged (settings -> autoloadSubtitles());
}

void KPlayerSettingsSubtitles::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setAutoloadSubtitles (c_subtitles_autoload -> isChecked());
  if ( settings -> autoloadSubtitles() )
  {
    settings -> setAutoloadAqtSubtitles (c_subtitles_aqt -> isChecked());
    settings -> setAutoloadJssSubtitles (c_subtitles_jss -> isChecked());
    settings -> setAutoloadRtSubtitles (c_subtitles_rt -> isChecked());
    settings -> setAutoloadSmiSubtitles (c_subtitles_smi -> isChecked());
    settings -> setAutoloadSrtSubtitles (c_subtitles_srt -> isChecked());
    settings -> setAutoloadSsaSubtitles (c_subtitles_ssa -> isChecked());
    settings -> setAutoloadSubSubtitles (c_subtitles_sub -> isChecked());
    settings -> setAutoloadTxtSubtitles (c_subtitles_txt -> isChecked());
    settings -> setAutoloadUtfSubtitles (c_subtitles_utf -> isChecked());
    settings -> setAutoloadOtherSubtitles (c_subtitles_other -> isChecked());
    if ( settings -> autoloadOtherSubtitles() )
      settings -> setAutoloadExtensionList (c_subtitles_list -> text());
  }
}

void KPlayerSettingsSubtitles::otherSubtitlesChanged (bool otherSubtitlesChecked)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( otherSubtitlesChecked )
    c_subtitles_list -> setText (settings -> autoloadExtensionList());
  else
    c_subtitles_list -> setText ("");
  c_subtitles_list -> setEnabled (otherSubtitlesChecked);
  if ( otherSubtitlesChecked && sender() )
  {
    c_subtitles_list -> setFocus();
    c_subtitles_list -> selectAll();
  }
}

void KPlayerSettingsSubtitles::autoloadSubtitlesChanged (bool autoloadSubtitlesChecked)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( autoloadSubtitlesChecked )
  {
    c_subtitles_aqt -> setChecked (settings -> autoloadAqtSubtitles());
    c_subtitles_jss -> setChecked (settings -> autoloadJssSubtitles());
    c_subtitles_rt -> setChecked (settings -> autoloadRtSubtitles());
    c_subtitles_smi -> setChecked (settings -> autoloadSmiSubtitles());
    c_subtitles_srt -> setChecked (settings -> autoloadSrtSubtitles());
    c_subtitles_ssa -> setChecked (settings -> autoloadSsaSubtitles());
    c_subtitles_sub -> setChecked (settings -> autoloadSubSubtitles());
    c_subtitles_txt -> setChecked (settings -> autoloadTxtSubtitles());
    c_subtitles_utf -> setChecked (settings -> autoloadUtfSubtitles());
    c_subtitles_other -> setChecked (settings -> autoloadOtherSubtitles());
  }
  else
  {
    c_subtitles_aqt -> setChecked (false);
    c_subtitles_jss -> setChecked (false);
    c_subtitles_rt -> setChecked (false);
    c_subtitles_smi -> setChecked (false);
    c_subtitles_srt -> setChecked (false);
    c_subtitles_ssa -> setChecked (false);
    c_subtitles_sub -> setChecked (false);
    c_subtitles_txt -> setChecked (false);
    c_subtitles_utf -> setChecked (false);
    c_subtitles_other -> setChecked (false);
  }
  c_subtitles_aqt -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_jss -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_rt -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_smi -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_srt -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_ssa -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_sub -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_txt -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_utf -> setEnabled (autoloadSubtitlesChecked);
  c_subtitles_other -> setEnabled (autoloadSubtitlesChecked);
  otherSubtitlesChanged (c_subtitles_other -> isChecked());
}

KPlayerSettingsVideo::KPlayerSettingsVideo (QWidget* parent, const char* name)
  : KPlayerSettingsVideoPage (parent, name)
{
  loadLists();
  load();
  QApplication::connect (kPlayerSettings(), SIGNAL (refreshed()), this, SLOT (refresh()));
}

void KPlayerSettingsVideo::refresh (void)
{
  QString driver (listEntry (c_video_driver -> currentText()));
  QString codec (listEntry (c_video_codec -> currentText()));
  loadLists();
  KPlayerSettings* settings = kPlayerSettings();
  int index = settings -> videoDriverIndex (driver);
  kdDebug() << "Video Driver: " << driver << " => " << index << " / " << settings -> videoDriverCount() << "\n";
  c_video_driver -> setCurrentItem (index);
  if ( ! driver.isEmpty() && index == 0 )
    driverChanged (index);
  index = settings -> videoCodecIndex (codec);
  kdDebug() << "Video Codec: " << codec << " => " << index << " / " << settings -> videoCodecCount() << "\n";
  c_video_codec -> setCurrentItem (index);
  if ( ! codec.isEmpty() && index == 0 )
    codecChanged (index);
}

void KPlayerSettingsVideo::loadLists (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( settings -> videoCodecCount() )
  {
    c_video_codec -> clear();
    c_video_codec -> insertItem (i18n("auto"));
    for ( int i = 0; i < settings -> videoCodecCount(); i ++ )
      c_video_codec -> insertItem (settings -> videoCodecName (i));
  }
  if ( settings -> videoDriverCount() )
  {
    c_video_driver -> clear();
    c_video_driver -> insertItem (i18n("auto"));
    for ( int i = 0; i < settings -> videoDriverCount(); i ++ )
      c_video_driver -> insertItem (settings -> videoDriverName (i));
  }
}

void KPlayerSettingsVideo::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_video_driver -> setCurrentItem (settings -> videoDriverIndex());
  driverChanged (c_video_driver -> currentItem());
  c_video_codec -> setCurrentItem (settings -> videoCodecIndex());
  codecChanged (c_video_codec -> currentItem());
  c_video_scaler -> setCurrentItem (settings -> videoScaler());
  c_video_doublebuffering -> setChecked (settings -> videoDoublebuffering());
  c_video_direct_rendering -> setChecked (settings -> videoDirectRendering());
}

void KPlayerSettingsVideo::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setVideoDriver (listEntry (c_video_driver -> currentText()));
  if ( c_video_driver -> currentItem() > 0 )
  {
    settings -> setVideoDriverFallback (c_video_driver_fallback -> isChecked());
    settings -> setVideoDevice (c_video_device -> text());
  }
  settings -> setVideoCodec (listEntry (c_video_codec -> currentText()));
  if ( c_video_codec -> currentItem() > 0 )
    settings -> setVideoCodecFallback (c_video_codec_fallback -> isChecked());
  settings -> setVideoScaler (c_video_scaler -> currentItem());
  settings -> setVideoDoublebuffering (c_video_doublebuffering -> isChecked());
  settings -> setVideoDirectRendering (c_video_direct_rendering -> isChecked());
}

void KPlayerSettingsVideo::driverChanged (int index)
{
  KPlayerSettings* settings = kPlayerSettings();
  if ( index > 0 )
  {
    c_video_driver_fallback -> setChecked (settings -> videoDriverFallback());
    c_video_device -> setText (settings -> videoDevice());
  }
  else
  {
    c_video_driver_fallback -> setChecked (false);
    c_video_device -> setText ("");
  }
  c_video_driver_fallback -> setEnabled (index > 0);
  c_video_device -> setEnabled (index > 0);
}

void KPlayerSettingsVideo::codecChanged (int index)
{
  if ( index > 0 )
    c_video_codec_fallback -> setChecked (kPlayerSettings() -> videoCodecFallback());
  else
    c_video_codec_fallback -> setChecked (false);
  c_video_codec_fallback -> setEnabled (index > 0);
}

KPlayerSettingsProgress::KPlayerSettingsProgress (QWidget* parent, const char* name)
  : KPlayerSettingsProgressPage (parent, name)
{
  load();
}

void KPlayerSettingsProgress::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_progress_seek -> setText (QString().setNum (settings -> progressNormalSeek()));
  c_progress_seek_units -> setCurrentItem (settings -> progressNormalSeekUnits());
  c_progress_fast -> setText (QString().setNum (settings -> progressFastSeek()));
  c_progress_fast_units -> setCurrentItem (settings -> progressFastSeekUnits());
  c_progress_marks -> setText (QString().setNum (settings -> progressMarks()));
}

void KPlayerSettingsProgress::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setProgressNormalSeekUnits (c_progress_seek_units -> currentItem());
  settings -> setProgressNormalSeek (labs (atol (c_progress_seek -> text())));
  settings -> setProgressFastSeekUnits (c_progress_fast_units -> currentItem());
  settings -> setProgressFastSeek (labs (atol (c_progress_fast -> text())));
  settings -> setProgressMarks (labs (atol (c_progress_marks -> text())));
}

KPlayerSettingsVolume::KPlayerSettingsVolume (QWidget* parent, const char* name)
  : KPlayerSettingsVolumePage (parent, name)
{
  load();
}

void KPlayerSettingsVolume::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_volume_minimum -> setText (QString().setNum (settings -> volumeMinimum()));
  c_volume_maximum -> setText (QString().setNum (settings -> volumeMaximum()));
  c_volume_marks -> setText (QString().setNum (settings -> volumeMarks()));
  c_volume_step -> setText (QString().setNum (settings -> volumeStep()));
  c_volume_reset -> setChecked (settings -> volumeReset());
  resetChanged (settings -> volumeReset());
}

void KPlayerSettingsVolume::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setVolumeMinimumMaximum (atol (c_volume_minimum -> text()), labs (atol (c_volume_maximum -> text())));
  settings -> setVolumeMarks (labs (atol (c_volume_marks -> text())));
  settings -> setVolumeStep (labs (atol (c_volume_step -> text())));
  settings -> setVolumeReset (c_volume_reset -> isChecked());
  if ( settings -> volumeReset() )
  {
    settings -> setVolumeDefault (labs (atol (c_volume_default -> text())));
    settings -> setVolumeEvery (c_volume_every -> currentItem());
  }
}

void KPlayerSettingsVolume::resetChanged (bool resetChecked)
{
  if ( resetChecked )
  {
    KPlayerSettings* settings = kPlayerSettings();
    c_volume_default -> setText (QString().setNum (settings -> volumeDefault()));
    c_volume_every -> setCurrentItem (0);
    c_volume_every -> setCurrentText (i18n("file"));
    c_volume_every -> setCurrentItem (settings -> volumeEvery());
  }
  else
  {
    c_volume_default -> setText ("");
    c_volume_every -> setCurrentItem (0);
    c_volume_every -> setCurrentText ("");
  }
  c_volume_default -> setEnabled (resetChecked);
  c_volume_every -> setEnabled (resetChecked);
  if ( resetChecked && sender() )
  {
    c_volume_default -> setFocus();
    c_volume_default -> selectAll();
  }
}

KPlayerSettingsContrast::KPlayerSettingsContrast (QWidget* parent, const char* name)
  : KPlayerSettingsContrastPage (parent, name)
{
  load();
}

void KPlayerSettingsContrast::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_contrast_minimum -> setText (QString().setNum (settings -> contrastMinimum()));
  c_contrast_maximum -> setText (QString().setNum (settings -> contrastMaximum()));
  c_contrast_marks -> setText (QString().setNum (settings -> contrastMarks()));
  c_contrast_step -> setText (QString().setNum (settings -> contrastStep()));
  c_contrast_reset -> setChecked (settings -> contrastReset());
  resetChanged (settings -> contrastReset());
}

void KPlayerSettingsContrast::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setContrastMinimumMaximum (atol (c_contrast_minimum -> text()), atol (c_contrast_maximum -> text()));
  settings -> setContrastMarks (labs (atol (c_contrast_marks -> text())));
  settings -> setContrastStep (labs (atol (c_contrast_step -> text())));
  settings -> setContrastReset (c_contrast_reset -> isChecked());
  if ( settings -> contrastReset() )
  {
    settings -> setContrastDefault (atol (c_contrast_default -> text()));
    settings -> setContrastEvery (c_contrast_every -> currentItem());
  }
}

void KPlayerSettingsContrast::resetChanged (bool resetChecked)
{
  if ( resetChecked )
  {
    KPlayerSettings* settings = kPlayerSettings();
    c_contrast_default -> setText (QString().setNum (settings -> contrastDefault()));
    c_contrast_every -> setCurrentItem (0);
    c_contrast_every -> setCurrentText (i18n("file"));
    c_contrast_every -> setCurrentItem (settings -> contrastEvery());
  }
  else
  {
    c_contrast_default -> setText ("");
    c_contrast_every -> setCurrentItem (0);
    c_contrast_every -> setCurrentText ("");
  }
  c_contrast_default -> setEnabled (resetChecked);
  c_contrast_every -> setEnabled (resetChecked);
  if ( resetChecked && sender() )
  {
    c_contrast_default -> setFocus();
    c_contrast_default -> selectAll();
  }
}

KPlayerSettingsBrightness::KPlayerSettingsBrightness (QWidget* parent, const char* name)
  : KPlayerSettingsBrightnessPage (parent, name)
{
  load();
}

void KPlayerSettingsBrightness::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_brightness_minimum -> setText (QString().setNum (settings -> brightnessMinimum()));
  c_brightness_maximum -> setText (QString().setNum (settings -> brightnessMaximum()));
  c_brightness_marks -> setText (QString().setNum (settings -> brightnessMarks()));
  c_brightness_step -> setText (QString().setNum (settings -> brightnessStep()));
  c_brightness_reset -> setChecked (settings -> brightnessReset());
  resetChanged (settings -> brightnessReset());
}

void KPlayerSettingsBrightness::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setBrightnessMinimumMaximum (atol (c_brightness_minimum -> text()), atol (c_brightness_maximum -> text()));
  settings -> setBrightnessMarks (labs (atol (c_brightness_marks -> text())));
  settings -> setBrightnessStep (labs (atol (c_brightness_step -> text())));
  settings -> setBrightnessReset (c_brightness_reset -> isChecked());
  if ( settings -> brightnessReset() )
  {
    settings -> setBrightnessDefault (atol (c_brightness_default -> text()));
    settings -> setBrightnessEvery (c_brightness_every -> currentItem());
  }
}

void KPlayerSettingsBrightness::resetChanged (bool resetChecked)
{
  if ( resetChecked )
  {
    KPlayerSettings* settings = kPlayerSettings();
    c_brightness_default -> setText (QString().setNum (settings -> brightnessDefault()));
    c_brightness_every -> setCurrentItem (0);
    c_brightness_every -> setCurrentText (i18n("file"));
    c_brightness_every -> setCurrentItem (settings -> brightnessEvery());
  }
  else
  {
    c_brightness_default -> setText ("");
    c_brightness_every -> setCurrentItem (0);
    c_brightness_every -> setCurrentText ("");
  }
  c_brightness_default -> setEnabled (resetChecked);
  c_brightness_every -> setEnabled (resetChecked);
  if ( resetChecked && sender() )
  {
    c_brightness_default -> setFocus();
    c_brightness_default -> selectAll();
  }
}

KPlayerSettingsHue::KPlayerSettingsHue (QWidget* parent, const char* name)
  : KPlayerSettingsHuePage (parent, name)
{
  load();
}

void KPlayerSettingsHue::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_hue_minimum -> setText (QString().setNum (settings -> hueMinimum()));
  c_hue_maximum -> setText (QString().setNum (settings -> hueMaximum()));
  c_hue_marks -> setText (QString().setNum (settings -> hueMarks()));
  c_hue_step -> setText (QString().setNum (settings -> hueStep()));
  c_hue_reset -> setChecked (settings -> hueReset());
  resetChanged (settings -> hueReset());
}

void KPlayerSettingsHue::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setHueMinimumMaximum (atol (c_hue_minimum -> text()), atol (c_hue_maximum -> text()));
  settings -> setHueMarks (labs (atol (c_hue_marks -> text())));
  settings -> setHueStep (labs (atol (c_hue_step -> text())));
  settings -> setHueReset (c_hue_reset -> isChecked());
  if ( settings -> hueReset() )
  {
    settings -> setHueDefault (atol (c_hue_default -> text()));
    settings -> setHueEvery (c_hue_every -> currentItem());
  }
}

void KPlayerSettingsHue::resetChanged (bool resetChecked)
{
  if ( resetChecked )
  {
    KPlayerSettings* settings = kPlayerSettings();
    c_hue_default -> setText (QString().setNum (settings -> hueDefault()));
    c_hue_every -> setCurrentItem (0);
    c_hue_every -> setCurrentText (i18n("file"));
    c_hue_every -> setCurrentItem (settings -> hueEvery());
  }
  else
  {
    c_hue_default -> setText ("");
    c_hue_every -> setCurrentItem (0);
    c_hue_every -> setCurrentText ("");
  }
  c_hue_default -> setEnabled (resetChecked);
  c_hue_every -> setEnabled (resetChecked);
  if ( resetChecked && sender() )
  {
    c_hue_default -> setFocus();
    c_hue_default -> selectAll();
  }
}

KPlayerSettingsSaturation::KPlayerSettingsSaturation (QWidget* parent, const char* name)
  : KPlayerSettingsSaturationPage (parent, name)
{
  load();
}

void KPlayerSettingsSaturation::load (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  c_saturation_minimum -> setText (QString().setNum (settings -> saturationMinimum()));
  c_saturation_maximum -> setText (QString().setNum (settings -> saturationMaximum()));
  c_saturation_marks -> setText (QString().setNum (settings -> saturationMarks()));
  c_saturation_step -> setText (QString().setNum (settings -> saturationStep()));
  c_saturation_reset -> setChecked (settings -> saturationReset());
  resetChanged (settings -> saturationReset());
}

void KPlayerSettingsSaturation::save (void)
{
  KPlayerSettings* settings = kPlayerSettings();
  settings -> setSaturationMinimumMaximum (atol (c_saturation_minimum -> text()), atol (c_saturation_maximum -> text()));
  settings -> setSaturationMarks (labs (atol (c_saturation_marks -> text())));
  settings -> setSaturationStep (labs (atol (c_saturation_step -> text())));
  settings -> setSaturationReset (c_saturation_reset -> isChecked());
  if ( settings -> saturationReset() )
  {
    settings -> setSaturationDefault (atol (c_saturation_default -> text()));
    settings -> setSaturationEvery (c_saturation_every -> currentItem());
  }
}

void KPlayerSettingsSaturation::resetChanged (bool resetChecked)
{
  if ( resetChecked )
  {
    KPlayerSettings* settings = kPlayerSettings();
    c_saturation_default -> setText (QString().setNum (settings -> saturationDefault()));
    c_saturation_every -> setCurrentItem (0);
    c_saturation_every -> setCurrentText (i18n("file"));
    c_saturation_every -> setCurrentItem (settings -> saturationEvery());
  }
  else
  {
    c_saturation_default -> setText ("");
    c_saturation_every -> setCurrentItem (0);
    c_saturation_every -> setCurrentText ("");
  }
  c_saturation_default -> setEnabled (resetChecked);
  c_saturation_every -> setEnabled (resetChecked);
  if ( resetChecked && sender() )
  {
    c_saturation_default -> setFocus();
    c_saturation_default -> selectAll();
  }
}
