/***************************************************************************
                          kplayerwidget.cpp
                          -----------------
    begin                : Sun Dec 01 2002
    copyright            : (C) 2002-2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

//#include <kapplication.h>
//#include <kconfig.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <krecentdocument.h>
//#include <kurldrag.h>
#include <kurlrequesterdlg.h>

#include <kdebug.h>

#include "kplayersettings.h"
#include "kplayerwidget.h"
#include "kplayerwidget.moc"

void KPlayerX11MapWindow (uint id);
void KPlayerX11UnmapWindow (uint id);
void KPlayerX11ClearExposeWindow (uint id);
void KPlayerX11SendConfigureEvent (uint id, int w, int h);
void KPlayerSetX11EventFilter (void);
void KPlayerResetX11EventFilter (void);

static KPlayerWidget* CurrentKPlayerWidget = 0;

KPlayerFileDialog::KPlayerFileDialog (const QString& startDir, const QString& filter, QWidget* parent, const char* name)
  : KFileDialog (startDir, filter, parent, name, true)
{
  ops -> clearHistory();
}

KPlayerWidget::KPlayerWidget (QWidget *parent, const char *name)
  : QWidget (parent, name), m_aspect (0, 0), m_size (0, 0)
{
  m_original_aspect = false;
  m_resizing = false;
  m_process = new KPlayerProcess;
  m_process -> setWinId (winId());
  KPlayerSettings::setKPlayerProcess (m_process);
  connect (m_process, SIGNAL (stateChanged(KPlayerProcess::State)), this, SLOT (playerStateChanged(KPlayerProcess::State)));
  setFocusPolicy (QWidget::NoFocus);
  setEnabled (false);
  setEraseColor (QColor (0, 0, 0));
  CurrentKPlayerWidget = this;
  KPlayerSetX11EventFilter();
  //setAcceptDrops (true);
}

KPlayerWidget::~KPlayerWidget()
{
  if ( m_process )
    delete m_process;
  KPlayerResetX11EventFilter();
  if ( CurrentKPlayerWidget == this )
    CurrentKPlayerWidget = 0;
}

void KPlayerWidget::maintainOriginalAspect (bool original)
{
  setFixedAspect (0, 0);
  m_original_aspect = original; // && ! kPlayerProcess() -> size(). isEmpty();
  setVideoSize (size());
}

void KPlayerWidget::maintainCurrentAspect (bool current)
{
  if ( current && ! kPlayerProcess() -> size(). isEmpty() )
  {
    setFixedAspect (width(), height());
    setVideoSize (size());
  }
  else
    setFixedAspect (0, 0);
}

bool KPlayerWidget::getFixedAspect (int& horz, int& vert)
{
  horz = m_original_aspect ? kPlayerProcess() -> width() : m_aspect.width();
  vert = m_original_aspect ? kPlayerProcess() -> height() : m_aspect.height();
  return isFixedAspect();
}

void KPlayerWidget::setFixedAspect (int horz, int vert)
{
  if ( kPlayerProcess() -> size(). isEmpty() )
    horz = vert = 0;
  m_aspect.setWidth (horz);
  m_aspect.setHeight (vert);
  m_original_aspect = false;
  setVideoSize (size());
}

void KPlayerWidget::setVideoSize (const QSize& size, bool canGrow, bool horizontally)
{
  m_size = size.expandedTo (QSize (1, 1));
  adjustAspect (m_size, canGrow, horizontally);
  kdDebug() << "SVSize " << size.width() << "x" << size.height() << " => " << m_size.width()
    << "x" << m_size.height() << " " << canGrow << " " << horizontally << "\n";
}

void KPlayerWidget::adjustWidth (QSize& size) const
{
  if ( m_original_aspect )
    size.setWidth (size.height() * kPlayerProcess() -> width() / kPlayerProcess() -> height());
  else
    size.setWidth (size.height() * m_aspect.width() / m_aspect.height());
}

void KPlayerWidget::adjustHeight (QSize& size) const
{
  if ( m_original_aspect )
    size.setHeight (size.width() * kPlayerProcess() -> height() / kPlayerProcess() -> width());
  else
    size.setHeight (size.width() * m_aspect.height() / m_aspect.width());
}

void KPlayerWidget::adjustAspect (QSize& size, bool canGrow, bool horizontally) const
{
  if ( ! isFixedAspect() || size.isEmpty() )
    return;
  if ( ! canGrow )
    horizontally = size.width() * (m_original_aspect ? kPlayerProcess() -> height() : m_aspect.height())
      > size.height() * (m_original_aspect ? kPlayerProcess() -> width() : m_aspect.width());
  if ( horizontally )
    adjustWidth (size);
  else
    adjustHeight (size);
}

/*void KPlayerWidget::resize (int w, int h)
{
  if ( w == width() && h == height() )
    return;
  hide();
  kdDebug() << "Resize " << width() << "x" << height() << " => " << w << "x" << h << "\n";
  QWidget::resize (w, h);
  show();
}

void KPlayerWidget::setGeometry (int left, int top, int w, int h)
{
  if ( left != x() || top != y() || w != width() || h != height() )
    return;
  hide();
  kdDebug() << "SetGeo " << x() << "x" << y() << " " << width() << "x" << height()
    << " => " << left << "x" << top << " " << w << "x" << h << "\n";
  QWidget::setGeometry (left, top, w, h);
  show();
}*/

KURL::List KPlayerWidget::openFiles (void)
{
  static QString filter = i18n("*|All files\n*.avi *.AVI|AVI files\n*.mpg *.mpeg *.MPG *.MPEG|MPEG files\n*.ogg *.OGG|OGG files\n*.mp3 *.MP3|MP3 files");

  KConfig* config = kPlayerConfig();
  config -> setGroup ("Dialog Options");
  QString dir = config -> readEntry ("Open File Directory");
//Saving dialog position did not work: dlg.pos() returns wrong position in Qt 3.1.1
//int x = config -> readNumEntry ("Open File Left");
//int y = config -> readNumEntry ("Open File Top");
  int width = config -> readNumEntry ("Open File Width");
  int height = config -> readNumEntry ("Open File Height");
//kdDebug() << "Geometry " << x << "x" << y << " " << width << "x" << height << "\n";

  KPlayerFileDialog dlg (dir, filter, this, "filedialog");
  dlg.setOperationMode (KFileDialog::Opening);
  dlg.setMode (KFile::Files | KFile::LocalOnly | KFile::ExistingOnly);
  dlg.setCaption (i18n("Open"));

  if ( width > 0 && height > 0 )
    dlg.resize (width, height);
    //dlg.setGeometry (x, y, width, height);
    //dlg.move (x, y);
//kdDebug() << "         " << dlg.x() << "x" << dlg.y() << " " << dlg.width() << "x" << dlg.height() << "\n";

  dlg.exec();

  config -> writeEntry ("Open File Directory", dlg.directory());
//config -> writeEntry ("Open File Left", dlg.x());
//config -> writeEntry ("Open File Top", dlg.y());
  config -> writeEntry ("Open File Width", dlg.width());
  config -> writeEntry ("Open File Height", dlg.height());
//kdDebug() << "Geometry " << coord.x() << "x" << coord.y() << " " << dlg.width() << "x" << dlg.height() << "\n";

//KURL url;
//url.setPath (dlg.selectedFile());
//if ( ! url.isEmpty() )
//  kdDebug() << "Open '" << dlg.selectedFile() << "'\n";
//return url;

  return dlg.selectedURLs();
}

KURL KPlayerWidget::openUrl (void)
{
  KConfig* config = kPlayerConfig();
  config -> setGroup ("Dialog Options");
  QString dir = config -> readEntry ("Open URL Directory");
//int x = config -> readNumEntry ("Open URL Left");
//int y = config -> readNumEntry ("Open URL Top");
  int width = config -> readNumEntry ("Open URL Width");
  int height = config -> readNumEntry ("Open URL Height");

  KURLRequesterDlg dlg (dir, this, "filedialog", true);
  dlg.setCaption (i18n("Open URL"));

  if ( width > 0 && height > 0 )
    dlg.resize (width, height);
    //dlg.setGeometry (x, y, width, height);

  dlg.exec();

  KURL url (dlg.selectedURL());
  if ( ! url.isEmpty() && ! url.isMalformed() )
    KRecentDocument::add (url);

  if ( dlg.result() == QDialog::Accepted )
    config -> writeEntry ("Open URL Directory", url.url());
//config -> writeEntry ("Open URL Left", dlg.x());
//config -> writeEntry ("Open URL Top", dlg.y());
  config -> writeEntry ("Open URL Width", dlg.width());
  config -> writeEntry ("Open URL Height", dlg.height());

  return url;
}

KURL KPlayerWidget::openSubtitle (void)
{
  static QString filter = i18n("*|All files\n*.aqt *.AQT *.jss *.JSS *.rt *.RT *.smi *.SMI *.srt *.SRT *.ssa *.SSA *.sub *.SUB *.txt *.TXT *.utf *.UTF|All subtitle files\n*.aqt *.AQT|AQT files\n*.jss *.JSS|JSS files\n*.rt *.RT|RT files\n*.smi *.SMI|SMI files\n*.srt *.SRT|SRT files\n*.ssa *.SSA|SSA files\n*.sub *.SUB|SUB files\n*.txt *.TXT|TXT files\n*.utf *.UTF|UTF files");

  KConfig* config = kPlayerConfig();
  config -> setGroup ("Dialog Options");
  QString dir = config -> readEntry ("Open Subtitle Directory");
//int x = config -> readNumEntry ("Open Subtitle Left");
//int y = config -> readNumEntry ("Open Subtitle Top");
  int width = config -> readNumEntry ("Open Subtitle Width");
  int height = config -> readNumEntry ("Open Subtitle Height");

  KPlayerFileDialog dlg (dir, filter, this, "filedialog");
  dlg.setOperationMode (KFileDialog::Opening);
  dlg.setMode (KFile::File | KFile::LocalOnly | KFile::ExistingOnly);
  dlg.setCaption (i18n("Open Subtitles"));

  if ( width > 0 && height > 0 )
    dlg.resize (width, height);
    //dlg.setGeometry (x, y, width, height);

  dlg.exec();

  config -> writeEntry ("Open Subtitle Directory", dlg.directory());
//config -> writeEntry ("Open Subtitle Left", dlg.x());
//config -> writeEntry ("Open Subtitle Top", dlg.y());
  config -> writeEntry ("Open Subtitle Width", dlg.width());
  config -> writeEntry ("Open Subtitle Height", dlg.height());

  KURL url;
  url.setPath (dlg.selectedFile());
  if ( ! url.isEmpty() )
    kdDebug() << "Subtitle '" << dlg.selectedFile() << "'\n";
  return url;
}

KURL KPlayerWidget::openSubtitleUrl (void)
{
  KConfig* config = kPlayerConfig();
  config -> setGroup ("Dialog Options");
  QString dir = config -> readEntry ("Open Subtitle URL Directory");
//int x = config -> readNumEntry ("Open Subtitle URL Left");
//int y = config -> readNumEntry ("Open Subtitle URL Top");
  int width = config -> readNumEntry ("Open Subtitle URL Width");
  int height = config -> readNumEntry ("Open Subtitle URL Height");

  KURLRequesterDlg dlg (dir, this, "filedialog", true);
  dlg.setCaption (i18n("Open Subtitle URL"));

  if ( width > 0 && height > 0 )
    dlg.resize (width, height);
    //dlg.setGeometry (x, y, width, height);

  dlg.exec();

  KURL url (dlg.selectedURL());
  if ( ! url.isEmpty() && ! url.isMalformed() )
    KRecentDocument::add (url);

  if ( dlg.result() == QDialog::Accepted )
    config -> writeEntry ("Open Subtitle URL Directory", url.url());
//config -> writeEntry ("Open Subtitle URL Left", dlg.x());
//config -> writeEntry ("Open Subtitle URL Top", dlg.y());
  config -> writeEntry ("Open Subtitle URL Width", dlg.width());
  config -> writeEntry ("Open Subtitle URL Height", dlg.height());

  return url;
}

// WM_NORMAL_HINTS, XSizeHints, XSetWMNormalHints() ???

void KPlayerWidget::zoomIn (void)
{
  setVideoSize (size() + kPlayerProcess() -> size() / 2);
}

void KPlayerWidget::zoomOut (void)
{
  setVideoSize (size() - kPlayerProcess() -> size() / 2);
}

bool KPlayerWidget::isZoomFactor (int m, int d) const
{
  QSize size = kPlayerProcess() -> size() * m / d;
  adjustAspect (size, true);
  return size == m_size;
}

void KPlayerWidget::setZoomFactor (int m, int d)
{
  setVideoSize (kPlayerProcess() -> size() * m / d);
}

/*bool KPlayerWidget::event (QEvent* ev)
{
//kdDebug() << "KPlayerWidget event: type " << ev -> type()
//  << " spontaneous " << ev -> spontaneous() << "\n";
  return QWidget::event (ev);
}*/

void KPlayerWidget::closeEvent (QCloseEvent* event)
{
  m_process -> stop();
  QWidget::closeEvent (event);
}

void KPlayerWidgetResizeHandler (bool resizing)
{
  if ( CurrentKPlayerWidget )
    CurrentKPlayerWidget -> resizeHandler (resizing);
}

void KPlayerWidgetMapHandler (uint wid)
{
  if ( CurrentKPlayerWidget )
    CurrentKPlayerWidget -> mapHandler (wid);
}

void KPlayerWidgetUnmapHandler (uint wid)
{
  if ( CurrentKPlayerWidget )
    CurrentKPlayerWidget -> unmapHandler (wid);
}

void KPlayerWidget::resizeHandler (bool resizing)
{
  m_resizing = resizing;
  if ( ! resizing && isFixedAspect() )
    QApplication::postEvent (this, new QResizeEvent (size(), size()));
}

void KPlayerWidget::mapHandler (uint wid)
{
  if ( wid == winId() )
  {
    kdDebug() << "Mapping; process state " << kPlayerProcess() -> state() << "\n";
    if ( ! kPlayerProcess() -> is09Version() )
      KPlayerX11UnmapWindow (winId());
    show();
    if ( ! kPlayerProcess() -> is09Version() )
      KPlayerX11MapWindow (winId());
    /*QSize s (size());
    resize (s.width() - 1, s.height() - 1);
    QApplication::sendEvent (this, new QResizeEvent (size(), size()));
    resize (s.width(), s.height());
    QApplication::sendEvent (this, new QResizeEvent (size(), size()));*/
  }
}

void KPlayerWidget::unmapHandler (uint wid)
{
  if ( wid == winId() )
  {
    kdDebug() << "Unmapping " << wid << "; process state " << kPlayerProcess() -> state() << "\n";
    hide();
    KPlayerX11MapWindow (winId());
    if ( kPlayerProcess() -> is09Version() )
    {
      KPlayerX11ClearExposeWindow (winId());
      KPlayerX11SendConfigureEvent (winId(), width(), height());
    }
  }
}

/*bool KPlayerWidget::event (QEvent* ev)
{
  kdDebug() << "    Event " << ev -> type() << " " << ev -> spontaneous() << "\n";
  return QWidget::event (ev);
}*/

void KPlayerWidget::showEvent (QShowEvent* event)
{
  QWidget::showEvent (event);
  if ( kPlayerProcess() -> is09Version() )
    KPlayerX11SendConfigureEvent (winId(), width(), height());
}

void KPlayerWidget::resizeEvent (QResizeEvent* event)
{
  static bool recursion = false;
  kdDebug() << "Widget " << event -> oldSize().width() << "x" << event -> oldSize().height()
    << " => " << event -> size().width() << "x" << event -> size().height() << "\n";
  QWidget::resizeEvent (event);
  //if ( event -> oldSize() == event -> size() )
  //  return;
/*if ( ! kPlayerProcess() -> size().isEmpty() && ! recursion )
  {
    QSize size (size());
    setGeometry (0, 0, 100, 100);
    //resize (size.width() - 1, size.height() - 1);
    hide();
    resize (size);
    show();
  }*/
  if ( kPlayerProcess() -> is09Version() )
  {
    KPlayerX11UnmapWindow (winId());
    KPlayerX11MapWindow (winId());
  }
  if ( m_resizing || recursion )
    return;
  emit resized();
  recursion = true;
//if ( kPlayerProcess() -> state() == KPlayerProcess::Playing && ! kPlayerProcess() -> size().isEmpty() )
  if ( kPlayerProcess() -> is09Version() )
  {
    KPlayerX11UnmapWindow (winId());
    KPlayerX11MapWindow (winId());
    KPlayerX11ClearExposeWindow (winId());
    KPlayerX11SendConfigureEvent (winId(), event -> size().width(), event -> size().height());
  }
  kdDebug() << "Widget resize; process state " << kPlayerProcess() -> state() << "\n";
  recursion = false;
}

void KPlayerWidget::playerStateChanged (KPlayerProcess::State state)
{
  kdDebug() << "State change received by the widget: " << state << "\n";
  if ( kPlayerProcess() -> is09Version() && state == KPlayerProcess::Playing && ! kPlayerProcess() -> size().isEmpty() )
  {
    KPlayerX11MapWindow (winId());
    KPlayerX11ClearExposeWindow (winId());
  }
  KPlayerX11SendConfigureEvent (winId(), width(), height());
/*if ( state == KPlayerProcess::Playing && ! kPlayerProcess() -> size().isEmpty() )
  {
    QSize s (size());
    resize (s.width() - 1, s.height() - 1);
    QApplication::sendEvent (this, new QResizeEvent (size(), size()));
    resize (s.width(), s.height());
    QApplication::sendEvent (this, new QResizeEvent (size(), size()));
  }*/
//{
    //kdDebug() << "Unmapping and mapping\n";
    //QSize size (size());
    //resize (size.width() - 1, size.height() - 1);
    //resize (size.width(), size.height());
    /*KPlayerX11UnmapWindow (winId());
    KPlayerX11MapWindow (winId());
    KPlayerX11ClearExposeWindow (winId());
    KPlayerX11SendConfigureEvent (winId(), width(), height());*/
//}
    // hackery needed to ensure proper display
/*  hide();
    QSize size (size());
//  resize (100, 100);
    resize (size.width() - 1, size.height() - 1);
    resize (size.width(), size.height());
    show();
    //kdDebug() << "Widget size: " << width() << "x" << height() << "\n";
  }*/
}

/*void KPlayerWidget::dragEnterEvent (QDragEnterEvent *event)
{
  kdDebug() << "Drag enter event\n";
  //event -> accept (KURLDrag::canDecode (event));
}

void KPlayerWidget::dropEvent (QDropEvent* event)
{
  kdDebug() << "Drop event\n";
  KURL::List urls;
  if ( ! KURLDrag::decode (event, urls) )
    return;
  openUrls (urls);
}*/

/*void KPlayerWidget::contextMenuEvent (QContextMenuEvent* event)
{
  kdDebug() << "KPlayerWorkspace context menu\n";
  emit contextMenu (event);
}*/

KPlayerWorkspace::KPlayerWorkspace (QWidget* parent, const char* name)
  : QWidget (parent, name)
{
  m_widget = new KPlayerWidget (this);
  m_free_resize_mode = true;
  setEraseColor (QColor (0, 0, 0));
  setFocusPolicy (QWidget::NoFocus);
  //setFocusPolicy (QWidget::StrongFocus);
  //m_widget -> setFocusProxy (this);
  //setEnabled (false);
  //setMinimumSize (39, 39);
  //setAcceptDrops (true);
}

void KPlayerWorkspace::setVideoSize (void)
{
  m_widget -> setVideoSize (size(), false);
  m_widget -> setGeometry ((width() - m_widget -> videoSize().width()) / 2,
    (height() - m_widget -> videoSize().height()) / 2,
    m_widget -> videoSize().width(), m_widget -> videoSize().height());
}

void KPlayerWorkspace::resizeEvent (QResizeEvent* event)
{
  kdDebug() << "WSpace " << event -> oldSize(). width() << "x" << event -> oldSize(). height()
    << " => " << event -> size(). width() << "x" << event -> size(). height() << "\n";
  QWidget::resizeEvent (event);
  if ( m_free_resize_mode )
    m_widget -> setGeometry (0, 0, width(), height());
  else
    setVideoSize();
}

/*void KPlayerWorkspace::dragEnterEvent (QDragEnterEvent *event)
{
  kdDebug() << "Workspace drag enter event\n";
  //event -> accept (KURLDrag::canDecode (event));
}

void KPlayerWorkspace::dropEvent (QDropEvent* event)
{
  kdDebug() << "Workspace drop event\n";
  KURL::List urls;
  if ( ! KURLDrag::decode (event, urls) )
    return;
  openUrls (urls);
}*/

void KPlayerWorkspace::contextMenuEvent (QContextMenuEvent* event)
{
  kdDebug() << "KPlayerWorkspace context menu\n";
  emit contextMenu (event);
}
