/***************************************************************************
                          x11.cpp
                          -------
    begin                : Wed Feb 26 2003
    copyright            : (C) 2003 by kiriuja
    email                : kplayer-dev@en-directo.net
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <X11/Xlib.h>
#include <kdebug.h>

//#define DEBUG_KPLAYER_X11

/*bool KPlayerX11TestGrab (Display* display, int winid)
{
  if ( int status = XGrabPointer (display, winid, False,
        (uint)(ButtonPressMask | ButtonReleaseMask | PointerMotionMask | EnterWindowMask | LeaveWindowMask),
        GrabModeAsync, GrabModeAsync, None, None, CurrentTime) )
  {
    const char *s =
      status == GrabNotViewable ? "GrabNotViewable" :
      status == AlreadyGrabbed  ? "AlreadyGrabbed" :
      status == GrabFrozen      ? "GrabFrozen" :
      status == GrabInvalidTime ? "GrabInvalidTime" : "Unknown";
    kdDebug() << "Grab failed, status: " << s << "\n";
    return false;
  }
  kdDebug() << "Grab succeeded\n";
  XUngrabPointer (display, CurrentTime);
  return true;
}*/

extern void KPlayerWidgetResizeHandler (bool);
extern void KPlayerWidgetMapHandler (uint);
extern void KPlayerWidgetUnmapHandler (uint);

typedef int (*QX11EventFilter) (XEvent*);

static QX11EventFilter PreviousX11EventFilter = 0;

#ifdef DEBUG_KPLAYER_X11
const char* KPlayerX11EventTypeNames [LASTEvent] = {
  "EVENT0",
  "EVENT1",
  "KeyPress",
  "KeyRelease",
  "ButtonPress",
  "ButtonRelease",
  "MotionNotify",
  "EnterNotify",
  "LeaveNotify",
  "FocusIn",
  "FocusOut",
  "KeymapNotify",
  "Expose",
  "GraphicsExpose",
  "NoExpose",
  "VisibilityNotify",
  "CreateNotify",
  "DestroyNotify",
  "UnmapNotify",
  "MapNotify",
  "MapRequest",
  "ReparentNotify",
  "ConfigureNotify",
  "ConfigureRequest",
  "GravityNotify",
  "ResizeRequest",
  "CirculateNotify",
  "CirculateRequest",
  "PropertyNotify",
  "SelectionClear",
  "SelectionRequest",
  "SelectionNotify",
  "ColormapNotify",
  "ClientMessage",
  "MappingNotify"
};

const char* KPlayerX11EventModeNames [4] = {
  "NotifyNormal",
  "NotifyGrab",
  "NotifyUngrab",
  "NotifyWhileGrabbed"
};

const char* KPlayerX11EventDetailNames [8] = {
  "NotifyAncestor",
  "NotifyVirtual",
  "NotifyInferior",
  "NotifyNonlinear",
  "NotifyNonlinearVirtual",
  "NotifyPointer",
  "NotifyPointerRoot",
  "NotifyDetailNone"
};

#endif

int KPlayerX11EventFilter (XEvent* event)
{
#ifdef DEBUG_KPLAYER_X11
  XAnyEvent* anyev = (XAnyEvent*) event;
  kdDebug() << "X11 event " << KPlayerX11EventTypeNames [event -> type] << " " << anyev -> window
    << " " << anyev -> send_event << " " << anyev -> serial << "\n";
#endif
  if ( event -> type == FocusIn || event -> type == FocusOut )
  {
    XFocusChangeEvent* ev = (XFocusChangeEvent*) event;
#ifdef DEBUG_KPLAYER_X11
    kdDebug() << "  Focus mode " << KPlayerX11EventModeNames [ev -> mode]
      << " detail " << KPlayerX11EventDetailNames [ev -> detail] << "\n";
#endif
    if ( ev -> detail == NotifyAncestor && (event -> type == FocusIn && ev -> mode == NotifyUngrab
        || event -> type == FocusOut && ev -> mode == NotifyGrab) )
    {
      kdDebug() << "Calling KPlayerWidgetResizeHandler (" << (ev -> mode == NotifyGrab) << ")\n";
      KPlayerWidgetResizeHandler (ev -> mode == NotifyGrab);
    }
  }
  else if ( event -> type == MapRequest )
  {
    XMapRequestEvent* ev = (XMapRequestEvent*) event;
#ifdef DEBUG_KPLAYER_X11
    kdDebug() << "  MapRequest " << ev -> window << "\n";
#endif
    KPlayerWidgetMapHandler (ev -> window);
  }
  else if ( event -> type == UnmapNotify )
  {
    XUnmapEvent* ev = (XUnmapEvent*) event;
#ifdef DEBUG_KPLAYER_X11
    kdDebug() << "  Unmap " << ev -> window << " " << ev -> from_configure << "\n";
#endif
    KPlayerWidgetUnmapHandler (ev -> window);
  }
#ifdef DEBUG_KPLAYER_X11
  else if ( event -> type == MapNotify )
  {
    XMapEvent* ev = (XMapEvent*) event;
    kdDebug() << "  Map " << ev -> window << " " << ev -> override_redirect << "\n";
  }
  else if ( event -> type == PropertyNotify )
  {
    XPropertyEvent* ev = (XPropertyEvent*) event;
    kdDebug() << "  Property " << ev -> atom << " " << ev -> time << " " << ev -> state << "\n";
  }
  else if ( event -> type == ConfigureNotify )
  {
    XConfigureEvent* ev = (XConfigureEvent*) event;
    kdDebug() << "  Configure " << ev -> window << " " << ev -> x << "x" << ev -> y
      << " " << ev -> width << "x" << ev -> height << " " << ev -> border_width
      << " " << ev -> above << " " << ev -> override_redirect << "\n";
  }
#endif
  if ( PreviousX11EventFilter )
    return PreviousX11EventFilter (event);
  return 0;
}

Display* qt_xdisplay (void);

void KPlayerX11MapWindow (uint id)
{
  XMapWindow (qt_xdisplay(), id);
  XFlush (qt_xdisplay());
}

void KPlayerX11UnmapWindow (uint id)
{
  XUnmapWindow (qt_xdisplay(), id);
  XFlush (qt_xdisplay());
}

void KPlayerX11ClearExposeWindow (uint id)
{
  XClearArea (qt_xdisplay(), id, 0, 0, 0, 0, True);
  XFlush (qt_xdisplay());
}

void KPlayerX11SendConfigureEvent (uint id, int w, int h)
{
  XConfigureEvent event = { ConfigureNotify, 0, True, qt_xdisplay(), id, id, 0, 0, w, h, 0, None, False };
  XSendEvent (qt_xdisplay(), event.event, True, StructureNotifyMask, (XEvent*) &event);
  XFlush (qt_xdisplay());
}

extern QX11EventFilter qt_set_x11_event_filter (QX11EventFilter);

static int KPlayerX11EventFilterCount = 0;

void KPlayerSetX11EventFilter (void)
{
  if ( ! KPlayerX11EventFilterCount ++ )
    PreviousX11EventFilter = qt_set_x11_event_filter (KPlayerX11EventFilter);
}

void KPlayerResetX11EventFilter (void)
{
  if ( -- KPlayerX11EventFilterCount )
    return;
  qt_set_x11_event_filter (PreviousX11EventFilter);
  PreviousX11EventFilter = 0;
}
